@@ -54,9 +54,18 @@
 #include "image/bmp.h"
 #include "image/ico.h"
 #endif
+#ifdef WITH_NS_SVG
+#include "image/svg.h"
+#endif
+#ifdef WITH_RSVG
+#include "image/rsvg.h"
+#endif
 #ifdef WITH_SPRITE
 #include "riscos/sprite.h"
 #endif
+#ifdef WITH_NSSPRITE
+#include "image/nssprite.h"
+#endif
 #ifdef WITH_DRAW
 #include "riscos/draw.h"
 #endif
@@ -66,6 +75,9 @@
 #ifdef WITH_ARTWORKS
 #include "riscos/artworks.h"
 #endif
+#ifdef WITH_PNG
+#include "image/png.h"
+#endif
 #include "utils/log.h"
 #include "utils/messages.h"
 #include "utils/talloc.h"
@@ -125,6 +137,7 @@ static const struct mime_entry mime_map[] = {
 #endif
 #ifdef WITH_JPEG
 	{"image/jpeg", CONTENT_JPEG},
+	{"image/jpg", CONTENT_JPEG},
 #endif
 #ifdef WITH_MNG
 	{"image/mng", CONTENT_MNG},
@@ -135,9 +148,13 @@ static const struct mime_entry mime_map[] = {
 #ifdef WITH_JPEG
 	{"image/pjpeg", CONTENT_JPEG},
 #endif
-#ifdef WITH_MNG
+#if defined(WITH_MNG) || defined(WITH_PNG)
 	{"image/png", CONTENT_PNG},
 #endif
+#if defined(WITH_NS_SVG) || defined (WITH_RSVG)
+	{"image/svg", CONTENT_SVG},
+	{"image/svg+xml", CONTENT_SVG},
+#endif
 #ifdef WITH_ARTWORKS
 	{"image/x-artworks", CONTENT_ARTWORKS},
 #endif
@@ -158,7 +175,7 @@ static const struct mime_entry mime_map[] = {
 #ifdef WITH_BMP
 	{"image/x-ms-bmp", CONTENT_BMP},
 #endif
-#ifdef WITH_SPRITE
+#if defined(WITH_SPRITE) || defined(WITH_NSSPRITE)
 	{"image/x-riscos-sprite", CONTENT_SPRITE},
 #endif
 #ifdef WITH_BMP
@@ -176,7 +193,7 @@ static const struct mime_entry mime_map[] = {
 };
 #define MIME_MAP_COUNT (sizeof(mime_map) / sizeof(mime_map[0]))
 
-const char *content_type_name[] = {
+const char * const content_type_name[] = {
 	"HTML",
 	"TEXTPLAIN",
 	"CSS",
@@ -190,12 +207,14 @@ const char *content_type_name[] = {
 	"BMP",
 	"ICO",
 #endif
-#ifdef WITH_MNG
+#if defined(WITH_MNG) || defined(WITH_PNG)
 	"PNG",
+#endif
+#ifdef WITH_MNG
 	"JNG",
 	"MNG",
 #endif
-#ifdef WITH_SPRITE
+#if defined(WITH_SPRITE) || defined(WITH_NSSPRITE)
 	"SPRITE",
 #endif
 #ifdef WITH_DRAW
@@ -210,12 +229,15 @@ const char *content_type_name[] = {
 #endif
 #ifdef WITH_ARTWORKS
 	"ARTWORKS",
+#endif
+#if defined(WITH_NS_SVG) || defined(WITH_RSVG)
+	"SVG",
 #endif
 	"OTHER",
 	"UNKNOWN"
 };
 
-const char *content_status_name[] = {
+const char * const content_status_name[] = {
 	"TYPE_UNKNOWN",
 	"LOADING",
 	"READY",
@@ -234,11 +256,11 @@ struct handler_entry {
 	bool (*redraw)(struct content *c, int x, int y,
 			int width, int height,
 			int clip_x0, int clip_y0, int clip_x1, int clip_y1,
-			float scale, unsigned long background_colour);
+			float scale, colour background_colour);
 	bool (*redraw_tiled)(struct content *c, int x, int y,
 			int width, int height,
 			int clip_x0, int clip_y0, int clip_x1, int clip_y1,
-			float scale, unsigned long background_colour,
+			float scale, colour background_colour,
 			bool repeat_x, bool repeat_y);
 	void (*open)(struct content *c, struct browser_window *bw,
 			struct content *page, unsigned int index,
@@ -273,10 +295,19 @@ static const struct handler_entry handler_map[] = {
 	{nsico_create, 0, nsico_convert, 0, nsico_destroy, 0,
 			nsico_redraw, nsico_redraw_tiled, 0, 0, false},
 #endif
+
+#ifdef WITH_PNG
+	{nspng_create, nspng_process_data, nspng_convert,
+		0, nspng_destroy, 0, nspng_redraw, nspng_redraw_tiled,
+                0, 0, false},
+#else
 #ifdef WITH_MNG
 	{nsmng_create, nsmng_process_data, nsmng_convert,
 		0, nsmng_destroy, 0, nsmng_redraw, nsmng_redraw_tiled,
-		0, 0, false},
+                0, 0, false},
+#endif
+#endif
+#ifdef WITH_MNG
 	{nsmng_create, nsmng_process_data, nsmng_convert,
 		0, nsmng_destroy, 0, nsmng_redraw, nsmng_redraw_tiled,
 		0, 0, false},
@@ -288,6 +319,10 @@ static const struct handler_entry handler_map[] = {
 	{0, 0, sprite_convert,
 		0, sprite_destroy, 0, sprite_redraw, 0, 0, 0, false},
 #endif
+#ifdef WITH_NSSPRITE
+	{0, 0, nssprite_convert,
+		0, nssprite_destroy, 0, nssprite_redraw, 0, 0, 0, false},
+#endif
 #ifdef WITH_DRAW
 	{0, 0, draw_convert,
 		0, draw_destroy, 0, draw_redraw, 0, 0, 0, false},
@@ -306,6 +341,14 @@ static const struct handler_entry handler_map[] = {
 #ifdef WITH_ARTWORKS
 	{0, 0, artworks_convert,
 		0, artworks_destroy, 0, artworks_redraw, 0, 0, 0, false},
+#endif
+#ifdef WITH_NS_SVG
+	{svg_create, 0, svg_convert,
+		0, svg_destroy, 0, svg_redraw, 0, 0, 0, false},
+#endif
+#ifdef WITH_RSVG
+	{rsvg_create, rsvg_process_data, rsvg_convert,
+		0, rsvg_destroy, 0, rsvg_redraw, 0, 0, 0, false},
 #endif
 	{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, false}
 };
@@ -370,11 +413,6 @@ struct content * content_create(const char *url)
 		talloc_free(c);
 		return 0;
 	}
-	c->cache_data = talloc(c, struct cache_data);
-	if (!c->cache_data) {
-		talloc_free(c);
-		return 0;
-	}
 	talloc_set_name_const(c, c->url);
 	c->type = CONTENT_UNKNOWN;
 	c->mime_type = 0;
@@ -403,16 +441,17 @@ struct content * content_create(const char *url)
 	c->http_code = 0;
 	c->no_error_pages = false;
 	c->download = false;
+	c->redirect_count = 0;
 	c->error_count = 0;
-	c->cache_data->req_time = 0;
-	c->cache_data->res_time = 0;
-	c->cache_data->date = 0;
-	c->cache_data->expires = 0;
-	c->cache_data->age = INVALID_AGE;
-	c->cache_data->max_age = INVALID_AGE;
-	c->cache_data->no_cache = false;
-	c->cache_data->etag = 0;
-	c->cache_data->last_modified = 0;
+	c->cache_data.req_time = 0;
+	c->cache_data.res_time = 0;
+	c->cache_data.date = 0;
+	c->cache_data.expires = 0;
+	c->cache_data.age = INVALID_AGE;
+	c->cache_data.max_age = INVALID_AGE;
+	c->cache_data.no_cache = false;
+	c->cache_data.etag = 0;
+	c->cache_data.last_modified = 0;
 
 	content_set_status(c, messages_get("Loading"));
 
@@ -586,6 +625,7 @@ bool content_set_type(struct content *c, content_type type,
 				return false;
 			}
 			content_remove_user(c, callback, p1, p2);
+			msg_data.new_url = NULL;
 			content_broadcast(clone, CONTENT_MSG_NEWPTR, msg_data);
 			fetchcache_go(clone, referer,
 					callback, p1, p2,
@@ -848,14 +888,15 @@ void content_clean(void)
 	next = 0;
 	for (c = content_list; c; c = c->next) {
 		next = c;
-		size += c->size + talloc_total_size(c);
+		c->talloc_size = talloc_total_size(c);
+		size += c->size + c->talloc_size;
 	}
 	for (c = next; c && (unsigned int) option_memory_cache_size < size;
 			c = prev) {
 		prev = c->prev;
 		if (c->user_list->next)
 			continue;
-		size -= c->size + talloc_total_size(c);
+		size -= c->size + c->talloc_size;
 		content_destroy(c);
 	}
 }
@@ -908,6 +949,13 @@ void content_reset(struct content *c)
 	c->size = 0;
 	talloc_free(c->mime_type);
 	c->mime_type = 0;
+	talloc_free(c->refresh);
+	c->refresh = 0;
+	talloc_free(c->title);
+	c->title = 0;
+	talloc_free(c->source_data);
+	c->source_data = 0;
+	c->source_size = c->source_allocated = 0;
 }
 
 
@@ -947,12 +995,31 @@ void content_quit(void)
  * Display content on screen.
  *
  * Calls the redraw function for the content, if it exists.
+ *
+ * \param  c		     content
+ * \param  x		     coordinate for top-left of redraw
+ * \param  y		     coordinate for top-left of redraw
+ * \param  width	     available width (not used for HTML redraw)
+ * \param  height	     available height (not used for HTML redraw)
+ * \param  clip_x0	     clip rectangle left
+ * \param  clip_y0	     clip rectangle top
+ * \param  clip_x1	     clip rectangle right
+ * \param  clip_y1	     clip rectangle bottom
+ * \param  scale	     scale for redraw
+ * \param  background_colour the background colour
+ * \return true if successful, false otherwise
+ *
+ * x, y and clip_* are coordinates from the top left of the canvas area.
+ *
+ * The top left corner of the clip rectangle is (clip_x0, clip_y0) and
+ * the bottom right corner of the clip rectangle is (clip_x1, clip_y1).
+ * Units for x, y and clip_* are pixels.
  */
 
 bool content_redraw(struct content *c, int x, int y,
 		int width, int height,
 		int clip_x0, int clip_y0, int clip_x1, int clip_y1,
-		float scale, unsigned long background_colour)
+		float scale, colour background_colour)
 {
 	assert(c != 0);
 //	LOG(("%p %s", c, c->url));
@@ -977,7 +1044,7 @@ bool content_redraw(struct content *c, int x, int y,
 bool content_redraw_tiled(struct content *c, int x, int y,
 		int width, int height,
 		int clip_x0, int clip_y0, int clip_x1, int clip_y1,
-		float scale, unsigned long background_colour,
+		float scale, colour background_colour,
 		bool repeat_x, bool repeat_y)
 {
 	int x0, y0, x1, y1;
