@@ -31,6 +31,8 @@
 #include <sys/types.h>
 #include <regex.h>
 #include <time.h>
+#include <unistd.h>
+#include <curl/curl.h>  /* for curl_getdate() */
 #include "utils/config.h"
 #include "content/content.h"
 #include "content/fetchcache.h"
@@ -47,10 +49,15 @@ static regex_t re_content_type;
 static void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 		unsigned long size);
 static char *fetchcache_parse_type(const char *s, char **params[]);
+static void fetchcache_parse_header(struct content *c, const char *data,
+		size_t size);
 static void fetchcache_error_page(struct content *c, const char *error);
-static void fetchcache_cache_update(struct content *c,
+static void fetchcache_cache_update(struct content *c);
+static void fetchcache_cache_clone(struct content *c,
 		const struct cache_data *data);
 static void fetchcache_notmodified(struct content *c, const void *data);
+static void fetchcache_redirect(struct content *c, const void *data,
+		unsigned long size);
 
 
 /**
@@ -97,22 +104,28 @@ struct content * fetchcache(const char *url,
 	if (strncasecmp(url, "file:///", 8) &&
 			strncasecmp(url, "file:/", 6) == 0) {
 		/* Manipulate file URLs into correct format */
-		if (strncasecmp(url, "file://", 7) == 0) {
+		int len = strlen(url) + 1;
+
+		if (strncasecmp(url, "file://", SLEN("file://")) == 0) {
 			/* file://path */
-			url1 = malloc(7 + strlen(url));
+			url1 = malloc(len + 1 /* + '/' */);
 			if (!url1)
 				return NULL;
 
-			strcpy(url1, "file://");
-			strcat(url1 + 7, url + 6);
+			memcpy(url1, "file:///", SLEN("file:///"));
+			memcpy(url1 + SLEN("file:///"), 
+				url + SLEN("file://"),
+				len - SLEN("file://"));
 		} else {
 			/* file:/... */
-			url1 = malloc(7 + strlen(url));
+			url1 = malloc(len + 2 /* + "//" */);
 			if (!url1)
 				return NULL;
 
-			strcpy(url1, "file://");
-			strcat(url1 + 7, url + 5);
+			memcpy(url1, "file:///", SLEN("file:///"));
+			memcpy(url1 + SLEN("file:///"), 
+				url + SLEN("file:/"),
+				len - SLEN("file:/"));
 		}
 	} else {
 		/* simply duplicate the URL */
@@ -131,7 +144,7 @@ struct content * fetchcache(const char *url,
 
 	if (!post_urlenc && !post_multipart && !download && !query) {
 		if ((c = content_get(url1)) != NULL) {
-			struct cache_data *cd = c->cache_data;
+			struct cache_data *cd = &c->cache_data;
 			int current_age, freshness_lifetime;
 
 			/* Calculate staleness of cached content as per
@@ -166,8 +179,8 @@ struct content * fetchcache(const char *url,
 			if (cd->last_modified)
 				date = cd->last_modified;
 			else
-				date = c->cache_data->date;
-			etag = c->cache_data->etag;
+				date = c->cache_data.date;
+			etag = c->cache_data.etag;
 		}
 	}
 
@@ -178,10 +191,10 @@ struct content * fetchcache(const char *url,
 
 	/* Fill in cache validation fields (if present) */
 	if (date)
-		c->cache_data->date = date;
+		c->cache_data.date = date;
 	if (etag) {
-		c->cache_data->etag = talloc_strdup(c, etag);
-		if (!c->cache_data->etag)
+		c->cache_data.etag = talloc_strdup(c, etag);
+		if (!c->cache_data.etag)
 			return NULL;
 	}
 
@@ -275,10 +288,19 @@ void fetchcache_go(struct content *content, const char *referer,
 		/* brand new content: start fetch */
 		char **headers;
 		int i = 0;
-		char *etag = content->cache_data->etag;
-		time_t date = content->cache_data->date;
-		content->cache_data->etag = 0;
-		content->cache_data->date = 0;
+		char *etag = content->cache_data.etag;
+		time_t date = content->cache_data.date;
+
+		content->cache_data.req_time = time(NULL);
+		content->cache_data.res_time = 0;
+		content->cache_data.date = 0;
+		content->cache_data.expires = 0;
+		content->cache_data.age = INVALID_AGE;
+		content->cache_data.max_age = INVALID_AGE;
+		content->cache_data.no_cache = false;
+		content->cache_data.etag = 0;
+		content->cache_data.last_modified = 0;
+
 		headers = malloc(3 * sizeof(char *));
 		if (!headers) {
 			content->status = CONTENT_STATUS_ERROR;
@@ -288,7 +310,9 @@ void fetchcache_go(struct content *content, const char *referer,
 			return;
 		}
 		if (etag) {
-			headers[i] = malloc(15 + strlen(etag) + 1);
+			int len = SLEN("If-None-Match: ") + strlen(etag) + 1;
+
+			headers[i] = malloc(len);
 			if (!headers[i]) {
 				free(headers);
 				content->status = CONTENT_STATUS_ERROR;
@@ -297,11 +321,14 @@ void fetchcache_go(struct content *content, const char *referer,
 						msg_data);
 				return;
 			}
-			sprintf(headers[i++], "If-None-Match: %s", etag);
+			snprintf(headers[i++], len, "If-None-Match: %s", etag);
 			talloc_free(etag);
 		}
 		if (date) {
-			headers[i] = malloc(19 + 29 + 1);
+			/* Maximum length of an RFC 1123 date is 29 bytes */
+			int len = SLEN("If-Modified-Since: ") + 29 + 1;
+
+			headers[i] = malloc(len);
 			if (!headers[i]) {
 				while (--i >= 0) {
 					free(headers[i]);
@@ -313,7 +340,7 @@ void fetchcache_go(struct content *content, const char *referer,
 						msg_data);
 				return;
 			}
-			sprintf(headers[i++], "If-Modified-Since: %s",
+			snprintf(headers[i++], len, "If-Modified-Since: %s",
 					rfc1123_date(date));
 		}
 		headers[i] = 0;
@@ -380,11 +407,10 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 	bool res;
 	struct content *c = p;
 	content_type type;
-	char *mime_type, *url;
+	char *mime_type;
 	char **params;
 	unsigned int i;
 	union content_msg_data msg_data;
-	url_func_result result;
 
 	switch (msg) {
 		case FETCH_TYPE:
@@ -402,7 +428,7 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 			type = content_lookup(mime_type);
 			res = content_set_type(c,
 					c->download ? CONTENT_OTHER : type,
-					mime_type, params);
+					mime_type, (const char **) params);
 			free(mime_type);
 			for (i = 0; params[i]; i++)
 				free(params[i]);
@@ -411,6 +437,25 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 				fetch_abort(c->fetch);
 				c->fetch = 0;
 			}
+
+			if (c->cache_data.date || c->cache_data.etag) {
+				/* We've just made a conditional request 
+				 * that returned with something other 
+				 * than 304. Therefore, there's a stale 
+				 * content floating around in the cache.
+				 * Hunt it down and mark it as stale, so 
+				 * it'll get cleaned when unused. We 
+				 * assume it's either READY or DONE -- 
+				 * anything else is of marginal staleness
+				 * (or in error, which will cause it to 
+				 * be flushed from the cache, anyway)
+				 */
+				struct content *stale_content = 
+						content_get_ready(c->url);
+
+				if (stale_content)
+					stale_content->fresh = false;
+			}
 			break;
 
 		case FETCH_PROGRESS:
@@ -425,6 +470,10 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 			content_broadcast(c, CONTENT_MSG_STATUS, msg_data);
 			break;
 
+		case FETCH_HEADER:
+			fetchcache_parse_header(c, data, size);
+			break;
+
 		case FETCH_DATA:
 			if (!content_process_data(c, data, size)) {
 				fetch_abort(c->fetch);
@@ -433,8 +482,7 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 			break;
 
 		case FETCH_FINISHED:
-			fetchcache_cache_update(c,
-					(const struct cache_data *)data);
+			fetchcache_cache_update(c);
 			c->fetch = 0;
 			content_set_status(c, messages_get("Converting"),
 					c->source_size);
@@ -457,44 +505,13 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 			break;
 
 		case FETCH_REDIRECT:
-			c->fetch = 0;
-			/* redirect URLs must be absolute by HTTP/1.1, but many sites send
-			 * relative ones: treat them as relative to requested URL */
-			result = url_join(data, c->url, &url);
-			/* set the status to ERROR so that the content is
-			 * destroyed in content_clean() */
-			c->status = CONTENT_STATUS_ERROR;
-			if (result == URL_FUNC_OK) {
-				bool same;
-
-				result = url_compare(c->url, url, &same);
-
-				/* check that we're not attempting to
-				 * redirect to the same URL */
-				if (result != URL_FUNC_OK || same) {
-					msg_data.error =
-						messages_get("BadRedirect");
-					content_broadcast(c,
-						CONTENT_MSG_ERROR, msg_data);
-				}
-				else {
-					msg_data.redirect = url;
-					content_broadcast(c,
-							CONTENT_MSG_REDIRECT,
-							msg_data);
-				}
-				free(url);
-			} else {
-				msg_data.error = messages_get("BadRedirect");
-				content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
-			}
+			fetchcache_redirect(c, data, size);
 			break;
 
 		case FETCH_NOTMODIFIED:
 			fetchcache_notmodified(c, data);
 			break;
 
-#ifdef WITH_AUTH
 		case FETCH_AUTH:
 			/* data -> string containing the Realm */
 			LOG(("FETCH_AUTH, '%s'", (const char *)data));
@@ -505,9 +522,7 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 			 * destroyed in content_clean() */
 			c->status = CONTENT_STATUS_ERROR;
 			break;
-#endif
 
-#ifdef WITH_SSL
 		case FETCH_CERT_ERR:
 			c->fetch = 0;
 			/* set the status to ERROR so that the content is
@@ -518,7 +533,6 @@ void fetchcache_callback(fetch_msg msg, void *p, const void *data,
 			msg_data.ssl.num = size;
 			content_broadcast(c, CONTENT_MSG_SSL, msg_data);
 			break;
-#endif
 
 		default:
 			assert(0);
@@ -566,6 +580,7 @@ char *fetchcache_parse_type(const char *s, char **params[])
 
 	r = regexec(&re_content_type, s, 2 + MAX_ATTRS * 3, pmatch, 0);
 	if (r) {
+		char *semi;
 		LOG(("failed to parse content-type '%s'", s));
 		/* The mime type must be first, so only copy up to the
 		 * first semicolon in the string. This allows us to have
@@ -573,7 +588,7 @@ char *fetchcache_parse_type(const char *s, char **params[])
 		 * Content-Type headers. Obviously, any truly broken
 		 * Content-Type headers will be unaffected by this heuristic
 		 */
-		char *semi = strchr(s, ';');
+		semi = strchr(s, ';');
 		if (semi)
 			type = strndup(s, semi - s);
 		else
@@ -614,6 +629,96 @@ no_memory:
 }
 
 
+/**
+ * Parse an HTTP response header.
+ *
+ * See RFC 2616 4.2.
+ */
+
+void fetchcache_parse_header(struct content *c, const char *data,
+		size_t size)
+{
+	size_t i;
+
+#define SKIP_ST(o) for (i = (o); i < size && (data[i] == ' ' || data[i] == '\t'); i++)
+
+	/* Set fetch response time if not already set */
+	if (c->cache_data.res_time == 0)
+		c->cache_data.res_time = time(NULL);
+
+	if (5 < size && strncasecmp(data, "Date:", 5) == 0) {
+		/* extract Date header */
+		SKIP_ST(5);
+		if (i < size)
+			c->cache_data.date = curl_getdate(&data[i], NULL);
+	} else if (4 < size && strncasecmp(data, "Age:", 4) == 0) {
+		/* extract Age header */
+		SKIP_ST(4);
+		if (i < size && '0' <= data[i] && data[i] <= '9')
+			c->cache_data.age = atoi(data + i);
+	} else if (8 < size && strncasecmp(data, "Expires:", 8) == 0) {
+		/* extract Expires header */
+		SKIP_ST(8);
+		if (i < size)
+			c->cache_data.expires = curl_getdate(&data[i], NULL);
+	} else if (14 < size && strncasecmp(data, "Cache-Control:", 14) == 0) {
+		/* extract and parse Cache-Control header */
+		size_t comma;
+		SKIP_ST(14);
+
+		while (i < size) {
+			for (comma = i; comma < size; comma++)
+				if (data[comma] == ',')
+					break;
+
+			SKIP_ST(i);
+
+			if (8 < comma - i && (strncasecmp(data + i, "no-cache", 8) == 0 || strncasecmp(data + i, "no-store", 8) == 0))
+				/* When we get a disk cache we should
+				 * distinguish between these two */
+				c->cache_data.no_cache = true;
+			else if (7 < comma - i && strncasecmp(data + i, "max-age", 7) == 0) {
+				for (; i < comma; i++)
+					if (data[i] == '=')
+						break;
+				SKIP_ST(i+1);
+				if (i < comma)
+					c->cache_data.max_age =
+							atoi(data + i);
+			}
+
+			i = comma + 1;
+		}
+	} else if (5 < size && strncasecmp(data, "ETag:", 5) == 0) {
+		/* extract ETag header */
+		talloc_free(c->cache_data.etag);
+		c->cache_data.etag = talloc_array(c, char, size);
+		if (!c->cache_data.etag) {
+			LOG(("malloc failed"));
+			return;
+		}
+		SKIP_ST(5);
+		strncpy(c->cache_data.etag, data + i, size - i);
+		c->cache_data.etag[size - i] = '\0';
+		for (i = size - i - 1; ((int) i) >= 0 &&
+				(c->cache_data.etag[i] == ' ' ||
+				c->cache_data.etag[i] == '\t' ||
+				c->cache_data.etag[i] == '\r' ||
+				c->cache_data.etag[i] == '\n'); --i)
+			c->cache_data.etag[i] = '\0';
+	} else if (14 < size && strncasecmp(data, "Last-Modified:", 14) == 0) {
+		/* extract Last-Modified header */
+		SKIP_ST(14);
+		if (i < size) {
+			c->cache_data.last_modified =
+					curl_getdate(&data[i], NULL);
+		}
+	}
+
+	return;
+}
+
+
 /**
  * Generate an error page.
  *
@@ -634,48 +739,65 @@ void fetchcache_error_page(struct content *c, const char *error)
 	if (!content_process_data(c, error_page, length))
 		return;
 	content_convert(c, c->width, c->height);
+
+	/* Mark content as non-fresh, so it'll get cleaned from the 
+	 * cache at the earliest opportunity */
+	c->fresh = false;
 }
 
 
 /**
- * Update a content's cache info
+ * Update a content's cache state
  *
- * \param The content
- * \param Cache data
+ * \param c  The content
  */
 
-void fetchcache_cache_update(struct content *c,
+void fetchcache_cache_update(struct content *c)
+{
+	if (c->cache_data.date == 0)
+		c->cache_data.date = time(NULL);
+
+	if (c->cache_data.no_cache)
+		c->fresh = false;
+}
+
+/**
+ * Clone cache info into a content
+ *
+ * \param c     The content
+ * \param data  Cache data
+ */
+
+void fetchcache_cache_clone(struct content *c,
 		const struct cache_data *data)
 {
 	assert(c && data);
 
-	c->cache_data->req_time = data->req_time;
-	c->cache_data->res_time = data->res_time;
+	c->cache_data.req_time = data->req_time;
+	c->cache_data.res_time = data->res_time;
 
 	if (data->date != 0)
-		c->cache_data->date = data->date;
-	else
-		c->cache_data->date = time(0);
+		c->cache_data.date = data->date;
 
 	if (data->expires != 0)
-		c->cache_data->expires = data->expires;
+		c->cache_data.expires = data->expires;
 
 	if (data->age != INVALID_AGE)
-		c->cache_data->age = data->age;
+		c->cache_data.age = data->age;
 
 	if (data->max_age != INVALID_AGE)
-		c->cache_data->max_age = data->max_age;
+		c->cache_data.max_age = data->max_age;
 
 	if (data->no_cache)
-		c->fresh = false;
+		c->cache_data.no_cache = data->no_cache;
 
 	if (data->etag) {
-		talloc_free(c->cache_data->etag);
-		c->cache_data->etag = talloc_strdup(c, data->etag);
+		talloc_free(c->cache_data.etag);
+		c->cache_data.etag = talloc_strdup(c, data->etag);
 	}
 
 	if (data->last_modified)
-		c->cache_data->last_modified = data->last_modified;
+		c->cache_data.last_modified = data->last_modified;
 }
 
 
@@ -688,7 +810,7 @@ void fetchcache_notmodified(struct content *c, const void *data)
 	struct content *fb;
 	union content_msg_data msg_data;
 
-	assert(c && data);
+	assert(c);
 	assert(c->status == CONTENT_STATUS_TYPE_UNKNOWN);
 
 	/* Look for cached content */
@@ -718,6 +840,8 @@ void fetchcache_notmodified(struct content *c, const void *data)
 			}
 
 			content_remove_user(c, callback, p1, p2);
+
+			msg_data.new_url = NULL;
 			callback(CONTENT_MSG_NEWPTR, fb, p1, p2, msg_data);
 
 			/* and catch user up with fallback's state */
@@ -751,15 +875,18 @@ void fetchcache_notmodified(struct content *c, const void *data)
 		c->fetch = 0;
 		c->status = CONTENT_STATUS_ERROR;
 
-		/* and update fallback's cache control data */
-		fetchcache_cache_update(fb,
-			(const struct cache_data *)data);
+		/* clone our cache control data into the fallback */
+		fetchcache_cache_clone(fb, &c->cache_data);
+		/* and update the fallback's cache state */
+		fetchcache_cache_update(fb);
 	}
 	else {
 		/* No cached content, so unconditionally refetch */
 		struct content_user *u;
 		const char *ref = fetch_get_referer(c->fetch);
+		const char *parent = fetch_get_parent_url(c->fetch);
 		char *referer = NULL;
+		char *parent_url = NULL;
 
 		if (ref) {
 			referer = strdup(ref);
@@ -773,23 +900,276 @@ void fetchcache_notmodified(struct content *c, const void *data)
 			}
 		}
 
+		if (parent) {
+			parent_url = strdup(parent);
+			if (!parent_url) {
+				c->type = CONTENT_UNKNOWN;
+				c->status = CONTENT_STATUS_ERROR;
+				msg_data.error = messages_get("NoMemory");
+				content_broadcast(c, CONTENT_MSG_ERROR,
+						msg_data);
+				free(referer);
+				return;
+			}
+		}
+
 		fetch_abort(c->fetch);
 		c->fetch = 0;
 
-		c->cache_data->date = 0;
-		talloc_free(c->cache_data->etag);
-		c->cache_data->etag = 0;
+		c->cache_data.date = 0;
+		talloc_free(c->cache_data.etag);
+		c->cache_data.etag = 0;
 
 		for (u = c->user_list->next; u; u = u->next) {
 			fetchcache_go(c, referer, u->callback, u->p1, u->p2,
 					c->width, c->height, 0, 0,
-					false, ref ? referer : c->url);
+					false, parent_url);
 		}
 
+		free(parent_url);
 		free(referer);
 	}
 }
 
+/**
+ * Redirect callback handler
+ */
+
+void fetchcache_redirect(struct content *c, const void *data,
+		unsigned long size)
+{
+	char *url, *url1;
+	char *referer, *parent_url;
+	char *scheme;
+	long http_code;
+	const char *ref;
+	const char *parent;
+	bool can_fetch;
+	bool parent_was_verifiable;
+	union content_msg_data msg_data;
+	url_func_result result;
+
+	/* Preconditions */
+	assert(c && data);
+	assert(c->status == CONTENT_STATUS_TYPE_UNKNOWN);
+
+	/* Extract fetch details */
+	http_code = fetch_http_code(c->fetch);
+	ref = fetch_get_referer(c->fetch);
+	parent = fetch_get_parent_url(c->fetch);
+	parent_was_verifiable = fetch_get_verifiable(c->fetch);
+
+	/* Ensure a redirect happened */
+	assert(300 <= http_code && http_code <= 399);
+	/* 304 is handled by fetch_notmodified() */
+	assert(http_code != 304);
+
+	/* Clone referer and parent url
+	 * originals are destroyed in fetch_abort() */
+	referer = ref ? strdup(ref) : NULL;
+	parent_url = parent ? strdup(parent) : NULL;
+
+	/* set the status to ERROR so that this content is
+	 * destroyed in content_clean() */
+	fetch_abort(c->fetch);
+	c->fetch = 0;
+	c->status = CONTENT_STATUS_ERROR;
+
+	/* Ensure that referer cloning succeeded 
+	 * _must_ be after content invalidation */
+	if (ref && !referer) {
+		LOG(("Failed cloning referer"));
+
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		return;
+	}
+
+	/* Ensure parent url cloning succeeded
+	 * _must_ be after content invalidation */
+	if (parent && !parent_url) {
+		LOG(("Failed cloning parent url"));
+
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		free(referer);
+		return;
+	}
+
+	/** \todo 300, 305, 307
+	 * More specifically:
+	 *  + 300 needs to serve up the fetch body to the user
+	 *  + 305 needs to refetch using the proxy specified in ::data
+	 *  + 307 needs to refetch.
+	 * 
+	 * If the original request method was either GET or HEAD, then follow
+	 * redirect unconditionally. If the original request method was neither
+	 * GET nor HEAD, then the user MUST be asked what to do.
+	 *
+	 * Note:
+	 *  For backwards compatibility, all 301, 302 and 303 redirects are
+	 *  followed unconditionally with a GET request to the new location.
+	 */
+	if (http_code != 301 && http_code != 302 && http_code != 303) {
+		LOG(("Unsupported redirect type %ld", http_code));
+
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		free(parent_url);
+		free(referer);
+		return;
+	}
+
+	/* Forcibly stop redirecting if we've followed too many redirects */
+#define REDIRECT_LIMIT 10
+	if (c->redirect_count > REDIRECT_LIMIT) {
+		LOG(("Too many nested redirects"));
+
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		free(parent_url);
+		free(referer);
+		return;
+	}
+#undef REDIRECT_LIMIT
+
+	/* redirect URLs must be absolute by HTTP/1.1, but many
+	 * sites send relative ones: treat them as relative to 
+	 * requested URL */
+	result = url_join(data, c->url, &url1);
+	if (result != URL_FUNC_OK) {
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		free(parent_url);
+		free(referer);
+		return;
+	}
+
+	/* Normalize redirect target -- this is vital as this URL may
+	 * be inserted into the urldb, which expects normalized URLs */
+	result = url_normalize(url1, &url);
+	if (result != URL_FUNC_OK) {
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		free(url1);
+		free(parent_url);
+		free(referer);
+		return;
+	}
+
+	/* No longer need url1 */
+	free(url1);
+
+	/* Ensure that redirects to file:/// URLs are trapped */
+	result = url_scheme(url, &scheme);
+	if (result != URL_FUNC_OK) {
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		free(url);
+		free(parent_url);
+		free(referer);
+		return;
+	}
+
+	if (strcasecmp(scheme, "file") == 0) {
+		msg_data.error = messages_get("BadRedirect");
+		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
+
+		free(scheme);
+		free(url);
+		free(parent_url);
+		free(referer);
+		return;
+	}
+
+	free(scheme); 
+	
+	/* Determine if we've got a fetch handler for this url */
+	can_fetch = fetch_can_fetch(url);
+
+	/* Process users of this content */
+	while (c->user_list->next) {
+		intptr_t p1, p2;
+		void (*callback)(content_msg msg,
+			struct content *c, intptr_t p1,
+			intptr_t p2,
+			union content_msg_data data);
+		struct content *replacement;
+	
+		p1 = c->user_list->next->p1;
+		p2 = c->user_list->next->p2;
+		callback = c->user_list->next->callback;
+
+		/* If we can't fetch this url, attempt to launch it */
+		if (!can_fetch) {
+			msg_data.launch_url = url;
+			callback(CONTENT_MSG_LAUNCH, c, p1, p2, msg_data);
+		}
+
+		/* Remove user */
+		content_remove_user(c, callback, p1, p2);
+
+		if (can_fetch) {
+			/* Get replacement content -- HTTP GET request */
+
+			/* A note about fetch verifiability: according to
+			 * both RFC2109 and 2965, redirects result in an
+			 * unverifiable fetch and thus cookies must be handled
+			 * differently. Unfortunately, however, other browsers
+			 * do not adhere to this rule and just process cookies
+			 * as per normal in this case. Websites have come to
+			 * depend upon this "feature", so we must do something
+			 * which approximates the appropriate behaviour.
+			 *
+			 * Therefore, a redirected fetch will preserve the
+			 * verifiability of the origin fetch. Thus, fetches
+			 * for embedded objects will remain unverifiable,
+			 * as expected.
+			 */
+			replacement = fetchcache(url, callback, p1, p2, 
+					c->width, c->height, c->no_error_pages,
+					NULL, NULL, parent_was_verifiable, 
+					c->download);
+			if (!replacement) {
+				msg_data.error = messages_get("BadRedirect");
+				content_broadcast(c, CONTENT_MSG_ERROR, 
+						msg_data);
+
+				free(url);
+				free(parent_url);
+				free(referer);
+				return;
+			}
+
+			/* Set replacement's redirect count to 1 greater 
+			 * than ours */
+			replacement->redirect_count = c->redirect_count + 1;
+
+			/* Notify user that content has changed */
+			msg_data.new_url = url;
+			callback(CONTENT_MSG_NEWPTR, replacement, 
+					p1, p2, msg_data);
+
+			/* Start fetching the replacement content */
+			fetchcache_go(replacement, referer, callback, p1, p2,
+					c->width, c->height, NULL, NULL,
+					parent_was_verifiable, parent_url);
+		}
+	}
+
+	/* Clean up */
+	free(url);
+	free(parent_url);
+	free(referer);
+}
+
 #ifdef TEST
 
 #include <unistd.h>
