@@ -11,7 +11,7 @@
  *
  * NetSurf is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
@@ -24,7 +24,7 @@
  * This implementation uses libcurl's 'multi' interface.
  *
  *
- * The CURL handles are cached in the cache_ring. There are at most
+ * The CURL handles are cached in the curl_handle_ring. There are at most
  * ::option_max_cached_fetch_handles in this ring.
  */
 
@@ -37,13 +37,8 @@
 #include <sys/select.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
-#ifdef riscos
-#include <unixlib/local.h>
-#endif
 #include "utils/config.h"
-#ifdef WITH_SSL
 #include <openssl/ssl.h>
-#endif
 #include "content/fetch.h"
 #include "content/fetchers/fetch_curl.h"
 #include "content/urldb.h"
@@ -58,13 +53,11 @@
 #include "utils/ring.h"
 #include "utils/useragent.h"
 
-#ifdef WITH_SSL
 /** SSL certificate info */
 struct cert_info {
 	X509 *cert;		/**< Pointer to certificate */
 	long err;		/**< OpenSSL error code */
 };
-#endif
 
 /** Information for a single fetch. */
 struct curl_fetch_info {
@@ -74,26 +67,20 @@ struct curl_fetch_info {
 	bool abort;		/**< Abort requested. */
 	bool stopped;		/**< Download stopped on purpose. */
 	bool only_2xx;		/**< Only HTTP 2xx responses acceptable. */
-	bool verifiable;	/**< Transaction is verifiable */
 	char *url;		/**< URL of this fetch. */
 	char *host;		/**< The hostname of this fetch. */
-	char *parent_fetch_url;	/**< URL of parent fetch (not necessarily
-				 * the same as the referer) */
 	struct curl_slist *headers;	/**< List of request headers. */
 	char *location;		/**< Response Location header, or 0. */
 	unsigned long content_length;	/**< Response Content-Length, or 0. */
 	char *cookie_string;	/**< Cookie string for this fetch */
 	char *realm;		/**< HTTP Auth Realm */
 	char *post_urlenc;	/**< Url encoded POST string, or 0. */
-	unsigned long http_code; /**< HTTP result code from cURL. */
+	long http_code; /**< HTTP result code from cURL. */
 	struct curl_httppost *post_multipart;	/**< Multipart post data, or 0. */
-	struct cache_data cachedata;	/**< Cache control data */
 	time_t last_modified;		/**< If-Modified-Since time */
 	time_t file_etag;		/**< ETag for local objects */
-#ifdef WITH_SSL
 #define MAX_CERTS 10
 	struct cert_info cert_data[MAX_CERTS];	/**< HTTPS certificate data */
-#endif
 };
 
 struct cache_handle {
@@ -109,6 +96,7 @@ CURLM *fetch_curl_multi;		/**< Global cURL multi handle. */
 static CURL *fetch_blank_curl;
 static struct cache_handle *curl_handle_ring = 0; /**< Ring of cached handles */
 static int curl_fetchers_registered = 0;
+static bool curl_with_openssl;
 
 static char fetch_error_buffer[CURL_ERROR_SIZE]; /**< Error buffer for cURL. */
 static char fetch_progress_buffer[256]; /**< Progress buffer for cURL */
@@ -119,17 +107,15 @@ static void fetch_curl_finalise(const char *scheme);
 static void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 		 bool only_2xx, const char *post_urlenc,
 		 struct form_successful_control *post_multipart,
-		 bool verifiable, const char *parent_url, const char **headers);
+		 const char **headers);
 static bool fetch_curl_start(void *vfetch);
 static bool fetch_curl_initiate_fetch(struct curl_fetch_info *fetch,
 		CURL *handle);
 static CURL *fetch_curl_get_handle(char *host);
 static void fetch_curl_cache_handle(CURL *handle, char *hostname);
 static CURLcode fetch_curl_set_options(struct curl_fetch_info *f);
-#ifdef WITH_SSL
-static CURLcode fetch_curl_sslctxfun(CURL *curl_handle, SSL_CTX *sslctx,
-		void *p);
-#endif
+static CURLcode fetch_curl_sslctxfun(CURL *curl_handle, void *_sslctx,
+				     void *p);
 static void fetch_curl_abort(void *vf);
 static void fetch_curl_stop(struct curl_fetch_info *f);
 static void fetch_curl_free(void *f);
@@ -137,20 +123,22 @@ static void fetch_curl_poll(const char *scheme_ignored);
 static void fetch_curl_done(CURL *curl_handle, CURLcode result);
 static int fetch_curl_progress(void *clientp, double dltotal, double dlnow,
 		double ultotal, double ulnow);
-static int fetch_curl_ignore(void);
-static size_t fetch_curl_data(void *data, size_t size, size_t nmemb,
-		struct curl_fetch_info *f);
+static int fetch_curl_ignore_debug(CURL *handle,
+				   curl_infotype type,
+				   char *data,
+				   size_t size,
+				   void *userptr);
+static size_t fetch_curl_data(char *data, size_t size, size_t nmemb,
+			      void *_f);
 static size_t fetch_curl_header(char *data, size_t size, size_t nmemb,
-		struct curl_fetch_info *f);
+				void *_f);
 static bool fetch_curl_process_headers(struct curl_fetch_info *f);
 static struct curl_httppost *fetch_curl_post_convert(
 		struct form_successful_control *control);
-#ifdef WITH_SSL
 static int fetch_curl_verify_callback(int preverify_ok,
 		X509_STORE_CTX *x509_ctx);
 static int fetch_curl_cert_verify_callback(X509_STORE_CTX *x509_ctx,
 		void *parm);
-#endif
 
 
 /**
@@ -197,7 +185,7 @@ void fetch_curl_register(void)
 	}
 	SETOPT(CURLOPT_ERRORBUFFER, fetch_error_buffer);
 	if (option_suppress_curl_debug)
-		SETOPT(CURLOPT_DEBUGFUNCTION, fetch_curl_ignore);
+		SETOPT(CURLOPT_DEBUGFUNCTION, fetch_curl_ignore_debug);
 	SETOPT(CURLOPT_WRITEFUNCTION, fetch_curl_data);
 	SETOPT(CURLOPT_HEADERFUNCTION, fetch_curl_header);
 	SETOPT(CURLOPT_PROGRESSFUNCTION, fetch_curl_progress);
@@ -214,6 +202,16 @@ void fetch_curl_register(void)
 	if (option_ca_path && strcmp(option_ca_path, ""))
 		SETOPT(CURLOPT_CAPATH, option_ca_path);
 
+	/* Detect whether the SSL CTX function API works */
+	curl_with_openssl = true;
+	code = curl_easy_setopt(fetch_blank_curl, 
+			CURLOPT_SSL_CTX_FUNCTION, NULL);
+	if (code != CURLE_OK) {
+		curl_with_openssl = false;
+	}
+
+	LOG(("cURL %slinked against openssl", curl_with_openssl ? "" : "not "));
+
 	/* cURL initialised okay, register the fetchers */
 
 	data = curl_version_info(CURLVERSION_NOW);
@@ -259,9 +257,9 @@ void fetch_curl_finalise(const char *scheme)
 	curl_fetchers_registered--;
 	LOG(("Finalise cURL fetcher %s", scheme));
 	if (curl_fetchers_registered == 0) {
+		CURLMcode codem;
 		/* All the fetchers have been finalised. */
 		LOG(("All cURL fetchers finalised, closing down cURL"));
-		CURLMcode codem;
 
 		curl_easy_cleanup(fetch_blank_curl);
 
@@ -280,9 +278,9 @@ void fetch_curl_finalise(const char *scheme)
  * The function returns immediately. The fetch may be queued for later
  * processing.
  *
- * A pointer to an opaque struct curl_fetch_info is returned, which can be passed to
- * fetch_abort() to abort the fetch at any time. Returns 0 if memory is
- * exhausted (or some other fatal error occurred).
+ * A pointer to an opaque struct curl_fetch_info is returned, which can be 
+ * passed to fetch_abort() to abort the fetch at any time. Returns 0 if memory 
+ * is exhausted (or some other fatal error occurred).
  *
  * The caller must supply a callback function which is called when anything
  * interesting happens. The callback function is first called with msg
@@ -299,7 +297,7 @@ void fetch_curl_finalise(const char *scheme)
 void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 		 bool only_2xx, const char *post_urlenc,
 		 struct form_successful_control *post_multipart,
-		 bool verifiable, const char *parent_url, const char **headers)
+		 const char **headers)
 {
 	char *host;
 	struct curl_fetch_info *fetch;
@@ -332,9 +330,7 @@ void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 	fetch->abort = false;
 	fetch->stopped = false;
 	fetch->only_2xx = only_2xx;
-	fetch->verifiable = verifiable;
 	fetch->url = strdup(url);
-	fetch->parent_fetch_url = parent_url ? strdup(parent_url) : 0;
 	fetch->headers = 0;
 	fetch->host = host;
 	fetch->location = 0;
@@ -348,24 +344,12 @@ void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 		fetch->post_urlenc = strdup(post_urlenc);
 	else if (post_multipart)
 		fetch->post_multipart = fetch_curl_post_convert(post_multipart);
-	fetch->cachedata.req_time = time(0);
-	fetch->cachedata.res_time = 0;
-	fetch->cachedata.date = 0;
-	fetch->cachedata.expires = 0;
-	fetch->cachedata.age = INVALID_AGE;
-	fetch->cachedata.max_age = INVALID_AGE;
-	fetch->cachedata.no_cache = false;
-	fetch->cachedata.etag = 0;
-	fetch->cachedata.last_modified = 0;
 	fetch->last_modified = 0;
 	fetch->file_etag = 0;
 	fetch->http_code = 0;
-#ifdef WITH_SSL
 	memset(fetch->cert_data, 0, sizeof(fetch->cert_data));
-#endif
 
 	if (!fetch->url ||
-	    (parent_url && !fetch->parent_fetch_url) ||
 	    (post_urlenc && !fetch->post_urlenc) ||
 	    (post_multipart && !fetch->post_multipart))
 		goto failed;
@@ -377,7 +361,6 @@ void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 	list = slist;
 
 	/* remove curl default headers */
-	APPEND(fetch->headers, "Accept:");
 	APPEND(fetch->headers, "Pragma:");
 
 	/* when doing a POST libcurl sends Expect: 100-continue" by default
@@ -423,7 +406,6 @@ void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 failed:
 	free(host);
 	free(fetch->url);
-	free(fetch->parent_fetch_url);
 	free(fetch->post_urlenc);
 	if (fetch->post_multipart)
 		curl_formfree(fetch->post_multipart);
@@ -559,16 +541,16 @@ fetch_curl_set_options(struct curl_fetch_info *f)
 	SETOPT(CURLOPT_REFERER, fetch_get_referer_to_send(f->fetch_handle));
 	SETOPT(CURLOPT_HTTPHEADER, f->headers);
 	if (f->post_urlenc) {
-		SETOPT(CURLOPT_HTTPPOST, 0);
+		SETOPT(CURLOPT_HTTPPOST, NULL);
 		SETOPT(CURLOPT_HTTPGET, 0L);
 		SETOPT(CURLOPT_POSTFIELDS, f->post_urlenc);
 	} else if (f->post_multipart) {
-		SETOPT(CURLOPT_POSTFIELDS, 0);
+		SETOPT(CURLOPT_POSTFIELDS, NULL);
 		SETOPT(CURLOPT_HTTPGET, 0L);
 		SETOPT(CURLOPT_HTTPPOST, f->post_multipart);
 	} else {
-		SETOPT(CURLOPT_POSTFIELDS, 0);
-		SETOPT(CURLOPT_HTTPPOST, 0);
+		SETOPT(CURLOPT_POSTFIELDS, NULL);
+		SETOPT(CURLOPT_HTTPPOST, NULL);
 		SETOPT(CURLOPT_HTTPGET, 1L);
 	}
 
@@ -576,19 +558,16 @@ fetch_curl_set_options(struct curl_fetch_info *f)
 	if (f->cookie_string) {
 		SETOPT(CURLOPT_COOKIE, f->cookie_string);
 	} else {
-		SETOPT(CURLOPT_COOKIE, 0);
+		SETOPT(CURLOPT_COOKIE, NULL);
 	}
 
-#ifdef WITH_AUTH
 	if ((auth = urldb_get_auth_details(f->url)) != NULL) {
 		SETOPT(CURLOPT_HTTPAUTH, CURLAUTH_ANY);
 		SETOPT(CURLOPT_USERPWD, auth);
 	} else {
-#endif
-		SETOPT(CURLOPT_USERPWD, 0);
-#ifdef WITH_AUTH
+		SETOPT(CURLOPT_USERPWD, NULL);
 	}
-#endif
+
 	if (option_http_proxy && option_http_proxy_host) {
 		SETOPT(CURLOPT_PROXY, option_http_proxy_host);
 		SETOPT(CURLOPT_PROXYPORT, (long) option_http_proxy_port);
@@ -607,40 +586,41 @@ fetch_curl_set_options(struct curl_fetch_info *f)
 		}
 	}
 
-#ifdef WITH_SSL
 	if (urldb_get_cert_permissions(f->url)) {
 		/* Disable certificate verification */
 		SETOPT(CURLOPT_SSL_VERIFYPEER, 0L);
 		SETOPT(CURLOPT_SSL_VERIFYHOST, 0L);
-		SETOPT(CURLOPT_SSL_CTX_FUNCTION, 0);
-		SETOPT(CURLOPT_SSL_CTX_DATA, 0);
+		if (curl_with_openssl) {
+			SETOPT(CURLOPT_SSL_CTX_FUNCTION, NULL);
+			SETOPT(CURLOPT_SSL_CTX_DATA, NULL);
+		}
 	} else {
 		/* do verification */
 		SETOPT(CURLOPT_SSL_VERIFYPEER, 1L);
 		SETOPT(CURLOPT_SSL_VERIFYHOST, 2L);
-		SETOPT(CURLOPT_SSL_CTX_FUNCTION, fetch_curl_sslctxfun);
-		SETOPT(CURLOPT_SSL_CTX_DATA, f);
+		if (curl_with_openssl) {
+			SETOPT(CURLOPT_SSL_CTX_FUNCTION, fetch_curl_sslctxfun);
+			SETOPT(CURLOPT_SSL_CTX_DATA, f);
+		}
 	}
-#endif
 
 	return CURLE_OK;
 }
 
 
-#ifdef WITH_SSL
 /**
  * cURL SSL setup callback
  */
 
 CURLcode
-fetch_curl_sslctxfun(CURL *curl_handle, SSL_CTX *sslctx, void *parm)
+fetch_curl_sslctxfun(CURL *curl_handle, void *_sslctx, void *parm)
 {
+	SSL_CTX *sslctx = _sslctx;
 	SSL_CTX_set_verify(sslctx, SSL_VERIFY_PEER, fetch_curl_verify_callback);
 	SSL_CTX_set_cert_verify_callback(sslctx, fetch_curl_cert_verify_callback,
 					 parm);
 	return CURLE_OK;
 }
-#endif
 
 
 /**
@@ -695,15 +675,12 @@ void fetch_curl_stop(struct curl_fetch_info *f)
 void fetch_curl_free(void *vf)
 {
 	struct curl_fetch_info *f = (struct curl_fetch_info *)vf;
-#ifdef WITH_SSL
 	int i;
-#endif
 
 	if (f->curl_handle)
 		curl_easy_cleanup(f->curl_handle);
 	free(f->url);
 	free(f->host);
-	free(f->parent_fetch_url);
 	free(f->location);
 	free(f->cookie_string);
 	free(f->realm);
@@ -712,15 +689,12 @@ void fetch_curl_free(void *vf)
 	free(f->post_urlenc);
 	if (f->post_multipart)
 		curl_formfree(f->post_multipart);
-	free(f->cachedata.etag);
 
-#ifdef WITH_SSL
 	for (i = 0; i < MAX_CERTS && f->cert_data[i].cert; i++) {
 		f->cert_data[i].cert->references--;
 		if (f->cert_data[i].cert->references == 0)
 			X509_free(f->cert_data[i].cert);
 	}
-#endif
 
 	free(f);
 }
@@ -741,7 +715,12 @@ void fetch_curl_poll(const char *scheme_ignored)
 	/* do any possible work on the current fetches */
 	do {
 		codem = curl_multi_perform(fetch_curl_multi, &running);
-		assert(codem == CURLM_OK || codem == CURLM_CALL_MULTI_PERFORM);
+		if (codem != CURLM_OK && codem != CURLM_CALL_MULTI_PERFORM) {
+			LOG(("curl_multi_perform: %i %s",
+					codem, curl_multi_strerror(codem)));
+			warn_user("MiscError", curl_multi_strerror(codem));
+			return;
+		}
 	} while (codem == CURLM_CALL_MULTI_PERFORM);
 
 	/* process curl results */
@@ -763,33 +742,30 @@ void fetch_curl_poll(const char *scheme_ignored)
 /**
  * Handle a completed fetch (CURLMSG_DONE from curl_multi_info_read()).
  *
- * \param  curl_handle  curl easy handle of fetch
+ * \param  curl_handle	curl easy handle of fetch
  */
 
 void fetch_curl_done(CURL *curl_handle, CURLcode result)
 {
 	bool finished = false;
 	bool error = false;
-#ifdef WITH_SSL
 	bool cert = false;
-#endif
-	bool abort;
+	bool abort_fetch;
 	struct curl_fetch_info *f;
+	char **_hideous_hack = (char **) (void *) &f;
 	CURLcode code;
-	struct cache_data cachedata;
-#ifdef WITH_SSL
 	struct cert_info certs[MAX_CERTS];
 	memset(certs, 0, sizeof(certs));
-#endif
 
 	/* find the structure associated with this fetch */
-	code = curl_easy_getinfo(curl_handle, CURLINFO_PRIVATE, &f);
+	/* For some reason, cURL thinks CURLINFO_PRIVATE should be a string?! */
+	code = curl_easy_getinfo(curl_handle, CURLINFO_PRIVATE, _hideous_hack);
 	assert(code == CURLE_OK);
 
-	abort = f->abort;
+	abort_fetch = f->abort;
 	LOG(("done %s", f->url));
 
-	if (!abort && result == CURLE_OK) {
+	if (abort_fetch == false && result == CURLE_OK) {
 		/* fetch completed normally */
 		if (f->stopped ||
 				(!f->had_headers &&
@@ -809,33 +785,23 @@ void fetch_curl_done(CURL *curl_handle, CURLcode result)
 		/* CURLE_WRITE_ERROR occurs when fetch_curl_data
 		 * returns 0, which we use to abort intentionally */
 		;
-#ifdef WITH_SSL
 	else if (result == CURLE_SSL_PEER_CERTIFICATE ||
 			result == CURLE_SSL_CACERT) {
 		memcpy(certs, f->cert_data, sizeof(certs));
 		memset(f->cert_data, 0, sizeof(f->cert_data));
 		cert = true;
 	}
-#endif
-	else
+	else {
+		LOG(("Unknown cURL response code %d", result));
 		error = true;
+	}
 
 	fetch_curl_stop(f);
 
-	/* If finished, acquire cache info to pass to callback */
-	if (finished) {
-		memcpy(&cachedata, &f->cachedata, sizeof(struct cache_data));
-		f->cachedata.etag = 0;
-	}
-
-	if (abort)
+	if (abort_fetch)
 		; /* fetch was aborted: no callback */
-	else if (finished) {
-		fetch_send_callback(FETCH_FINISHED, f->fetch_handle,
-				&cachedata, 0);
-		free(cachedata.etag);
-	}
-#ifdef WITH_SSL
+	else if (finished)
+		fetch_send_callback(FETCH_FINISHED, f->fetch_handle, 0, 0);
 	else if (cert) {
 		int i;
 		BIO *mem;
@@ -917,7 +883,6 @@ void fetch_curl_done(CURL *curl_handle, CURLcode result)
 				&ssl_certs, i);
 
 	}
-#endif
 	else if (error)
 		fetch_send_callback(FETCH_ERROR, f->fetch_handle,
 				fetch_error_buffer, 0);
@@ -967,7 +932,11 @@ int fetch_curl_progress(void *clientp, double dltotal, double dlnow,
  * Used to ignore cURL debug.
  */
 
-int fetch_curl_ignore(void)
+int fetch_curl_ignore_debug(CURL *handle,
+			    curl_infotype type,
+			    char *data,
+			    size_t size,
+			    void *userptr)
 {
 	return 0;
 }
@@ -977,9 +946,10 @@ int fetch_curl_ignore(void)
  * Callback function for cURL.
  */
 
-size_t fetch_curl_data(void *data, size_t size, size_t nmemb,
-		       struct curl_fetch_info *f)
+size_t fetch_curl_data(char *data, size_t size, size_t nmemb,
+		       void *_f)
 {
+	struct curl_fetch_info *f = _f;
 	CURLcode code;
 
 	/* ensure we only have to get this information once */
@@ -1021,19 +991,25 @@ size_t fetch_curl_data(void *data, size_t size, size_t nmemb,
 
 /**
  * Callback function for headers.
+ *
+ * See RFC 2616 4.2.
  */
 
 size_t fetch_curl_header(char *data, size_t size, size_t nmemb,
-			 struct curl_fetch_info *f)
+			 void *_f)
 {
+	struct curl_fetch_info *f = _f;
 	int i;
 	size *= nmemb;
 
-#define SKIP_ST(o) for (i = (o); i < (int) size && (data[i] == ' ' || data[i] == '\t'); i++)
+	if (f->abort) {
+		f->stopped = true;
+		return 0;
+	}
 
-	/* Set fetch response time if not already set */
-	if (f->cachedata.res_time == 0)
-		f->cachedata.res_time = time(0);
+	fetch_send_callback(FETCH_HEADER, f->fetch_handle, data, size);
+
+#define SKIP_ST(o) for (i = (o); i < (int) size && (data[i] == ' ' || data[i] == '\t'); i++)
 
 	if (12 < size && strncasecmp(data, "Location:", 9) == 0) {
 		/* extract Location header */
@@ -1057,7 +1033,6 @@ size_t fetch_curl_header(char *data, size_t size, size_t nmemb,
 		SKIP_ST(15);
 		if (i < (int)size && '0' <= data[i] && data[i] <= '9')
 			f->content_length = atol(data + i);
-#ifdef WITH_AUTH
 	} else if (17 < size && strncasecmp(data, "WWW-Authenticate:", 17) == 0) {
 		/* extract the first Realm from WWW-Authenticate header */
 		free(f->realm);
@@ -1086,90 +1061,11 @@ size_t fetch_curl_header(char *data, size_t size, size_t nmemb,
 					f->realm[i] == '\n'); --i)
 				f->realm[i] = '\0';
 		}
-#endif
-	} else if (5 < size && strncasecmp(data, "Date:", 5) == 0) {
-		/* extract Date header */
-		SKIP_ST(5);
-		if (i < (int) size)
-			f->cachedata.date = curl_getdate(&data[i], NULL);
-	} else if (4 < size && strncasecmp(data, "Age:", 4) == 0) {
-		/* extract Age header */
-		SKIP_ST(4);
-		if (i < (int) size && '0' <= data[i] && data[i] <= '9')
-			f->cachedata.age = atoi(data + i);
-	} else if (8 < size && strncasecmp(data, "Expires:", 8) == 0) {
-		/* extract Expires header */
-		SKIP_ST(8);
-		if (i < (int) size)
-			f->cachedata.expires = curl_getdate(&data[i], NULL);
-	} else if (14 < size && strncasecmp(data, "Cache-Control:", 14) == 0) {
-		/* extract and parse Cache-Control header */
-		int comma;
-		SKIP_ST(14);
-
-		while (i < (int) size) {
-			for (comma = i; comma < (int) size; comma++)
-				if (data[comma] == ',')
-					break;
-
-			SKIP_ST(i);
-
-			if (8 < comma - i && (strncasecmp(data + i, "no-cache", 8) == 0 || strncasecmp(data + i, "no-store", 8) == 0))
-				/* When we get a disk cache we should
-				 * distinguish between these two */
-				f->cachedata.no_cache = true;
-			else if (7 < comma - i && strncasecmp(data + i, "max-age", 7) == 0) {
-				for (; i < comma; i++)
-					if (data[i] == '=')
-						break;
-				SKIP_ST(i+1);
-				if (i < comma)
-					f->cachedata.max_age =
-							atoi(data + i);
-			}
-
-			i = comma + 1;
-		}
-	} else if (5 < size && strncasecmp(data, "ETag:", 5) == 0) {
-		/* extract ETag header */
-		free(f->cachedata.etag);
-		f->cachedata.etag = malloc(size);
-		if (!f->cachedata.etag) {
-			LOG(("malloc failed"));
-			return size;
-		}
-		SKIP_ST(5);
-		strncpy(f->cachedata.etag, data + i, size - i);
-		f->cachedata.etag[size - i] = '\0';
-		for (i = size - i - 1; i >= 0 &&
-				(f->cachedata.etag[i] == ' ' ||
-				f->cachedata.etag[i] == '\t' ||
-				f->cachedata.etag[i] == '\r' ||
-				f->cachedata.etag[i] == '\n'); --i)
-			f->cachedata.etag[i] = '\0';
-	} else if (14 < size && strncasecmp(data, "Last-Modified:", 14) == 0) {
-		/* extract Last-Modified header */
-		SKIP_ST(14);
-		if (i < (int) size) {
-			f->cachedata.last_modified =
-					curl_getdate(&data[i], NULL);
-		}
 	} else if (11 < size && strncasecmp(data, "Set-Cookie:", 11) == 0) {
 		/* extract Set-Cookie header */
 		SKIP_ST(11);
 
-		/* If the fetch is unverifiable and there's no parent fetch
-		 * url, err on the side of caution and do not set the
-		 cookie */
-
-		if (f->verifiable || f->parent_fetch_url) {
-			/* If the transaction's verifiable, we don't require
-			 * that the request uri and the parent domain match,
-			 * so don't pass in the parent in this case. */
-			urldb_set_cookie(&data[i], f->url,
-					f->verifiable ? 0
-						      : f->parent_fetch_url);
-		}
+		fetch_set_cookie(f->fetch_handle, &data[i]);
 	}
 
 	return size;
@@ -1193,10 +1089,6 @@ bool fetch_curl_process_headers(struct curl_fetch_info *f)
 
 	f->had_headers = true;
 
-	/* Set fetch response time if not already set */
-	if (f->cachedata.res_time == 0)
-		f->cachedata.res_time = time(0);
-
 	if (!f->http_code)
 	{
 		code = curl_easy_getinfo(f->curl_handle, CURLINFO_HTTP_CODE,
@@ -1209,8 +1101,7 @@ bool fetch_curl_process_headers(struct curl_fetch_info *f)
 
 	if (http_code == 304 && !f->post_urlenc && !f->post_multipart) {
 		/* Not Modified && GET request */
-		fetch_send_callback(FETCH_NOTMODIFIED, f->fetch_handle,
-				    (const char *)&f->cachedata, 0);
+		fetch_send_callback(FETCH_NOTMODIFIED, f->fetch_handle, 0, 0);
 		return true;
 	}
 
@@ -1222,12 +1113,10 @@ bool fetch_curl_process_headers(struct curl_fetch_info *f)
 	}
 
 	/* handle HTTP 401 (Authentication errors) */
-#ifdef WITH_AUTH
 	if (http_code == 401) {
 		fetch_send_callback(FETCH_AUTH, f->fetch_handle, f->realm,0);
 		return true;
 	}
-#endif
 
 	/* handle HTTP errors (non 2xx response codes) */
 	if (f->only_2xx && strncmp(f->url, "http", 4) == 0 &&
@@ -1248,11 +1137,12 @@ bool fetch_curl_process_headers(struct curl_fetch_info *f)
 	if (url_path && stat(url_path, &s) == 0) {
 		/* file: URL and file exists */
 		/* create etag */
-		free(f->cachedata.etag);
-		f->cachedata.etag = malloc(13);
-		if (f->cachedata.etag)
-			sprintf(f->cachedata.etag,
-					"\"%10d\"", (int)s.st_mtime);
+		char etag_buf[20];
+		snprintf(etag_buf, sizeof etag_buf,
+				"ETag: \"%10d\"", (int) s.st_mtime);
+		/* And send it to the header handler */
+		fetch_send_callback(FETCH_HEADER, f->fetch_handle, etag_buf,
+				strlen(etag_buf));
 
 		/* don't set last modified time so as to ensure that local
 		 * files are revalidated at all times. */
@@ -1262,7 +1152,7 @@ bool fetch_curl_process_headers(struct curl_fetch_info *f)
 				f->last_modified > s.st_mtime &&
 				f->file_etag == s.st_mtime) {
 			fetch_send_callback(FETCH_NOTMODIFIED, f->fetch_handle,
-					    (const char *)&f->cachedata, 0);
+					    0, 0);
 			curl_free(url_path);
 			return true;
 		}
@@ -1388,7 +1278,6 @@ fetch_curl_post_convert(struct form_successful_control *control)
 }
 
 
-#ifdef WITH_SSL
 /**
  * OpenSSL Certificate verification callback
  * Stores certificate details in fetch struct.
@@ -1431,4 +1320,3 @@ int fetch_curl_cert_verify_callback(X509_STORE_CTX *x509_ctx, void *parm)
 
 	return ok;
 }
-#endif
