@@ -35,6 +35,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
+#include <math.h>
 #define CSS_INTERNALS
 #define NDEBUG
 #include "css/css.h"
@@ -182,6 +183,29 @@ static css_text_decoration css_text_decoration_parse(const char * const s,
 		int length);
 
 
+/** Invalid hex */
+#define IH 0xffffffff
+/** ASCII to hexadeximal conversion */
+static const unsigned int ascii_to_hex[] = {
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x00 - 0x0f */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x10 - 0x1f */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x20 - 0x2f */
+	 0,  1,  2,  3,  4,  5,  6,  7,  8,  9, IH, IH, IH, IH, IH, IH,	/* 0x30 - 0x3f */
+	IH, 10, 11, 12, 13, 14, 15, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x40 - 0x4f */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x50 - 0x5f */
+	IH, 10, 11, 12, 13, 14, 15, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x60 - 0x6f */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x70 - 0x7f */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x80 - 0x8f */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0x90 - 0x9f */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0xa0 - 0xaf */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0xb0 - 0xbf */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0xc0 - 0xcf */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0xd0 - 0xdf */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH,	/* 0xe0 - 0xef */
+	IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH, IH 	/* 0xf0 - 0xff */
+};
+
+
 /** An entry in css_property_table. */
 struct css_property_entry {
 	const char name[25];
@@ -621,29 +645,32 @@ bool css_compare_selectors(const struct css_selector *n0,
  * Property parsers.
  */
 
-/* TODO: consider CSS_NODE_NUMBER whenever a value may be '0' */
-
 int parse_length(struct css_length * const length,
 		const struct css_node * const v, bool non_negative)
 {
 	css_unit u;
 	float value;
 	int num_length;
-	if (v->type == CSS_NODE_NUMBER && atof(v->data) == 0) {
+
+	if (v->type == CSS_NODE_NUMBER && fabs(atof(v->data)) < 0.0001) {
 		length->unit = CSS_UNIT_PX;
 		length->value = 0;
 		return 0;
 	}
-	if ((v->type != CSS_NODE_DIMENSION) && (v->type != CSS_NODE_NUMBER))
+
+	if (v->type != CSS_NODE_DIMENSION && v->type != CSS_NODE_NUMBER)
 		return 1;
+
 	num_length = strspn(v->data, "0123456789+-.");
+
 	if (v->type == CSS_NODE_DIMENSION) {
-		u = css_unit_parse(v->data + num_length, v->data_length - num_length);
+		u = css_unit_parse(v->data + num_length,
+				v->data_length - num_length);
 		if (u == CSS_UNIT_UNKNOWN) {
 			return 1;
 		}
 	} else {
-	  	u = CSS_UNIT_PX;
+		u = CSS_UNIT_PX;
 	}
 	value = atof(v->data);
 	if (non_negative && value < 0)
@@ -657,44 +684,36 @@ int parse_length(struct css_length * const length,
 colour named_colour(const char *name)
 {
 	struct css_colour_entry *col;
-	unsigned int r, g, b;
+	int length;
 
 	col = bsearch(name, css_colour_table,
 			sizeof css_colour_table / sizeof css_colour_table[0],
 			sizeof css_colour_table[0],
 			(int (*)(const void *, const void *)) strcasecmp);
-	if (col == 0) {
-		/* A common error is the omission of the '#' from the
-		 * start of a colour specified in #rrggbb format.
-		 * This attempts to detect and recover from this.
-		 */
-		if (strlen(name) == 6 &&
-		    sscanf(name, "%2x%2x%2x", &r, &g, &b) == 3) {
-			return (b << 16) | (g << 8) | r;
-		}
-		else
-			return TRANSPARENT;
-	}
-	return col->col;
+	if (col != 0)
+		return col->col;
+
+	/* A common error is the omission of the '#' from the
+	 * start of a colour specified in #rrggbb or #rgb format.
+	 * This attempts to detect and recover from this.
+	 */
+	length = strlen(name);
+	if ((length == 3) || (length == 6))
+		return hex_colour(name, length);
+	return CSS_COLOR_NONE;
 }
 
 
 colour parse_colour(const struct css_node * const v)
 {
 	colour c = CSS_COLOR_NONE;
-	unsigned int r, g, b;
 	struct css_colour_entry *col;
 	char colour_name[21];
 
 	switch (v->type) {
 		case CSS_NODE_HASH:
-			if (v->data_length == 4) {
-				if (sscanf(v->data + 1, "%1x%1x%1x", &r, &g, &b) == 3)
-					c = (b << 20) | (b << 16) | (g << 12) | (g << 8) | (r << 4) | r;
-			} else if (v->data_length == 7) {
-				if (sscanf(v->data + 1, "%2x%2x%2x", &r, &g, &b) == 3)
-					c = (b << 16) | (g << 8) | r;
-			}
+			if ((v->data_length == 4) || (v->data_length == 7))
+				c = hex_colour(v->data + 1, v->data_length - 1);
 			break;
 
 		case CSS_NODE_FUNCTION:
@@ -722,10 +741,47 @@ colour parse_colour(const struct css_node * const v)
 			break;
 	}
 
+	/* Hex colour values without a preceding # are invalid but it is a
+	 * common omission that other browsers cater for. */
+	if (c == CSS_COLOR_NONE && (v->type == CSS_NODE_DELIM ||
+			v->type == CSS_NODE_IDENT || v->type == CSS_NODE_NUMBER ||
+			v->type == CSS_NODE_DIMENSION)) {
+		if ((v->data_length == 3) || (v->data_length == 6))
+			return hex_colour(v->data, v->data_length);
+	}
 	return c;
 }
 
 
+/**
+ * Parse an RGB value in hexadecimal notation.
+ */
+colour hex_colour(const char *text, int length)
+{
+	colour c;
+
+	/* parse RGB */
+	if (length == 3) {
+		c = ascii_to_hex[(int)text[0]] | (ascii_to_hex[(int)text[1]] << 8) |
+				(ascii_to_hex[(int)text[2]] << 16);
+		if (c & (0xff << 24))
+			return CSS_COLOR_NONE;
+		return c | (c << 4);
+	}
+
+	/* parse RRGGBB */
+	if (length == 6) {
+		c = ascii_to_hex[(int)text[1]] | (ascii_to_hex[(int)text[0]] << 4) |
+				(ascii_to_hex[(int)text[3]] << 8) | (ascii_to_hex[(int)text[2]] << 12) |
+				(ascii_to_hex[(int)text[5]] << 16) | (ascii_to_hex[(int)text[4]] << 20);
+		if (c & (0xff << 24))
+			return CSS_COLOR_NONE;
+		return c;
+	}
+	return CSS_COLOR_NONE;
+}
+
+
 /**
  * Parse an RGB value in functional notation.
  */
@@ -857,12 +913,17 @@ void parse_background(struct css_style * const s,
 	struct css_background_position vert =
 			{ CSS_BACKGROUND_POSITION_PERCENT, { 0 } };
 	struct css_background_position horz2, vert2;
+	bool had_colour = false, had_image = false, had_repeat = false,
+	     had_attachment = false, had_position = false;
 
 	while (v) {
 		switch (v->type) {
 			case CSS_NODE_URI:
 			case CSS_NODE_STRING:
 				/* background-image */
+				if (had_image)
+					goto error;
+				had_image = true;
 				if (!css_background_image_parse(v, &bi2,
 						&bi_uri))
 					goto error;
@@ -874,6 +935,9 @@ void parse_background(struct css_style * const s,
 			case CSS_NODE_NUMBER:
 			case CSS_NODE_PERCENTAGE:
 				/* background-position */
+				if (had_position)
+					goto error;
+				had_position = true;
 				if (!css_background_position_parse(&v,
 						&horz2, &vert2))
 					goto error;
@@ -886,6 +950,9 @@ void parse_background(struct css_style * const s,
 				if (v->data_length == 4 &&
 						strncasecmp(v->data, "none",
 						4) == 0) {
+					if (had_image)
+						goto error;
+					had_image = true;
 					bi = CSS_BACKGROUND_IMAGE_NONE;
 					v = v->next;
 					break;
@@ -895,6 +962,9 @@ void parse_background(struct css_style * const s,
 				br2 = css_background_repeat_parse(v->data,
 						v->data_length);
 				if (br2 != CSS_BACKGROUND_REPEAT_UNKNOWN) {
+					if (had_repeat)
+						goto error;
+					had_repeat = true;
 					br = br2;
 					v = v->next;
 					break;
@@ -904,6 +974,9 @@ void parse_background(struct css_style * const s,
 				ba2 = css_background_attachment_parse(v->data,
 						v->data_length);
 				if (ba2 != CSS_BACKGROUND_ATTACHMENT_UNKNOWN) {
+					if (had_attachment)
+						goto error;
+					had_attachment = true;
 					ba = ba2;
 					v = v->next;
 					break;
@@ -912,6 +985,9 @@ void parse_background(struct css_style * const s,
 				/* background-position */
 				if (css_background_position_parse(&v,
 						&horz2, &vert2)) {
+					if (had_position)
+						goto error;
+					had_position = true;
 					horz = horz2;
 					vert = vert2;
 					break;
@@ -921,6 +997,9 @@ void parse_background(struct css_style * const s,
 			case CSS_NODE_HASH:
 			case CSS_NODE_FUNCTION:
 				/* background-color */
+				if (had_colour)
+					goto error;
+				had_colour = true;
 				c2 = parse_colour(v);
 				if (c2 != CSS_COLOR_NONE) {
 					c = c2;
@@ -1636,9 +1715,8 @@ void parse_clip(struct css_style * const s, const struct css_node * v)
 			for (i = 0; i != 4; i++) {
 				switch (t->type) {
 				case CSS_NODE_IDENT:
-					if (t->data_length == 4 &&
-							strncasecmp(t->data, "auto", 4) == 0) {
-						rect[i].rect = CSS_CLIP_AUTO;
+					if (t->data_length == 4 && strncasecmp(t->data, "auto", 4) == 0) {
+						rect[i].rect = CSS_CLIP_RECT_AUTO;
 					}
 					else
 						return;
@@ -2154,8 +2232,11 @@ void parse_height(struct css_style * const s, const struct css_node * const v)
 	if (v->type == CSS_NODE_IDENT && v->data_length == 4 &&
 			strncasecmp(v->data, "auto", 4) == 0)
 		s->height.height = CSS_HEIGHT_AUTO;
-	else if ((v->type == CSS_NODE_DIMENSION || v->type == CSS_NODE_NUMBER) &&
-			parse_length(&s->height.length, v, true) == 0)
+	else if (v->type == CSS_NODE_PERCENTAGE) {
+		s->height.height = CSS_HEIGHT_PERCENT;
+		s->height.value.percent = atof(v->data);
+	} else if ((v->type == CSS_NODE_DIMENSION || v->type == CSS_NODE_NUMBER) &&
+			parse_length(&s->height.value.length, v, true) == 0)
 		s->height.height = CSS_HEIGHT_LENGTH;
 }
 
