@@ -22,9 +22,15 @@
 #include <stdbool.h>
 
 extern bool netsurf_quit;
+extern bool netsurf_embedded;
 extern bool verbose_log;
 extern const char * const netsurf_version;
 extern const int netsurf_version_major;
 extern const int netsurf_version_minor;
 
+extern void netsurf_init(int argc, char** argv);
+extern void netsurf_exit(void);
+extern int netsurf_main(int argc, char** argv);
+extern int netsurf_main_loop(void);
+
 #endif
