@@ -17,51 +17,29 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <inttypes.h>
+#include <string.h>
 #include "gtk/gtk_window.h"
 #include "desktop/browser.h"
 #include "desktop/options.h"
 #include "desktop/textinput.h"
+#include "desktop/selection.h"
 #include "gtk/gtk_gui.h"
+#include "gtk/options.h"
 #include "gtk/gtk_scaffolding.h"
 #include "gtk/gtk_plotters.h"
 #include "gtk/gtk_schedule.h"
+#include "gtk/gtk_tabs.h"
 #undef NDEBUG
 #include "utils/log.h"
 #include "utils/utils.h"
 #include <gdk/gdkkeysyms.h>
 #include <assert.h>
 
-struct gui_window {
-        /* All gui_window objects have an ultimate scaffold */
-        nsgtk_scaffolding	*scaffold;
-        /* A gui_window is the rendering of a browser_window */
-        struct browser_window	*bw;
+struct gui_window *window_list = 0;	/**< first entry in win list*/
+int temp_open_background = -1;
 
-        /* These are the storage for the rendering */
-	int			caretx, carety, careth;
-	gui_pointer_shape	current_pointer;
-	int			last_x, last_y;
-
-        /* Within GTK, a gui_window is a scrolled window
-         * with a viewport inside
-         * with a gtkfixed in that
-         * with a drawing area in that
-         * The scrolled window is optional and only chosen
-         * for frames which need it. Otherwise we just use
-         * a viewport.
-         */
-        GtkScrolledWindow	*scrolledwindow;
-	GtkViewport		*viewport;
-        GtkFixed                *fixed;
-	GtkDrawingArea		*drawing_area;
-
-        /* Keep gui_windows in a list for cleanup later */
-        struct gui_window	*next, *prev;
-};
-
-static struct gui_window *window_list = 0;	/**< first entry in win list*/
-
-static wchar_t gdkkey_to_nskey(GdkEventKey *);
+static uint32_t gdkkey_to_nskey(GdkEventKey *);
 static void nsgtk_gui_window_attach_child(struct gui_window *parent,
                                           struct gui_window *child);
 /* Methods which apply only to a gui_window */
@@ -71,13 +49,22 @@ static gboolean nsgtk_window_motion_notify_event(GtkWidget *, GdkEventMotion *,
 						gpointer);
 static gboolean nsgtk_window_button_press_event(GtkWidget *, GdkEventButton *,
 						gpointer);
+static gboolean nsgtk_window_button_release_event(GtkWidget *, GdkEventButton *,
+						gpointer);
 static gboolean nsgtk_window_keypress_event(GtkWidget *, GdkEventKey *,
 						gpointer);
 static gboolean nsgtk_window_size_allocate_event(GtkWidget *, GtkAllocation *,
 						gpointer);
+
 /* Other useful bits */
 static void nsgtk_redraw_caret(struct gui_window *g);
 
+static GdkCursor *nsgtk_create_menu_cursor(void);
+
+struct browser_window *nsgtk_get_browser_window(struct gui_window *g)
+{
+	return g->bw;
+}
 
 nsgtk_scaffolding *nsgtk_get_scaffold(struct gui_window *g)
 {
@@ -96,7 +83,8 @@ float nsgtk_get_scale_for_gui(struct gui_window *g)
 
 /* Create a gui_window */
 struct gui_window *gui_create_browser_window(struct browser_window *bw,
-                                             struct browser_window *clone)
+                                             struct browser_window *clone,
+                                             bool new_tab)
 {
 	struct gui_window *g;		/**< what we're creating to return */
         GtkPolicyType scrollpolicy;
@@ -110,6 +98,12 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
         LOG(("Creating gui window %p for browser window %p", g, bw));
 
 	g->bw = bw;
+	g->mouse = malloc(sizeof(*g->mouse));
+       	if (!g->mouse) {
+		warn_user("NoMemory", 0);
+		return 0;
+	}
+	g->mouse->state = 0;
 	g->current_pointer = GUI_POINTER_DEFAULT;
 	if (clone != NULL)
 		bw->scale = clone->scale;
@@ -119,19 +113,21 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	g->careth = 0;
 
         /* Attach ourselves to the list (push_top) */
-        if (window_list)
-                window_list->prev = g;
-        g->next = window_list;
-        g->prev = NULL;
-        window_list = g;
+	if (window_list)
+		window_list->prev = g;
+		g->next = window_list;
+		g->prev = NULL;
+		window_list = g;
 
-        if (bw->parent != NULL) {
-                /* Find our parent's scaffolding */
-                g->scaffold = bw->parent->window->scaffold;
-        } else {
-                /* Now construct and attach a scaffold */
-                g->scaffold = nsgtk_new_scaffolding(g);
-        }
+	if (bw->parent != NULL)
+		/* Find our parent's scaffolding */
+		g->scaffold = bw->parent->window->scaffold;
+	else if (new_tab) {
+		g->scaffold = clone->window->scaffold;
+	}
+	else
+		/* Now construct and attach a scaffold */
+		g->scaffold = nsgtk_new_scaffolding(g);
 
         /* Construct our primary elements */
         g->fixed = GTK_FIXED(gtk_fixed_new());
@@ -139,32 +135,41 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
         gtk_fixed_put(g->fixed, GTK_WIDGET(g->drawing_area), 0, 0);
         gtk_container_set_border_width(GTK_CONTAINER(g->fixed), 0);
 
-        if (bw->parent != NULL ) {
-                g->scrolledwindow = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
-                gtk_scrolled_window_add_with_viewport(g->scrolledwindow,
-                                                      GTK_WIDGET(g->fixed));
-                gtk_scrolled_window_set_shadow_type(g->scrolledwindow,
-                                                    GTK_SHADOW_NONE);
-                g->viewport = GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(g->scrolledwindow)));
-                /* Attach ourselves into our parent at the right point */
-                nsgtk_gui_window_attach_child(bw->parent->window, g);
-        } else {
-                g->scrolledwindow = 0;
-                g->viewport = GTK_VIEWPORT(gtk_viewport_new(NULL, NULL)); /* Need to attach adjustments */
-                gtk_container_add(GTK_CONTAINER(g->viewport), GTK_WIDGET(g->fixed));
+	g->scrolledwindow = GTK_SCROLLED_WINDOW(gtk_scrolled_window_new(NULL, NULL));
+	g_object_set_data(G_OBJECT(g->scrolledwindow), "gui_window", g);
+	gtk_scrolled_window_add_with_viewport(g->scrolledwindow,
+					      GTK_WIDGET(g->fixed));
+	gtk_scrolled_window_set_shadow_type(g->scrolledwindow,
+					    GTK_SHADOW_NONE);
+	g->viewport = GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(g->scrolledwindow)));
+	g->tab = NULL;
 
-                /* Attach our viewport into the scaffold */
-                nsgtk_attach_toplevel_viewport(g->scaffold, g->viewport);
-        }
+	if (bw->parent != NULL)
+		/* Attach ourselves into our parent at the right point */
+		nsgtk_gui_window_attach_child(bw->parent->window, g);
+	else {	
+		/* Attach our viewport into the scaffold */
+		bool tempback = true;
+		switch (temp_open_background) {
+		case -1:
+			tempback = !(option_focus_new);
+			break;
+		case 0:
+			tempback = false;
+			break;
+		case 1:
+			tempback = true;
+			break;
+		}
+		nsgtk_tab_add(g, tempback);
+	}
 
-        gtk_container_set_border_width(GTK_CONTAINER(g->viewport), 0);
-        gtk_viewport_set_shadow_type(g->viewport, GTK_SHADOW_NONE);
-        if (g->scrolledwindow)
-                gtk_widget_show(GTK_WIDGET(g->scrolledwindow));
-        /* And enable visibility from our viewport down */
-        gtk_widget_show(GTK_WIDGET(g->viewport));
-        gtk_widget_show(GTK_WIDGET(g->fixed));
-        gtk_widget_show(GTK_WIDGET(g->drawing_area));
+	gtk_container_set_border_width(GTK_CONTAINER(g->viewport), 0);
+	gtk_viewport_set_shadow_type(g->viewport, GTK_SHADOW_NONE);
+	if (g->scrolledwindow)
+		gtk_widget_show(GTK_WIDGET(g->scrolledwindow));
+		/* And enable visibility from our viewport down */
+		gtk_widget_show_all(GTK_WIDGET(g->viewport));
 
         switch(bw->scrolling) {
         case SCROLLING_NO:
@@ -213,6 +218,7 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 				GDK_EXPOSURE_MASK |
 				GDK_LEAVE_NOTIFY_MASK |
 				GDK_BUTTON_PRESS_MASK |
+				GDK_BUTTON_RELEASE_MASK |
 				GDK_POINTER_MOTION_MASK |
 				GDK_KEY_PRESS_MASK |
 				GDK_KEY_RELEASE_MASK);
@@ -228,7 +234,9 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	CONNECT(g->drawing_area, "motion_notify_event",
 		nsgtk_window_motion_notify_event, g);
 	CONNECT(g->drawing_area, "button_press_event",
-	    	nsgtk_window_button_press_event, g);
+		nsgtk_window_button_press_event, g);
+	CONNECT(g->drawing_area, "button_release_event",
+	    nsgtk_window_button_release_event, g);
 	CONNECT(g->drawing_area, "key_press_event",
 		nsgtk_window_keypress_event, g);
 	CONNECT(g->viewport, "size_allocate",
@@ -307,6 +315,7 @@ gboolean nsgtk_window_expose_event(GtkWidget *widget,
 
 	plot = nsgtk_plotters;
 	nsgtk_plot_set_scale(g->bw->scale);
+	current_redraw_browser = g->bw;
 	content_redraw(c, 0, 0,
 			widget->allocation.width * scale,
 			widget->allocation.height * scale,
@@ -315,6 +324,7 @@ gboolean nsgtk_window_expose_event(GtkWidget *widget,
 			event->area.x + event->area.width,
 			event->area.y + event->area.height,
 			g->bw->scale, 0xFFFFFF);
+	current_redraw_browser = NULL;
 
 	if (g->careth != 0)
 		nsgtk_plot_caret(g->caretx, g->carety, g->careth);
@@ -331,9 +341,35 @@ gboolean nsgtk_window_motion_notify_event(GtkWidget *widget,
                                           GdkEventMotion *event, gpointer data)
 {
 	struct gui_window *g = data;
+	bool shift = event->state & GDK_SHIFT_MASK;
+	bool ctrl = event->state & GDK_CONTROL_MASK;
 
-	browser_window_mouse_track(g->bw, 0, event->x / g->bw->scale,
-                                   event->y / g->bw->scale);
+   	if (g->mouse->state & BROWSER_MOUSE_PRESS_1){
+		/* Start button 1 drag */
+		browser_window_mouse_click(g->bw, BROWSER_MOUSE_DRAG_1,
+				g->mouse->pressed_x, g->mouse->pressed_y);
+		/* Replace PRESS with HOLDING and declare drag in progress */
+		g->mouse->state ^= (BROWSER_MOUSE_PRESS_1 |
+				BROWSER_MOUSE_HOLDING_1);
+		g->mouse->state |= BROWSER_MOUSE_DRAG_ON;
+	}
+	else if (g->mouse->state & BROWSER_MOUSE_PRESS_2){
+		/* Start button 2 drag */
+		browser_window_mouse_click(g->bw, BROWSER_MOUSE_DRAG_2,
+				g->mouse->pressed_x, g->mouse->pressed_y);
+		/* Replace PRESS with HOLDING and declare drag in progress */
+		g->mouse->state ^= (BROWSER_MOUSE_PRESS_2 |
+				BROWSER_MOUSE_HOLDING_2);
+		g->mouse->state |= BROWSER_MOUSE_DRAG_ON;
+	}
+	/* Handle modifiers being removed */
+  	if (g->mouse->state & BROWSER_MOUSE_MOD_1 && !shift)
+  		g->mouse->state ^= BROWSER_MOUSE_MOD_1;
+ 	if (g->mouse->state & BROWSER_MOUSE_MOD_2 && !ctrl)
+  		g->mouse->state ^= BROWSER_MOUSE_MOD_2;
+
+	browser_window_mouse_track(g->bw, g->mouse->state,
+			event->x / g->bw->scale, event->y / g->bw->scale);
 
 	g->last_x = event->x;
 	g->last_y = event->y;
@@ -345,27 +381,76 @@ gboolean nsgtk_window_button_press_event(GtkWidget *widget,
                                          GdkEventButton *event, gpointer data)
 {
 	struct gui_window *g = data;
-	int button = BROWSER_MOUSE_CLICK_1;
+	GtkWindow *window = nsgtk_get_window_for_scaffold(g->scaffold);
 
-	if (event->button == 2) /* 2 == middle button on X */
-		button = BROWSER_MOUSE_CLICK_2;
+	gtk_window_set_focus(window, NULL);
+
+	g->mouse->pressed_x = event->x / g->bw->scale;
+	g->mouse->pressed_y = event->y / g->bw->scale;
 
 	if (event->button == 3) {
-		/* 3 == right button on X */
-		nsgtk_scaffolding_popup_menu(g->scaffold, event->button);
+		/** \todo
+		 * Firstly, MOUSE_PRESS_2 on GTK is a middle click, which doesn't
+		 * appear correct to me.  Secondly, right-clicks are not passed to
+		 * browser_window_mouse_click() at all, which also seems incorrect
+		 * since they should result in browser_window_remove_caret().
+		 *
+		 * I would surmise we need a MOUSE_PRESS_3, unless right-clicking is
+		 * supposed to be mapped to MOUSE_PRESS_2, but that doesn't appear
+		 * correct either.
+		 */
+		browser_window_remove_caret(g->bw);
+		nsgtk_scaffolding_popup_menu(g->scaffold, g->mouse->pressed_x, g->mouse->pressed_y);
 		return TRUE;
 	}
 
-	browser_window_mouse_click(g->bw, button,
-                                   event->x / g->bw->scale,
-				   event->y / g->bw->scale);
+	switch (event->button) {
+		case 1: g->mouse->state = BROWSER_MOUSE_PRESS_1; break;
+		case 2: g->mouse->state = BROWSER_MOUSE_PRESS_2; break;
+	}
+	/* Handle the modifiers too */
+	if (event->state & GDK_SHIFT_MASK)
+		g->mouse->state |= BROWSER_MOUSE_MOD_1;
+	if (event->state & GDK_CONTROL_MASK)
+		g->mouse->state |= BROWSER_MOUSE_MOD_2;
 
-        gtk_widget_grab_focus(widget);
+	browser_window_mouse_click(g->bw, g->mouse->state, g->mouse->pressed_x,
+			g->mouse->pressed_y);
 
 	return TRUE;
 }
 
-wchar_t gdkkey_to_nskey(GdkEventKey *key)
+gboolean nsgtk_window_button_release_event(GtkWidget *widget,
+                                         GdkEventButton *event, gpointer data)
+{
+	struct gui_window *g = data;
+	bool shift = event->state & GDK_SHIFT_MASK;
+	bool ctrl = event->state & GDK_CONTROL_MASK;
+
+	/* If the mouse state is PRESS then we are waiting for a release to emit
+	 * a click event, otherwise just reset the state to nothing*/
+	if (g->mouse->state & BROWSER_MOUSE_PRESS_1)
+		g->mouse->state ^= (BROWSER_MOUSE_PRESS_1 | BROWSER_MOUSE_CLICK_1);
+	else if (g->mouse->state & BROWSER_MOUSE_PRESS_2)
+		g->mouse->state ^= (BROWSER_MOUSE_PRESS_2 | BROWSER_MOUSE_CLICK_2);
+
+	/* Handle modifiers being removed */
+  	if (g->mouse->state & BROWSER_MOUSE_MOD_1 && !shift)
+  		g->mouse->state ^= BROWSER_MOUSE_MOD_1;
+ 	if (g->mouse->state & BROWSER_MOUSE_MOD_2 && !ctrl)
+  		g->mouse->state ^= BROWSER_MOUSE_MOD_2;
+
+	if (g->mouse->state & (BROWSER_MOUSE_CLICK_1|BROWSER_MOUSE_CLICK_2))
+		browser_window_mouse_click(g->bw, g->mouse->state, event->x / g->bw->scale,
+			event->y / g->bw->scale);
+	else
+		browser_window_mouse_drag_end(g->bw, 0, event->x, event->y);
+
+	g->mouse->state = 0;
+	return TRUE;
+}
+
+uint32_t gdkkey_to_nskey(GdkEventKey *key)
 {
         /* this function will need to become much more complex to support
          * everything that the RISC OS version does.  But this will do for
@@ -399,7 +484,8 @@ wchar_t gdkkey_to_nskey(GdkEventKey *key)
                 case GDK_Hyper_L:
                 case GDK_Hyper_R:               return 0;
 
-                default:                        return key->keyval;
+                default:                        return gdk_keyval_to_unicode(
+								key->keyval);
         }
 }
 
@@ -407,8 +493,7 @@ gboolean nsgtk_window_keypress_event(GtkWidget *widget, GdkEventKey *event,
 					gpointer data)
 {
 	struct gui_window *g = data;
-	wchar_t nskey = gdkkey_to_nskey(event);
-
+	uint32_t nskey = gdkkey_to_nskey(event);
 	if (browser_window_key_press(g->bw, nskey))
 		return TRUE;
 
@@ -516,8 +601,11 @@ gboolean nsgtk_window_size_allocate_event(GtkWidget *widget,
 
 void nsgtk_reflow_all_windows(void)
 {
-	for (struct gui_window *g = window_list; g; g = g->next)
+	for (struct gui_window *g = window_list; g; g = g->next) {
+                nsgtk_tab_options_changed(GTK_WIDGET(
+							nsgtk_scaffolding_get_notebook(g)));
 		g->bw->reformat_pending = true;
+        }
 
 	browser_reformat_pending = true;
 }
@@ -538,7 +626,7 @@ void nsgtk_window_process_reformats(void)
 			continue;
 		g->bw->reformat_pending = false;
 		browser_window_reformat(g->bw,
-				widget->allocation.width - 2,
+				widget->allocation.width,
 				widget->allocation.height);
 	}
 }
@@ -564,14 +652,14 @@ void gui_window_destroy(struct gui_window *g)
 	assert(g != NULL);
 	assert(g->bw != NULL);
 	LOG(("     Scaffolding: %p", g->scaffold));
-        LOG(("     Window name: %s", g->bw->name));
+	LOG(("     Window name: %s", g->bw->name));
 
-        /* If we're a top-level gui_window, destroy our scaffold */
-        if (g->scrolledwindow == NULL) {
-	  	gtk_widget_destroy(GTK_WIDGET(g->viewport));
-                nsgtk_scaffolding_destroy(g->scaffold);
+	/* If we're a top-level gui_window, destroy our scaffold */
+	if (g->scrolledwindow == NULL) {
+		gtk_widget_destroy(GTK_WIDGET(g->viewport));
+		nsgtk_scaffolding_destroy(g->scaffold);
 	} else {
-	  	gtk_widget_destroy(GTK_WIDGET(g->scrolledwindow));
+		gtk_widget_destroy(GTK_WIDGET(g->scrolledwindow));
 	}
 
 	free(g);
@@ -629,12 +717,25 @@ void gui_window_set_scroll(struct gui_window *g, int sx, int sy)
 {
         GtkAdjustment *vadj = gtk_viewport_get_vadjustment(g->viewport);
         GtkAdjustment *hadj = gtk_viewport_get_hadjustment(g->viewport);
+        gdouble vlower, vpage, vupper, hlower, hpage, hupper, x = (double)sx, y = (double)sy;
 
         assert(vadj);
         assert(hadj);
 
-        gtk_adjustment_set_value(vadj, (double)sy);
-        gtk_adjustment_set_value(hadj, (double)sx);
+        g_object_get(vadj, "page-size", &vpage, "lower", &vlower, "upper", &vupper, NULL);
+        g_object_get(hadj, "page-size", &hpage, "lower", &hlower, "upper", &hupper, NULL);
+
+        if (x < hlower)
+                x = hlower;
+        if (x > (hupper - hpage))
+                x = hupper - hpage;
+        if (y < vlower)
+                y = vlower;
+        if (y > (vupper - vpage))
+                y = vupper - vpage;
+
+        gtk_adjustment_set_value(vadj, y);
+        gtk_adjustment_set_value(hadj, x);
 }
 
 
@@ -663,6 +764,39 @@ void gui_window_update_extent(struct gui_window *g)
 
 }
 
+static GdkCursor *nsgtk_create_menu_cursor(void)
+{
+	static char menu_cursor_bits[] = {
+	0x00, 0x00, 0x80, 0x7F, 0x88, 0x40, 0x9E, 0x5E, 0x88, 0x40, 0x80, 0x56,
+ 	0x80, 0x40, 0x80, 0x5A, 0x80, 0x40, 0x80, 0x5E, 0x80, 0x40, 0x80, 0x56,
+	0x80, 0x40, 0x80, 0x7F, 0x00, 0x00, 0x00, 0x00, };
+
+	static char menu_cursor_mask_bits[] = {
+	0xC0, 0xFF, 0xC8, 0xFF, 0xDF, 0xFF, 0xFF, 0xFF, 0xDF, 0xFF, 0xC8, 0xFF,
+	0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0xFF,
+	0xC0, 0xFF, 0xC0, 0xFF, 0xC0, 0xFF, 0x00, 0x00, };
+
+	static GdkCursor *r;
+	static GdkColor fg = { 0, 0, 0, 0 };
+	static GdkColor bg = { 0, 65535, 65535, 65535 };
+
+	GdkPixmap *source, *mask;
+
+	if (r != NULL)
+		return r;
+
+	source = gdk_bitmap_create_from_data(NULL, menu_cursor_bits,
+						16, 16);
+	mask = gdk_bitmap_create_from_data (NULL, menu_cursor_mask_bits,
+						16, 16);
+
+	r = gdk_cursor_new_from_pixmap(source, mask, &fg, &bg, 8, 8);
+	gdk_pixmap_unref(source);
+	gdk_pixmap_unref(mask);
+
+	return r;
+}
+
 void gui_window_set_pointer(struct gui_window *g, gui_pointer_shape shape)
 {
         GdkCursor *cursor = NULL;
@@ -676,7 +810,7 @@ void gui_window_set_pointer(struct gui_window *g, gui_pointer_shape shape)
 
 	switch (shape) {
         case GUI_POINTER_POINT:
-                cursortype = GDK_HAND1;
+                cursortype = GDK_HAND2;
                 break;
         case GUI_POINTER_CARET:
                 cursortype = GDK_XTERM;
@@ -718,7 +852,8 @@ void gui_window_set_pointer(struct gui_window *g, gui_pointer_shape shape)
                 cursortype = GDK_QUESTION_ARROW;
                 break;
         case GUI_POINTER_MENU:
-                cursortype = GDK_RIGHTBUTTON;
+                cursor = nsgtk_create_menu_cursor();
+                nullcursor = true;
                 break;
         case GUI_POINTER_PROGRESS:
                 /* In reality, this needs to be the funky left_ptr_watch
@@ -756,7 +891,7 @@ void gui_window_place_caret(struct gui_window *g, int x, int y, int height)
 
 	g->caretx = x;
 	g->carety = y + 1;
-	g->careth = height;
+	g->careth = height - 2;
 
 	nsgtk_redraw_caret(g);
 
@@ -803,38 +938,6 @@ void gui_drag_save_selection(struct selection *s, struct gui_window *g)
 
 }
 
-void gui_start_selection(struct gui_window *g)
-{
-
-}
-
-void gui_paste_from_clipboard(struct gui_window *g, int x, int y)
-{
-
-}
-
-bool gui_empty_clipboard(void)
-{
-	return true;
-}
-
-bool gui_add_to_clipboard(const char *text, size_t length, bool space)
-{
-  	return true;
-}
-
-bool gui_commit_clipboard(void)
-{
-	return true;
-}
-
-
-bool gui_copy_to_clipboard(struct selection *s)
-{
-	return true;
-}
-
-
 void gui_window_get_dimensions(struct gui_window *g, int *width, int *height,
                                bool scaled)
 {
@@ -845,6 +948,8 @@ void gui_window_get_dimensions(struct gui_window *g, int *width, int *height,
 		*width /= g->bw->scale;
 		*height /= g->bw->scale;
 	}
+	LOG(("\tWINDOW WIDTH:  %i\n", *width));
+	LOG(("\tWINDOW HEIGHT: %i\n", *height));
 }
 
 bool gui_window_frame_resize_start(struct gui_window *g)
