@@ -1,5 +1,5 @@
 /*
- * Copyright 2005 James Bursa <bursa@users.sourceforge.net>
+ * Copyright 2005-2007 James Bursa <bursa@users.sourceforge.net>
  * Copyright 2003 Phil Mellor <monkeyson@users.sourceforge.net>
  * Copyright 2005 John M Bell <jmb202@ecs.soton.ac.uk>
  *
@@ -24,20 +24,27 @@
 
 #include <assert.h>
 #include <stdbool.h>
+#include <stdio.h>
 #include <string.h>
 #include "content/content.h"
 #include "css/css.h"
+#include "desktop/options.h"
 #include "render/box.h"
 #include "render/form.h"
 #include "utils/log.h"
 #include "utils/talloc.h"
 
-
-static bool box_contains_point(struct box *box, int x, int y);
+static bool box_contains_point(struct box *box, int x, int y, bool *physically);
 
 #define box_is_float(box) (box->type == BOX_FLOAT_LEFT || \
 		box->type == BOX_FLOAT_RIGHT)
 
+typedef struct box_duplicate_llist box_duplicate_llist;
+struct box_duplicate_llist {
+	struct box_duplicate_llist *prev;
+	struct box *box;
+};
+static struct box_duplicate_llist *box_duplicate_last = NULL;
 
 /**
  * Create a box tree node.
@@ -86,6 +93,8 @@ struct box * box_create(struct css_style *style,
 	box->columns = 1;
 	box->rows = 1;
 	box->start_column = 0;
+	box->inline_new_line = false;
+	box->printed = false;
 	box->next = NULL;
 	box->prev = NULL;
 	box->children = NULL;
@@ -94,6 +103,7 @@ struct box * box_create(struct css_style *style,
 	box->fallback = NULL;
 	box->inline_end = NULL;
 	box->float_children = NULL;
+	box->float_container = NULL;
 	box->next_float = NULL;
 	box->list_marker = NULL;
 	box->col = NULL;
@@ -107,7 +117,6 @@ struct box * box_create(struct css_style *style,
 	return box;
 }
 
-
 /**
  * Add a child to a box tree node.
  *
@@ -293,12 +302,13 @@ void box_bounds(struct box *box, struct rect *r)
  * \endcode
  */
 
-struct box *box_at_point(struct box *box, int x, int y,
+struct box *box_at_point(struct box *box, const int x, const int y,
 		int *box_x, int *box_y,
 		struct content **content)
 {
 	int bx = *box_x, by = *box_y;
 	struct box *child, *sibling;
+	bool physically;
 
 	assert(box);
 
@@ -315,10 +325,15 @@ struct box *box_at_point(struct box *box, int x, int y,
 
 	/* consider floats second, since they will often overlap other boxes */
 	for (child = box->float_children; child; child = child->next_float) {
-		if (box_contains_point(child, x - bx, y - by)) {
+		if (box_contains_point(child, x - bx, y - by, &physically)) {
 			*box_x = bx + child->x - child->scroll_x;
 			*box_y = by + child->y - child->scroll_y;
-			return child;
+
+			if (physically)
+				return child;
+			else
+				return box_at_point(child, x, y, box_x, box_y,
+						content);
 		}
 	}
 
@@ -327,10 +342,25 @@ non_float_children:
 	for (child = box->children; child; child = child->next) {
 		if (box_is_float(child))
 			continue;
-		if (box_contains_point(child, x - bx, y - by)) {
+		if (box_contains_point(child, x - bx, y - by, &physically)) {
 			*box_x = bx + child->x - child->scroll_x;
 			*box_y = by + child->y - child->scroll_y;
-			return child;
+
+			if (physically)
+				return child;
+			else
+				return box_at_point(child, x, y, box_x, box_y,
+						content);
+		}
+	}
+
+	/* marker boxes */
+	if (box->list_marker) {
+		if (box_contains_point(box->list_marker, x - bx, y - by,
+				&physically)) {
+			*box_x = bx + box->list_marker->x;
+			*box_y = by + box->list_marker->y;
+			return box->list_marker;
 		}
 	}
 
@@ -343,12 +373,19 @@ siblings:
 			for (sibling = box->next_float; sibling;
 					sibling = sibling->next_float) {
 				if (box_contains_point(sibling,
-						x - bx, y - by)) {
+						x - bx, y - by, &physically)) {
 					*box_x = bx + sibling->x -
 							sibling->scroll_x;
 					*box_y = by + sibling->y -
 							sibling->scroll_y;
-					return sibling;
+
+					if (physically)
+						return sibling;
+					else
+						return box_at_point(sibling,
+								x, y,
+								box_x, box_y,
+								content);
 				}
 			}
 			/* ascend to float's parent */
@@ -365,13 +402,20 @@ siblings:
 					sibling = sibling->next) {
 				if (box_is_float(sibling))
 					continue;
-				if (box_contains_point(sibling,
-						x - bx, y - by)) {
+				if (box_contains_point(sibling, x - bx, y - by,
+						&physically)) {
 					*box_x = bx + sibling->x -
 							sibling->scroll_x;
 					*box_y = by + sibling->y -
 							sibling->scroll_y;
-					return sibling;
+
+					if (physically)
+						return sibling;
+					else
+						return box_at_point(sibling,
+								x, y,
+								box_x, box_y,
+								content);
 				}
 			}
 			box = box->parent;
@@ -385,31 +429,56 @@ siblings:
 /**
  * Determine if a point lies within a box.
  *
- * \param  box  box to consider
- * \param  x    coordinate relative to box parent
- * \param  y    coordinate relative to box parent
+ * \param  box		box to consider
+ * \param  x		coordinate relative to box parent
+ * \param  y		coordinate relative to box parent
+ * \param  physically	if function returning true, physically is set true if
+ *			point is within the box's physical dimensions and false
+ *			if the point is not within the box's physical dimensions
+ *			but is in the area defined by the box's descendants.
+ *			if function returning false, physically is undefined.
  * \return  true if the point is within the box or a descendant box
  *
  * This is a helper function for box_at_point().
  */
 
-bool box_contains_point(struct box *box, int x, int y)
+bool box_contains_point(struct box *box, int x, int y, bool *physically)
 {
-	if ((box->style && box->style->overflow != CSS_OVERFLOW_VISIBLE) ||
-			box->inline_end) {
-		if (box->x <= x + box->border[LEFT] &&
-				x < box->x + box->padding[LEFT] + box->width +
-				box->border[RIGHT] + box->padding[RIGHT] &&
-				box->y <= y + box->border[TOP] &&
-				y < box->y + box->padding[TOP] + box->height +
-				box->border[BOTTOM] + box->padding[BOTTOM])
-			return true;
-	} else {
+	if (box->x <= x + box->border[LEFT] &&
+			x < box->x + box->padding[LEFT] + box->width +
+			box->border[RIGHT] + box->padding[RIGHT] &&
+			box->y <= y + box->border[TOP] &&
+			y < box->y + box->padding[TOP] + box->height +
+			box->border[BOTTOM] + box->padding[BOTTOM]) {
+		*physically = true;
+		return true;
+	}
+	if (box->list_marker && box->list_marker->x <= x +
+			box->list_marker->border[LEFT] &&
+			x < box->list_marker->x +
+			box->list_marker->padding[LEFT] +
+			box->list_marker->width +
+			box->list_marker->border[RIGHT] +
+			box->list_marker->padding[RIGHT] &&
+			box->list_marker->y <= y +
+			box->list_marker->border[TOP] &&
+			y < box->list_marker->y +
+			box->list_marker->padding[TOP] +
+			box->list_marker->height +
+			box->list_marker->border[BOTTOM] +
+			box->list_marker->padding[BOTTOM]) {
+		*physically = true;
+		return true;
+	}
+	if ((box->style && box->style->overflow == CSS_OVERFLOW_VISIBLE) ||
+			!box->style) {
 		if (box->x + box->descendant_x0 <= x &&
 				x < box->x + box->descendant_x1 &&
 				box->y + box->descendant_y0 <= y &&
-				y < box->y + box->descendant_y1)
+				y < box->y + box->descendant_y1) {
+			*physically = false;
 			return true;
+		}
 	}
 	return false;
 }
@@ -445,6 +514,36 @@ struct box *box_object_at_point(struct content *c, int x, int y)
 }
 
 
+/**
+ * Find the box containing an href at the given coordinates, if any.
+ *
+ * \param  c  content to search, must have type CONTENT_HTML
+ * \param  x  coordinates in document units
+ * \param  y  coordinates in document units
+ */
+
+struct box *box_href_at_point(struct content *c, int x, int y)
+{
+	struct box *box = c->data.html.layout;
+	int box_x = 0, box_y = 0;
+	struct content *content = c;
+	struct box *href_box = 0;
+
+	assert(c->type == CONTENT_HTML);
+
+	while ((box = box_at_point(box, x, y, &box_x, &box_y, &content))) {
+		if (box->style &&
+				box->style->visibility == CSS_VISIBILITY_HIDDEN)
+			continue;
+
+		if (box->href)
+			href_box = box;
+	}
+
+	return href_box;
+}
+
+
 /**
  * Find a box based upon its id attribute.
  *
@@ -498,110 +597,381 @@ bool box_visible(struct box *box)
 
 
 /**
- * Print a box tree to stderr.
+ * Print a box tree to a file.
  */
 
-void box_dump(struct box *box, unsigned int depth)
+void box_dump(FILE *stream, struct box *box, unsigned int depth)
 {
 	unsigned int i;
 	struct box *c, *prev;
 
 	for (i = 0; i != depth; i++)
-		fprintf(stderr, "  ");
+		fprintf(stream, "  ");
 
-	fprintf(stderr, "%p ", box);
-	fprintf(stderr, "x%i y%i w%i h%i ", box->x, box->y,
+	fprintf(stream, "%p ", box);
+	fprintf(stream, "x%i y%i w%i h%i ", box->x, box->y,
 			box->width, box->height);
 	if (box->max_width != UNKNOWN_MAX_WIDTH)
-		fprintf(stderr, "min%i max%i ", box->min_width, box->max_width);
-	fprintf(stderr, "(%i %i %i %i) ",
+		fprintf(stream, "min%i max%i ", box->min_width, box->max_width);
+	fprintf(stream, "(%i %i %i %i) ",
 			box->descendant_x0, box->descendant_y0,
 			box->descendant_x1, box->descendant_y1);
 
+	fprintf(stream, "m(%i %i %i %i) ",
+			box->margin[TOP], box->margin[LEFT],
+			box->margin[BOTTOM], box->margin[RIGHT]);
+
 	switch (box->type) {
-	case BOX_BLOCK:            fprintf(stderr, "BLOCK "); break;
-	case BOX_INLINE_CONTAINER: fprintf(stderr, "INLINE_CONTAINER "); break;
-	case BOX_INLINE:           fprintf(stderr, "INLINE "); break;
-	case BOX_INLINE_END:       fprintf(stderr, "INLINE_END "); break;
-	case BOX_INLINE_BLOCK:     fprintf(stderr, "INLINE_BLOCK "); break;
-	case BOX_TABLE:            fprintf(stderr, "TABLE [columns %i] ",
+	case BOX_BLOCK:            fprintf(stream, "BLOCK "); break;
+	case BOX_INLINE_CONTAINER: fprintf(stream, "INLINE_CONTAINER "); break;
+	case BOX_INLINE:           fprintf(stream, "INLINE "); break;
+	case BOX_INLINE_END:       fprintf(stream, "INLINE_END "); break;
+	case BOX_INLINE_BLOCK:     fprintf(stream, "INLINE_BLOCK "); break;
+	case BOX_TABLE:            fprintf(stream, "TABLE [columns %i] ",
 					   box->columns); break;
-	case BOX_TABLE_ROW:        fprintf(stderr, "TABLE_ROW "); break;
-	case BOX_TABLE_CELL:       fprintf(stderr, "TABLE_CELL [columns %i, "
+	case BOX_TABLE_ROW:        fprintf(stream, "TABLE_ROW "); break;
+	case BOX_TABLE_CELL:       fprintf(stream, "TABLE_CELL [columns %i, "
 					   "start %i, rows %i] ", box->columns,
 					   box->start_column, box->rows); break;
-	case BOX_TABLE_ROW_GROUP:  fprintf(stderr, "TABLE_ROW_GROUP "); break;
-	case BOX_FLOAT_LEFT:       fprintf(stderr, "FLOAT_LEFT "); break;
-	case BOX_FLOAT_RIGHT:      fprintf(stderr, "FLOAT_RIGHT "); break;
-	case BOX_BR:               fprintf(stderr, "BR "); break;
-	case BOX_TEXT:             fprintf(stderr, "TEXT "); break;
-	default:                   fprintf(stderr, "Unknown box type ");
+	case BOX_TABLE_ROW_GROUP:  fprintf(stream, "TABLE_ROW_GROUP "); break;
+	case BOX_FLOAT_LEFT:       fprintf(stream, "FLOAT_LEFT "); break;
+	case BOX_FLOAT_RIGHT:      fprintf(stream, "FLOAT_RIGHT "); break;
+	case BOX_BR:               fprintf(stream, "BR "); break;
+	case BOX_TEXT:             fprintf(stream, "TEXT "); break;
+	default:                   fprintf(stream, "Unknown box type ");
 	}
 
 	if (box->text)
-		fprintf(stderr, "%li '%.*s' ", (unsigned long) box->byte_offset,
+		fprintf(stream, "%li '%.*s' ", (unsigned long) box->byte_offset,
 				(int) box->length, box->text);
 	if (box->space)
-		fprintf(stderr, "space ");
+		fprintf(stream, "space ");
 	if (box->object)
-		fprintf(stderr, "(object '%s') ", box->object->url);
+		fprintf(stream, "(object '%s') ", box->object->url);
+	if (box->gadget)
+		fprintf(stream, "(gadget) ");
 	if (box->style)
-		css_dump_style(box->style);
+		css_dump_style(stream, box->style);
 	if (box->href)
-		fprintf(stderr, " -> '%s'", box->href);
+		fprintf(stream, " -> '%s'", box->href);
 	if (box->target)
-		fprintf(stderr, " |%s|", box->target);
+		fprintf(stream, " |%s|", box->target);
 	if (box->title)
-		fprintf(stderr, " [%s]", box->title);
+		fprintf(stream, " [%s]", box->title);
 	if (box->id)
-		fprintf(stderr, " <%s>", box->id);
+		fprintf(stream, " <%s>", box->id);
 	if (box->type == BOX_INLINE || box->type == BOX_INLINE_END)
-		fprintf(stderr, " inline_end %p", box->inline_end);
+		fprintf(stream, " inline_end %p", box->inline_end);
 	if (box->float_children)
-		fprintf(stderr, " float_children %p", box->float_children);
+		fprintf(stream, " float_children %p", box->float_children);
 	if (box->next_float)
-		fprintf(stderr, " next_float %p", box->next_float);
+		fprintf(stream, " next_float %p", box->next_float);
 	if (box->col) {
-		fprintf(stderr, " (columns");
+		fprintf(stream, " (columns");
 		for (i = 0; i != box->columns; i++)
-			fprintf(stderr, " (%s %i %i %i)",
+			fprintf(stream, " (%s %s %i %i %i)",
 					((const char *[]) {"UNKNOWN", "FIXED",
 					"AUTO", "PERCENT", "RELATIVE"})
 					[box->col[i].type],
+					((const char *[]) {"normal",
+					"positioned"})
+					[box->col[i].positioned],
 					box->col[i].width,
 					box->col[i].min, box->col[i].max);
-		fprintf(stderr, ")");
+		fprintf(stream, ")");
 	}
-	fprintf(stderr, "\n");
+	fprintf(stream, "\n");
 
 	if (box->list_marker) {
 		for (i = 0; i != depth; i++)
-			fprintf(stderr, "  ");
-		fprintf(stderr, "list_marker:\n");
-		box_dump(box->list_marker, depth + 1);
+			fprintf(stream, "  ");
+		fprintf(stream, "list_marker:\n");
+		box_dump(stream, box->list_marker, depth + 1);
 	}
 
 	for (c = box->children; c && c->next; c = c->next)
 		;
 	if (box->last != c)
-		fprintf(stderr, "warning: box->last %p (should be %p) "
+		fprintf(stream, "warning: box->last %p (should be %p) "
 				"(box %p)\n", box->last, c, box);
 	for (prev = 0, c = box->children; c; prev = c, c = c->next) {
 		if (c->parent != box)
-			fprintf(stderr, "warning: box->parent %p (should be "
+			fprintf(stream, "warning: box->parent %p (should be "
 					"%p) (box on next line)\n",
 					c->parent, box);
 		if (c->prev != prev)
-			fprintf(stderr, "warning: box->prev %p (should be "
+			fprintf(stream, "warning: box->prev %p (should be "
 					"%p) (box on next line)\n",
 					c->prev, prev);
-		box_dump(c, depth + 1);
+		box_dump(stream, c, depth + 1);
 	}
 	if (box->fallback) {
 		for (i = 0; i != depth; i++)
-			fprintf(stderr, "  ");
-		fprintf(stderr, "fallback:\n");
+			fprintf(stream, "  ");
+		fprintf(stream, "fallback:\n");
 		for (c = box->fallback; c; c = c->next)
-			box_dump(c, depth + 1);
+			box_dump(stream, c, depth + 1);
+	}
+}
+
+/* Box tree duplication below
+*/
+
+/** structure for translating addresses in the box tree */
+struct box_dict_element{
+	struct box *old, *new;
+};
+
+static bool box_duplicate_main_tree(struct box *box, struct content *c,
+		int *count);
+static void box_duplicate_create_dict(struct box *old_box, struct box *new_box,
+		struct box_dict_element **dict);
+static void box_duplicate_update( struct box *box,
+		struct box_dict_element *dict,
+		int n);
+
+static int box_compare_dict_elements(const struct box_dict_element *a,
+		const struct box_dict_element *b);
+
+int box_compare_dict_elements(const struct box_dict_element *a,
+		const struct box_dict_element *b)
+{
+	return (a->old < b->old) ? -1 : ((a->old > b->old) ? 1 : 0);
+}
+
+/** Duplication of a box tree. We assume that all the content is fetched,
+fallbacks have been applied where necessary and we reuse a lot of content
+like strings, fetched objects etc - just replicating all we need to create
+two different layouts.
+\return true on success, false on lack of memory
+*/
+struct box* box_duplicate_tree(struct box *root, struct content *c)
+{
+	struct box *new_root;/**< Root of the new box tree*/
+	int box_number = 0;
+	struct box_dict_element *box_dict, *box_dict_end;
+
+	box_duplicate_last = NULL;
+
+	/* 1. Duplicate parent - children structure, list_markers*/
+	new_root = talloc_memdup(c, root, sizeof (struct box));
+	if (!box_duplicate_main_tree(new_root, c, &box_number))
+		return NULL;
+
+	/* 2. Create address translation dictionary*/
+	/*TODO: dont save unnecessary addresses*/
+
+	box_dict_end = box_dict = malloc(box_number *
+			sizeof(struct box_dict_element));
+
+	if (box_dict == NULL)
+		return NULL;
+	box_duplicate_create_dict(root, new_root, &box_dict_end);
+
+	assert((box_dict_end - box_dict) == box_number);
+
+	/*3. Sort it*/
+
+	qsort(box_dict, (box_dict_end - box_dict), sizeof(struct box_dict_element),
+	      (int (*)(const void *, const void *))box_compare_dict_elements);
+
+	/* 4. Update inline_end and float_children pointers */
+
+	box_duplicate_update(new_root, box_dict, (box_dict_end - box_dict));
+
+	free(box_dict);
+
+	return new_root;
+}
+
+/**
+ * Recursively duplicates children of an element, and also if present - its
+ * list_marker, style and text.
+ * \param box Current box to duplicate its children
+ * \param c talloc memory pool
+ * \param count number of boxes seen so far
+ * \return true if successful, false otherwise (lack of memory)
+*/
+bool box_duplicate_main_tree(struct box *box, struct content *c, int *count)
+{
+	struct box *b, *prev;
+
+	prev = NULL;
+
+	for (b = box->children; b; b = b->next) {
+		struct box *copy;
+
+		/*Copy child*/
+		copy = talloc_memdup(c, b, sizeof (struct box));
+		if (copy == NULL)
+			return false;
+
+		copy->parent = box;
+
+		if (prev != NULL)
+			prev->next = copy;
+		else
+			box->children = copy;
+
+		if (copy->type == BOX_INLINE) {
+			struct box_duplicate_llist *temp;
+
+			temp = malloc(sizeof(struct box_duplicate_llist));
+			if (temp == NULL)
+				return false;
+			temp->prev = box_duplicate_last;
+			temp->box = copy;
+			box_duplicate_last = temp;
+		}
+		else if (copy->type == BOX_INLINE_END) {
+			struct box_duplicate_llist *temp;
+
+			box_duplicate_last->box->inline_end = copy;
+			copy->inline_end = box_duplicate_last->box;
+
+			temp = box_duplicate_last;
+			box_duplicate_last = temp->prev;
+			free(temp);
+		}
+
+		/* Recursively visit child */
+		if (!box_duplicate_main_tree(copy, c, count))
+			return false;
+
+		prev = copy;
+	}
+
+	box->last = prev;
+
+	if (box->object && option_suppress_images && (
+#ifdef WITH_JPEG
+			box->object->type == CONTENT_JPEG ||
+#endif
+#ifdef WITH_GIF
+			box->object->type == CONTENT_GIF ||
+#endif
+#ifdef WITH_BMP
+			box->object->type ==  CONTENT_BMP ||
+			box->object->type == CONTENT_ICO ||
+#endif
+#if defined(WITH_MNG) || defined(WITH_PNG)
+			box->object->type == CONTENT_PNG ||
+#endif
+#ifdef WITH_MNG
+			box->object->type == CONTENT_JNG ||
+			box->object->type == CONTENT_MNG ||
+#endif
+#if defined(WITH_SPRITE) || defined(WITH_NSSPRITE)
+			box->object->type == CONTENT_SPRITE ||
+#endif
+#ifdef WITH_DRAW
+			box->object->type == CONTENT_DRAW ||
+#endif
+#ifdef WITH_PLUGIN
+			box->object->type == CONTENT_PLUGIN ||
+#endif
+			box->object->type == CONTENT_DIRECTORY ||
+#ifdef WITH_THEME_INSTALL
+			box->object->type == CONTENT_THEME ||
+#endif
+#ifdef WITH_ARTWORKS
+			box->object->type == CONTENT_ARTWORKS ||
+#endif
+#if defined(WITH_NS_SVG) || defined(WITH_RSVG)
+			box->object->type == CONTENT_SVG ||
+#endif
+			false))
+		box->object = NULL;
+
+	if (box->list_marker) {
+		box->list_marker = talloc_memdup(c, box->list_marker,
+				sizeof *box->list_marker);
+		if (box->list_marker == NULL)
+			return false;
+		box->list_marker->parent = box;
+	}
+
+	if (box->text) {
+		box->text = talloc_memdup(c, box->text, box->length);
+		if (box->text == NULL)
+			return false;
+	}
+
+	if (box->style) {
+		box->style = talloc_memdup(c, box->style, sizeof *box->style);
+		if (box->style == NULL)
+			return false;
+	}
+
+	/*Make layout calculate the size of this element later
+	(might change because of font change etc.) */
+	box->width = UNKNOWN_WIDTH;
+	box->min_width = 0;
+	box->max_width = UNKNOWN_MAX_WIDTH;
+
+	(*count)++;
+
+	return true;
+}
+
+/**
+ * Recursively creates a dictionary of addresses - binding the address of a box
+ * with its copy.
+ * \param old_box original box
+ * \param new_box copy of the original box
+ * \param dict pointer to a pointer to the current position in the dictionary
+ */
+void box_duplicate_create_dict(struct box *old_box, struct box *new_box,
+		struct box_dict_element **dict)
+{
+	/**Children of the old and new boxes*/
+	struct box *b_old, *b_new;
+
+	for (b_old = old_box->children, b_new = new_box->children;
+	     b_old != NULL && b_new != NULL;
+	     b_old = b_old->next, b_new = b_new->next)
+		box_duplicate_create_dict(b_old, b_new, dict);
+
+	/*The new tree should be a exact copy*/
+	assert(b_old == NULL && b_new == NULL);
+
+	(*dict)->old = old_box;
+	(*dict)->new = new_box;
+	(*dict)++;
+}
+
+/**
+ * Recursively updates pointers in box tree.
+ * \param box current box in the new box tree
+ * \param box_dict box pointers dictionary
+ * \param n number of memory addresses in the dictionary
+ */
+void box_duplicate_update(struct box *box,
+		struct box_dict_element *box_dict,
+  		int n)
+{
+	struct box_dict_element *box_dict_element;
+	struct box *b;
+	struct box_dict_element element;
+
+	for (b = box->children; b; b = b->next)
+		box_duplicate_update(b, box_dict, n);
+
+	if (box->float_children) {
+		element.old = box->float_children;
+		box_dict_element = bsearch(&element,
+				box_dict, n,
+				sizeof(struct box_dict_element),
+				(int (*)(const void *, const void *))box_compare_dict_elements);
+		box->float_children = box_dict_element->new;
+	}
+
+	if (box->next_float) {
+		element.old = box->next_float;
+		box_dict_element = bsearch(&element,
+				box_dict, n,
+				sizeof(struct box_dict_element),
+				(int (*)(const void *, const void *))box_compare_dict_elements);
+		box->next_float = box_dict_element->new;
 	}
 }
