@@ -28,9 +28,7 @@
 #include "css/css.h"
 #include "render/box.h"
 #include "render/table.h"
-#ifdef riscos
 #include "desktop/gui.h"
-#endif
 #define NDEBUG
 #include "utils/log.h"
 #include "utils/talloc.h"
@@ -141,7 +139,7 @@ bool box_normalise_block(struct box *block, struct content *c)
 			style = talloc_memdup(c, block->style, sizeof *style);
 			if (!style)
 				return false;
-			css_cascade(style, &css_blank_style);
+			css_cascade(style, &css_blank_style, NULL);
 			table = box_create(style, block->href, block->target,
 					0, 0, c);
 			if (!table) {
@@ -225,7 +223,7 @@ bool box_normalise_table(struct box *table, struct content * c)
 				free(col_info.spans);
 				return false;
 			}
-			css_cascade(style, &css_blank_style);
+			css_cascade(style, &css_blank_style, NULL);
 			row_group = box_create(style, table->href,
 					table->target, 0, 0, c);
 			if (!row_group) {
@@ -238,8 +236,6 @@ bool box_normalise_table(struct box *table, struct content * c)
 				table->children = row_group;
 			else
 				child->prev->next = row_group;
-			if (table->last == child)
-				table->last = row_group;
 			row_group->prev = child->prev;
 			while (child != 0 && (
 					child->type == BOX_BLOCK ||
@@ -257,6 +253,8 @@ bool box_normalise_table(struct box *table, struct content * c)
 			row_group->next = next_child = child;
 			if (row_group->next)
 				row_group->next->prev = row_group;
+			else
+				table->last = row_group;
 			row_group->parent = table;
 			if (!box_normalise_table_row_group(row_group,
 					&col_info, c)) {
@@ -286,23 +284,54 @@ bool box_normalise_table(struct box *table, struct content * c)
 	free(col_info.spans);
 
 	if (table->children == 0) {
-		LOG(("table->children == 0, removing"));
-		if (table->prev == 0)
-			table->parent->children = table->next;
-		else
-			table->prev->next = table->next;
-		if (table->next != 0)
-			table->next->prev = table->prev;
-		box_free(table);
-	} else {
-		box_normalise_table_spans(table);
-		if (!table_calculate_column_types(table))
+		struct box *row;
+
+		LOG(("table->children == 0, creating implied row"));
+
+		assert(table->style != NULL);
+		style = talloc_memdup(c, table->style, sizeof *style);
+		if (!style) {
 			return false;
-		if (table->style->border_collapse ==
-				CSS_BORDER_COLLAPSE_COLLAPSE)
-			table_collapse_borders(table);
+		}
+		css_cascade(style, &css_blank_style, NULL);
+		row_group = box_create(style, table->href,
+				table->target, 0, 0, c);
+		if (!row_group) {
+			talloc_free(style);
+			return false;
+		}
+		row_group->type = BOX_TABLE_ROW_GROUP;
+
+		style = talloc_memdup(c, row_group->style, sizeof *style);
+		if (!style) {
+			box_free(row_group);
+			return false;
+		}
+		css_cascade(style, &css_blank_style, NULL);
+		row = box_create(style, row_group->href,
+				row_group->target, 0, 0, c);
+		if (!row) {
+			talloc_free(style);
+			box_free(row_group);
+			return false;
+		}
+		row->type = BOX_TABLE_ROW;
+
+		row->parent = row_group;
+		row_group->children = row_group->last = row;
+
+		row_group->parent = table;
+		table->children = table->last = row_group;
+
+		table->rows = 1;
 	}
 
+	box_normalise_table_spans(table);
+	if (!table_calculate_column_types(table))
+		return false;
+	if (table->style->border_collapse == CSS_BORDER_COLLAPSE_COLLAPSE)
+		table_collapse_borders(table);
+
 	LOG(("table %p done", table));
 
 	return true;
@@ -401,7 +430,7 @@ bool box_normalise_table_row_group(struct box *row_group,
 					sizeof *style);
 			if (!style)
 				return false;
-			css_cascade(style, &css_blank_style);
+			css_cascade(style, &css_blank_style, NULL);
 			row = box_create(style, row_group->href,
 					row_group->target, 0, 0, c);
 			if (!row) {
@@ -430,6 +459,8 @@ bool box_normalise_table_row_group(struct box *row_group,
 			row->next = next_child = child;
 			if (row->next)
 				row->next->prev = row;
+			else
+				row_group->last = row;
 			row->parent = row_group;
 			if (!box_normalise_table_row(row, col_info,
 					c))
@@ -452,14 +483,23 @@ bool box_normalise_table_row_group(struct box *row_group,
 	}
 
 	if (row_group->children == 0) {
-		LOG(("row_group->children == 0, removing"));
-		if (row_group->prev == 0)
-			row_group->parent->children = row_group->next;
-		else
-			row_group->prev->next = row_group->next;
-		if (row_group->next != 0)
-			row_group->next->prev = row_group->prev;
-		box_free(row_group);
+		LOG(("row_group->children == 0, inserting implied row"));
+		assert(row_group->style != NULL);
+		style = talloc_memdup(c, row_group->style, sizeof *style);
+		if (!style) {
+			return false;
+		}
+		css_cascade(style, &css_blank_style, NULL);
+		row = box_create(style, row_group->href,
+				row_group->target, 0, 0, c);
+		if (!row) {
+			talloc_free(style);
+			return false;
+		}
+		row->type = BOX_TABLE_ROW;
+
+		row->parent = row_group;
+		row_group->children = row_group->last = row;
 	}
 
 	LOG(("row_group %p done", row_group));
@@ -474,7 +514,7 @@ bool box_normalise_table_row(struct box *row,
 {
 	struct box *child;
 	struct box *next_child;
-	struct box *cell;
+	struct box *cell = NULL;
 	struct css_style *style;
 	unsigned int i;
 
@@ -501,7 +541,13 @@ bool box_normalise_table_row(struct box *row,
 			style = talloc_memdup(c, row->style, sizeof *style);
 			if (!style)
 				return false;
-			css_cascade(style, &css_blank_style);
+			css_cascade(style, &css_blank_style, NULL);
+			if (child->style && (child->style->position ==
+					CSS_POSITION_ABSOLUTE ||
+					child->style->position ==
+					CSS_POSITION_FIXED)) {
+				style->position = child->style->position;
+			}
 			cell = box_create(style, row->href, row->target, 0, 0,
 					c);
 			if (!cell) {
@@ -565,18 +611,26 @@ bool box_normalise_table_row(struct box *row,
 	col_info->current_column = 0;
 	col_info->extra = false;
 
-	if (row->children == 0) {
-		LOG(("row->children == 0, removing"));
-		if (row->prev == 0)
-			row->parent->children = row->next;
-		else
-			row->prev->next = row->next;
-		if (row->next != 0)
-			row->next->prev = row->prev;
-		box_free(row);
-	} else {
+	/* Removing empty rows causes ill effects for HTML such as:
+	 *
+	 * <tr><td colspan="2">1</td></tr><tr></tr><tr><td>2</td></tr>
+	 *
+	 * as it breaks the colspan value. Additionally, both MSIE and FF
+	 * render in the same manner as NetSurf does with the empty row
+	 * culling commented out.
+	 */
+//	if (row->children == 0) {
+//		LOG(("row->children == 0, removing"));
+//		if (row->prev == 0)
+//			row->parent->children = row->next;
+//		else
+//			row->prev->next = row->next;
+//		if (row->next != 0)
+//			row->next->prev = row->prev;
+//		box_free(row);
+//	} else {
 		col_info->num_rows++;
-	}
+//	}
 
 	LOG(("row %p done", row));
 
