@@ -23,6 +23,7 @@
 #include <assert.h>
 #include <stdbool.h>
 #include <string.h>
+#include <strings.h>
 #include "content/content.h"
 #include "render/box.h"
 #include "render/imagemap.h"
@@ -247,11 +248,11 @@ bool imagemap_extract(xmlNode *node, struct content *c)
 	assert(c != NULL);
 
 	if (node->type == XML_ELEMENT_NODE) {
-		if (strcmp(node->name, "map") == 0) {
-			if ((name = (char *)xmlGetProp(node,
-					(const xmlChar *)"id")) == NULL) {
-				if ((name = (char *)xmlGetProp(node,
-					(const xmlChar *)"name")) ==
+		if (strcmp((const char *) node->name, "map") == 0) {
+			if ((name = (char *) xmlGetProp(node,
+					(const xmlChar *) "id")) == NULL) {
+				if ((name = (char *) xmlGetProp(node,
+					(const xmlChar *) "name")) ==
 						NULL)
 					return true;
 			}
@@ -306,13 +307,15 @@ bool imagemap_extract_map(xmlNode *node, struct content *c,
 		/** \todo ignore <area> elements if there are other
 		 *	block-level elements present in map
 		 */
-		if (strcmp(node->name, "area") == 0 ||
-		    strcmp(node->name, "a") == 0) {
-			return imagemap_addtolist(node,
-				c->data.html.base_url, entry);
+		if (strcmp((const char *) node->name, "area") == 0 ||
+		    strcmp((const char *) node->name, "a") == 0) {
+			if (!imagemap_addtolist(node,
+					c->data.html.base_url, entry))
+				return false;
 		}
+	} else {
+		return true;
 	}
-	else return true;
 
 	for (this_node = node->children; this_node != 0;
 					this_node = this_node->next) {
@@ -341,7 +344,7 @@ bool imagemap_addtolist(xmlNode *n, char *base_url, struct mapentry **entry)
 	assert(base_url != NULL);
 	assert(entry != NULL);
 
-	if (strcmp(n->name, "area") == 0) {
+	if (strcmp((const char *) n->name, "area") == 0) {
 		/* nohref attribute present - ignore this entry */
 		if (xmlGetProp(n, (const xmlChar*)"nohref") != 0) {
 			return true;
@@ -381,7 +384,8 @@ bool imagemap_addtolist(xmlNode *n, char *base_url, struct mapentry **entry)
 	}
 
 	/* extract area shape */
-	if (strcasecmp(shape, "rect") == 0) {
+	if (strcasecmp(shape, "rect") == 0 ||
+			strcasecmp(shape, "rectangle") == 0) {
 		new_map->type = IMAGEMAP_RECT;
 	}
 	else if (strcasecmp(shape, "circle") == 0) {
@@ -447,6 +451,8 @@ bool imagemap_addtolist(xmlNode *n, char *base_url, struct mapentry **entry)
 	}
 
 	if (new_map->type != IMAGEMAP_DEFAULT) {
+		int x, y;
+		float *xcoords, *ycoords;
 		/* coordinates are a comma-separated list of values */
 		val = strtok(coords, ",");
 		num = 1;
@@ -516,8 +522,6 @@ bool imagemap_addtolist(xmlNode *n, char *base_url, struct mapentry **entry)
 				xmlFree(coords);
 				return false;
 			}
-			int x, y;
-			float *xcoords, *ycoords;
 			while (val) {
 				x = atoi(val);
 				val = strtok('\0', ",");
