@@ -1,9 +1,10 @@
 /*
  * Copyright 2003 Phil Mellor <monkeyson@users.sourceforge.net>
- * Copyright 2004-2007 James Bursa <bursa@users.sourceforge.net>
+ * Copyright 2004-2008 James Bursa <bursa@users.sourceforge.net>
  * Copyright 2003 John M Bell <jmb202@ecs.soton.ac.uk>
  * Copyright 2005 Richard Wilson <info@tinct.net>
  * Copyright 2004 Andrew Timmins <atimmins@blueyonder.co.uk>
+ * Copyright 2004-2009 John Tytgat <joty@netsurf-browser.org>
  *
  * This file is part of NetSurf, http://www.netsurf-browser.org/
  *
@@ -31,25 +32,25 @@
 #include <time.h>
 #include <features.h>
 #include <unixlib/local.h>
-#include <unixlib/sigstate.h>
 #include <curl/curl.h>
-#include <oslib/font.h>
-#include <oslib/help.h>
-#include <oslib/hourglass.h>
-#include <oslib/inetsuite.h>
-#include <oslib/os.h>
-#include <oslib/osbyte.h>
-#include <oslib/osfile.h>
-#include <oslib/osfscontrol.h>
-#include <oslib/osgbpb.h>
-#include <oslib/osmodule.h>
-#include <oslib/osspriteop.h>
-#include <oslib/pdriver.h>
-#include <oslib/plugin.h>
-#include <oslib/wimp.h>
-#include <oslib/wimpspriteop.h>
-#include <oslib/uri.h>
-#include <rufl.h>
+#include <hubbub/hubbub.h>
+#include "oslib/font.h"
+#include "oslib/help.h"
+#include "oslib/hourglass.h"
+#include "oslib/inetsuite.h"
+#include "oslib/os.h"
+#include "oslib/osbyte.h"
+#include "oslib/osfile.h"
+#include "oslib/osfscontrol.h"
+#include "oslib/osgbpb.h"
+#include "oslib/osmodule.h"
+#include "oslib/osspriteop.h"
+#include "oslib/pdriver.h"
+#include "oslib/plugin.h"
+#include "oslib/wimp.h"
+#include "oslib/wimpspriteop.h"
+#include "oslib/uri.h"
+#include "rufl.h"
 #include "utils/config.h"
 #include "content/content.h"
 #include "content/urldb.h"
@@ -72,24 +73,19 @@
 #ifdef WITH_PLUGIN
 #include "riscos/plugin.h"
 #endif
-#ifdef WITH_PRINT
 #include "riscos/print.h"
-#endif
 #include "riscos/query.h"
 #include "riscos/save.h"
 #include "riscos/save_complete.h"
 #include "riscos/textselection.h"
 #include "riscos/theme.h"
 #include "riscos/treeview.h"
-#ifdef WITH_URI
 #include "riscos/uri.h"
-#endif
-#ifdef WITH_URL
 #include "riscos/url_protocol.h"
-#endif
 #include "riscos/url_complete.h"
 #include "riscos/wimp.h"
 #include "riscos/wimp_event.h"
+#include "riscos/wimputils.h"
 #include "utils/filename.h"
 #include "utils/log.h"
 #include "utils/messages.h"
@@ -137,6 +133,9 @@
 #ifndef FILETYPE_ARTWORKS
 #define FILETYPE_ARTWORKS 0xd94
 #endif
+#ifndef FILETYPE_SVG
+#define FILETYPE_SVG 0xaad
+#endif
 
 extern bool ro_plot_patterned_lines;
 
@@ -148,6 +147,9 @@ int __feature_imagefs_is_file = 1;		/**< For UnixLib. */
 /* default filename handling */
 int __riscosify_control = __RISCOSIFY_NO_SUFFIX |
 			__RISCOSIFY_NO_REVERSE_SUFFIX;
+#ifndef __ELF__
+extern int __dynamic_num;
+#endif
 
 const char * NETSURF_DIR;
 
@@ -180,59 +182,55 @@ static struct {
 } prev_sigs;
 
 /** Accepted wimp user messages. */
-static wimp_MESSAGE_LIST(42) task_messages = { {
+static ns_wimp_message_list task_messages = { 
 	message_HELP_REQUEST,
-	message_DATA_SAVE,
-	message_DATA_SAVE_ACK,
-	message_DATA_LOAD,
-	message_DATA_LOAD_ACK,
-	message_DATA_OPEN,
-	message_PRE_QUIT,
-	message_SAVE_DESKTOP,
-	message_MENU_WARNING,
-	message_MENUS_DELETED,
-	message_WINDOW_INFO,
-	message_CLAIM_ENTITY,
-	message_DATA_REQUEST,
-	message_DRAGGING,
-	message_DRAG_CLAIM,
-	message_MODE_CHANGE,
-	message_FONT_CHANGED,
-#ifdef WITH_URI
-	message_URI_PROCESS,
-	message_URI_RETURN_RESULT,
-#endif
-#ifdef WITH_URL
-	message_INET_SUITE_OPEN_URL,
-#endif
+	{
+		message_DATA_SAVE,
+		message_DATA_SAVE_ACK,
+		message_DATA_LOAD,
+		message_DATA_LOAD_ACK,
+		message_DATA_OPEN,
+		message_PRE_QUIT,
+		message_SAVE_DESKTOP,
+		message_MENU_WARNING,
+		message_MENUS_DELETED,
+		message_WINDOW_INFO,
+		message_CLAIM_ENTITY,
+		message_DATA_REQUEST,
+		message_DRAGGING,
+		message_DRAG_CLAIM,
+		message_MODE_CHANGE,
+		message_FONT_CHANGED,
+		message_URI_PROCESS,
+		message_URI_RETURN_RESULT,
+		message_INET_SUITE_OPEN_URL,
 #ifdef WITH_PLUGIN
-	message_PLUG_IN_OPENING,
-	message_PLUG_IN_CLOSED,
-	message_PLUG_IN_RESHAPE_REQUEST,
-	message_PLUG_IN_FOCUS,
-	message_PLUG_IN_URL_ACCESS,
-	message_PLUG_IN_STATUS,
-	message_PLUG_IN_BUSY,
-	message_PLUG_IN_STREAM_NEW,
-	message_PLUG_IN_STREAM_WRITE,
-	message_PLUG_IN_STREAM_WRITTEN,
-	message_PLUG_IN_STREAM_DESTROY,
-	message_PLUG_IN_OPEN,
-	message_PLUG_IN_CLOSE,
-	message_PLUG_IN_RESHAPE,
-	message_PLUG_IN_STREAM_AS_FILE,
-	message_PLUG_IN_NOTIFY,
-	message_PLUG_IN_ABORT,
-	message_PLUG_IN_ACTION,
-	/* message_PLUG_IN_INFORMED, (not provided by oslib) */
+		message_PLUG_IN_OPENING,
+		message_PLUG_IN_CLOSED,
+		message_PLUG_IN_RESHAPE_REQUEST,
+		message_PLUG_IN_FOCUS,
+		message_PLUG_IN_URL_ACCESS,
+		message_PLUG_IN_STATUS,
+		message_PLUG_IN_BUSY,
+		message_PLUG_IN_STREAM_NEW,
+		message_PLUG_IN_STREAM_WRITE,
+		message_PLUG_IN_STREAM_WRITTEN,
+		message_PLUG_IN_STREAM_DESTROY,
+		message_PLUG_IN_OPEN,
+		message_PLUG_IN_CLOSE,
+		message_PLUG_IN_RESHAPE,
+		message_PLUG_IN_STREAM_AS_FILE,
+		message_PLUG_IN_NOTIFY,
+		message_PLUG_IN_ABORT,
+		message_PLUG_IN_ACTION,
+		/* message_PLUG_IN_INFORMED, (not provided by oslib) */
 #endif
-#ifdef WITH_PRINT
-	message_PRINT_SAVE,
-	message_PRINT_ERROR,
-	message_PRINT_TYPE_ODD,
-#endif
-	0
-} };
+		message_PRINT_SAVE,
+		message_PRINT_ERROR,
+		message_PRINT_TYPE_ODD,
+		0
+	}
+};
 
 static struct
 {
@@ -271,6 +269,10 @@ static void ro_msg_save_desktop(wimp_message *message);
 static void ro_msg_window_info(wimp_message *message);
 static void ro_gui_view_source_bounce(wimp_message *message);
 
+static void *myrealloc(void *ptr, size_t len, void *pw)
+{
+	return realloc(ptr, len);
+}
 
 /**
  * Initialise the gui (RISC OS specific part).
@@ -311,6 +313,10 @@ void gui_init(int argc, char** argv)
 			ro_plot_patterned_lines = false;
 	}
 
+	if (hubbub_initialise("NetSurf:Resources.Aliases", myrealloc, NULL) !=
+			HUBBUB_OK)
+		die("Failed to initialise HTML parsing library.");
+
 	/* Read in the options */
 	options_read("NetSurf:Choices");
 
@@ -380,13 +386,6 @@ void gui_init(int argc, char** argv)
 	prev_sigs.sigint = signal(SIGINT, ro_gui_signal);
 	prev_sigs.sigsegv = signal(SIGSEGV, ro_gui_signal);
 	prev_sigs.sigterm = signal(SIGTERM, ro_gui_signal);
-	/* Ignore SIGPIPE - this is necessary as OpenSSL can generate these
-	 * and the default action is to terminate the app. There's no easy
-	 * way of determining the cause of the SIGPIPE (other than using
-	 * sigaction() and some mechanism for getting the file descriptor
-	 * out of libcurl). However, we expect nothing else to generate a
-	 * SIGPIPE, anyway, so may as well just ignore them all. */
-	signal(SIGPIPE, SIG_IGN);
 
 	if (prev_sigs.sigabrt == SIG_ERR || prev_sigs.sigfpe == SIG_ERR ||
 			prev_sigs.sigill == SIG_ERR ||
@@ -418,9 +417,7 @@ void gui_init(int argc, char** argv)
 	filename_initialise();
 
 	/* Initialise save complete functionality */
-#ifdef WITH_SAVE_COMPLETE
 	save_complete_init();
-#endif
 
 	/* Initialise bitmap memory pool */
 	bitmap_initialise_memory();
@@ -431,7 +428,7 @@ void gui_init(int argc, char** argv)
 
 	/* Initialise with the wimp */
 	error = xwimp_initialise(wimp_VERSION_RO38, task_name,
-			(const wimp_message_list *) &task_messages, 0,
+			PTR_WIMP_MESSAGE_LIST(&task_messages), 0,
 			&task_handle);
 	if (error) {
 		LOG(("xwimp_initialise: 0x%x: %s",
@@ -746,12 +743,12 @@ void gui_init2(int argc, char** argv)
 			LOG(("malloc failed"));
 			die("Insufficient memory for URL");
 		}
-		snprintf(url, 80, "file:///<NetSurf$Dir>/Docs/intro_%s",
+		snprintf(url, 80, "file:///<NetSurf$Dir>/Docs/welcome/index_%s",
 			option_language);
 	}
 
 	if (open_window)
-			browser_window_create(url, NULL, 0, true);
+			browser_window_create(url, NULL, 0, true, false);
 
 	free(url);
 }
@@ -775,6 +772,8 @@ void gui_quit(void)
 	xwimp_close_down(task_handle);
  	free(default_stylesheet_url);
 	free(adblock_stylesheet_url);
+	/* We don't care if this fails */
+	hubbub_finalise(myrealloc, NULL);
 	xhourglass_off();
 }
 
@@ -844,10 +843,45 @@ void ro_gui_signal(int sig)
 		for (c = content_list; c; c = c->next)
 			if (c->type == CONTENT_HTML && c->data.html.layout) {
 				LOG(("Dumping: '%s'", c->url));
-				box_dump(c->data.html.layout, 0);
+				box_dump(stderr, c->data.html.layout, 0);
 			}
 		options_dump();
 		/*rufl_dump_state();*/
+
+#ifndef __ELF__
+		/* save WimpSlot and DA to files if NetSurf$CoreDump exists */
+		int used;
+		xos_read_var_val_size("NetSurf$CoreDump", 0, 0, &used, 0, 0);
+		if (used) {
+			int curr_slot;
+			xwimp_slot_size(-1, -1, &curr_slot, 0, 0);
+			LOG(("saving WimpSlot, size 0x%x", curr_slot));
+			xosfile_save("$.NetSurf_Slot", 0x8000, 0,
+					(byte *) 0x8000,
+					(byte *) 0x8000 + curr_slot);
+
+			if (__dynamic_num != -1) {
+				int size;
+				byte *base_address;
+				xosdynamicarea_read(__dynamic_num, &size,
+						&base_address, 0, 0, 0, 0, 0);
+				LOG(("saving DA %i, base %p, size 0x%x",
+						__dynamic_num,
+						base_address, size));
+				xosfile_save("$.NetSurf_DA",
+						(bits) base_address, 0,
+						base_address,
+						base_address + size);
+			}
+		}
+#else
+		/* Save WimpSlot and UnixLib managed DAs when UnixEnv$coredump
+		 * defines a coredump directory.  */
+		_kernel_oserror *err = __unixlib_write_coredump (NULL);
+		if (err != NULL)
+			LOG(("Coredump failed: %s", err->errmess));
+#endif
+
 		xhourglass_colours(old_sand, old_glass, 0, 0);
 		xhourglass_off();
 
@@ -882,7 +916,8 @@ void gui_poll(bool active)
 {
 	wimp_event_no event;
 	wimp_block block;
-	const wimp_poll_flags mask = wimp_MASK_LOSE | wimp_MASK_GAIN | wimp_SAVE_FP;
+	const wimp_poll_flags mask = wimp_MASK_LOSE | wimp_MASK_GAIN |
+			wimp_SAVE_FP;
 
 	/* Poll wimp. */
 	xhourglass_off();
@@ -917,7 +952,7 @@ void gui_poll(bool active)
 	gui_last_poll = clock();
 	ro_gui_handle_event(event, &block);
 	schedule_run();
-        ro_gui_window_update_boxes();
+	ro_gui_window_update_boxes();
 
 	if (browser_reformat_pending && event == wimp_NULL_REASON_CODE)
 		ro_gui_window_process_reformats();
@@ -1005,7 +1040,8 @@ void gui_multitask(void)
 		return;
 
 	xhourglass_off();
-	event = wimp_poll(wimp_MASK_LOSE | wimp_MASK_GAIN | wimp_SAVE_FP, &block, 0);
+	event = wimp_poll(wimp_MASK_LOSE | wimp_MASK_GAIN | wimp_SAVE_FP,
+			&block, 0);
 	xhourglass_on();
 	gui_last_poll = clock();
 
@@ -1057,7 +1093,8 @@ void ro_gui_null_reason_code(void)
 			if (gui_track_wimp_w == dialog_url_complete)
 				ro_gui_url_complete_mouse_at(&pointer);
 			else if (gui_track_gui_window)
-				ro_gui_window_mouse_at(gui_track_gui_window, &pointer);
+				ro_gui_window_mouse_at(gui_track_gui_window,
+						&pointer);
 			break;
 	}
 }
@@ -1090,9 +1127,13 @@ void ro_gui_open_window_request(wimp_open *open)
 
 void ro_gui_close_window_request(wimp_close *close)
 {
-	if (ro_gui_wimp_event_close_window(close->w))
-		return;
-	ro_gui_dialog_close(close->w);
+	if (ro_gui_alt_pressed())
+		ro_gui_window_close_all();
+	else {
+		if (ro_gui_wimp_event_close_window(close->w))
+			return;
+		ro_gui_dialog_close(close->w);
+	}
 }
 
 
@@ -1163,12 +1204,13 @@ bool ro_gui_icon_bar_click(wimp_pointer *pointer)
 
 	} else if (pointer->buttons == wimp_CLICK_SELECT) {
 		if (option_homepage_url && option_homepage_url[0]) {
-			browser_window_create(option_homepage_url, NULL, 0, true);
+			browser_window_create(option_homepage_url, NULL, 0,
+					true, false);
 		} else {
 			snprintf(url, sizeof url,
-					"file:///<NetSurf$Dir>/Docs/intro_%s",
+					"file:///<NetSurf$Dir>/Docs/welcome/index_%s",
 					option_language);
-			browser_window_create(url, NULL, 0, true);
+			browser_window_create(url, NULL, 0, true, false);
 		}
 
 	} else if (pointer->buttons == wimp_CLICK_ADJUST) {
@@ -1241,10 +1283,17 @@ void ro_gui_drag_end(wimp_dragged *drag)
 
 void ro_gui_keypress(wimp_key *key)
 {
-	os_error *error;
+	if (key->c == wimp_KEY_ESCAPE &&
+		(gui_current_drag_type == GUI_DRAG_SAVE ||
+		 gui_current_drag_type == GUI_DRAG_DOWNLOAD_SAVE)) {
 
-	if (!ro_gui_wimp_event_keypress(key)) {
-		error = xwimp_process_key(key->c);
+		/* Allow Escape key to be used for cancelling a drag save
+			(easier than finding somewhere safe to abort the drag) */
+		ro_gui_drag_box_cancel();
+		gui_current_drag_type = GUI_DRAG_NONE;
+	}
+	else if (!ro_gui_wimp_event_keypress(key)) {
+		os_error *error = xwimp_process_key(key->c);
 		if (error) {
 			LOG(("xwimp_process_key: 0x%x: %s",
 					error->errnum, error->errmess));
@@ -1268,20 +1317,16 @@ void ro_gui_user_message(wimp_event_no event, wimp_message *message)
 			ro_msg_terminate_filename((wimp_full_message_data_xfer*)message);
 
 			if (event == wimp_USER_MESSAGE_ACKNOWLEDGE) {
-#ifdef WITH_PRINT
-				if (print_current_window)
-					print_dataload_bounce(message);
-#endif
+				if (ro_print_current_window)
+					ro_print_dataload_bounce(message);
 			}
 			else
 				ro_msg_dataload(message);
 			break;
 
 		case message_DATA_LOAD_ACK:
-#ifdef WITH_PRINT
-			if (print_current_window)
-				print_cleanup();
-#endif
+			if (ro_print_current_window)
+				ro_print_cleanup();
 			break;
 
 		case message_MENU_WARNING:
@@ -1310,7 +1355,6 @@ void ro_gui_user_message(wimp_event_no event, wimp_message *message)
 			ro_gui_wimp_get_desktop_font();
 			break;
 
-#ifdef WITH_URI
 		case message_URI_PROCESS:
 			if (event != wimp_USER_MESSAGE_ACKNOWLEDGE)
 				ro_uri_message_received(message);
@@ -1318,8 +1362,6 @@ void ro_gui_user_message(wimp_event_no event, wimp_message *message)
 		case message_URI_RETURN_RESULT:
 			ro_uri_bounce(message);
 			break;
-#endif
-#ifdef WITH_URL
 		case message_INET_SUITE_OPEN_URL:
 			if (event == wimp_USER_MESSAGE_ACKNOWLEDGE) {
 				ro_url_bounce(message);
@@ -1328,7 +1370,6 @@ void ro_gui_user_message(wimp_event_no event, wimp_message *message)
 				ro_url_message_received(message);
 			}
 			break;
-#endif
 #ifdef WITH_PLUGIN
 		case message_PLUG_IN_OPENING:
 			plugin_opening(message);
@@ -1374,18 +1415,16 @@ void ro_gui_user_message(wimp_event_no event, wimp_message *message)
 		case message_PLUG_IN_ACTION:
 			break;
 #endif
-#ifdef WITH_PRINT
 		case message_PRINT_SAVE:
 			if (event == wimp_USER_MESSAGE_ACKNOWLEDGE)
-				print_save_bounce(message);
+				ro_print_save_bounce(message);
 			break;
 		case message_PRINT_ERROR:
-			print_error(message);
+			ro_print_error(message);
 			break;
 		case message_PRINT_TYPE_ODD:
-			print_type_odd(message);
+			ro_print_type_odd(message);
 			break;
-#endif
 
 		case message_QUIT:
 			netsurf_quit = true;
@@ -1475,6 +1514,7 @@ void ro_msg_dataload(wimp_message *message)
 		case osfile_TYPE_SPRITE:
 		case osfile_TYPE_TEXT:
 		case FILETYPE_ARTWORKS:
+		case FILETYPE_SVG:
 			/* display the actual file */
 			url = path_to_url(message->data.data_xfer.file_name);
 			break;
@@ -1513,7 +1553,7 @@ void ro_msg_dataload(wimp_message *message)
 				ro_gui_tree_start_edit(hotlist_tree, &node->data, NULL);
 		}
 	} else {
-		browser_window_create(url, 0, 0, true);
+		browser_window_create(url, 0, 0, true, false);
 	}
 
 	/* send DataLoadAck */
@@ -1768,7 +1808,8 @@ void ro_msg_datasave(wimp_message *message)
 		case FILETYPE_JPEG:
 		case osfile_TYPE_SPRITE:
 		case osfile_TYPE_TEXT:
-		case FILETYPE_ARTWORKS: {
+		case FILETYPE_ARTWORKS:
+		case FILETYPE_SVG: {
 			os_error *error;
 
 			dataxfer->your_ref = dataxfer->my_ref;
@@ -1796,10 +1837,8 @@ void ro_msg_datasave_ack(wimp_message *message)
 {
 	ro_msg_terminate_filename((wimp_full_message_data_xfer*)message);
 
-#ifdef WITH_PRINT
-	if (print_ack(message))
+	if (ro_print_ack(message))
 		return;
-#endif
 
 	switch (gui_current_drag_type) {
 		case GUI_DRAG_DOWNLOAD_SAVE:
@@ -1846,7 +1885,7 @@ void ro_msg_dataopen(wimp_message *message)
 			url = malloc(80);
 			if (url)
 				snprintf(url, 80,
-					"file:///<NetSurf$Dir>/Docs/intro_%s",
+					"file:///<NetSurf$Dir>/Docs/welcome/index_%s",
 					option_language);
 		}
 		if (!url)
@@ -1871,7 +1910,7 @@ void ro_msg_dataopen(wimp_message *message)
 		return;
 
 	/* create a new window with the file */
-	browser_window_create(url, NULL, 0, true);
+	browser_window_create(url, NULL, 0, true, false);
 
 	free(url);
 }
@@ -1954,7 +1993,10 @@ void ro_msg_window_info(wimp_message *message)
 	g = ro_gui_window_lookup(wi->w);
 
 	/* ic_<task name> will suffice for our other windows */
-	if (g) ro_gui_window_iconise(g, wi);
+	if (g) {
+		ro_gui_window_iconise(g, wi);
+		ro_gui_dialog_close_persistent(wi->w);
+	}
 }
 
 
@@ -1968,16 +2010,16 @@ void ro_msg_window_info(wimp_message *message)
 char *path_to_url(const char *path)
 {
 	int spare;
-	char *buffer = 0;
-	char *url = 0;
+	char *buffer, *url, *escurl;
 	os_error *error;
+	url_func_result url_err;
 
 	error = xosfscontrol_canonicalise_path(path, 0, 0, 0, 0, &spare);
 	if (error) {
 		LOG(("xosfscontrol_canonicalise_path failed: 0x%x: %s",
 				error->errnum, error->errmess));
 		warn_user("PathToURL", error->errmess);
-		return 0;
+		return NULL;
 	}
 
 	buffer = malloc(1 - spare);
@@ -1987,7 +2029,7 @@ char *path_to_url(const char *path)
 		warn_user("NoMemory", 0);
 		free(buffer);
 		free(url);
-		return 0;
+		return NULL;
 	}
 
 	error = xosfscontrol_canonicalise_path(path, buffer, 0, 0, 1 - spare,
@@ -1998,14 +2040,30 @@ char *path_to_url(const char *path)
 		warn_user("PathToURL", error->errmess);
 		free(buffer);
 		free(url);
-		return 0;
+		return NULL;
 	}
 
-	strcpy(url, "file://");
-	__unixify(buffer, __RISCOSIFY_NO_REVERSE_SUFFIX, url + 7,
-			1 - spare + 3 /* 10 - "file://" */, 0);
-	free(buffer);
-	return url;
+	memcpy(url, "file://", SLEN("file://"));
+	if (__unixify(buffer, __RISCOSIFY_NO_REVERSE_SUFFIX,
+			url + SLEN("file://"),
+			1 - spare + 10 - SLEN("file://"),
+			0) == NULL) {
+		LOG(("__unixify failed: %s", buffer));
+		free(buffer);
+		free(url);
+		return NULL;
+	}
+	free(buffer); buffer = NULL;
+
+	/* We don't want '/' to be escaped.  */
+	url_err = url_escape(url, SLEN("file://"), false, "/", &escurl);
+	free(url); url = NULL;
+	if (url_err != URL_FUNC_OK) {
+		LOG(("url_escape failed: %s", url));
+		return NULL;
+	}
+
+	return escurl;
 }
 
 
@@ -2054,17 +2112,19 @@ char *url_to_path(const char *url)
 
 void ro_gui_get_screen_properties(void)
 {
-	const os_VDU_VAR_LIST(5) vars = {
-		{ os_MODEVAR_XWIND_LIMIT,
-		os_MODEVAR_YWIND_LIMIT,
-		os_MODEVAR_XEIG_FACTOR,
-		os_MODEVAR_YEIG_FACTOR,
-		os_VDUVAR_END_LIST }
+	static const ns_os_vdu_var_list vars = {
+		os_MODEVAR_XWIND_LIMIT,
+		{
+			os_MODEVAR_YWIND_LIMIT,
+			os_MODEVAR_XEIG_FACTOR,
+			os_MODEVAR_YEIG_FACTOR,
+			os_VDUVAR_END_LIST
+		}
 	};
 	os_error *error;
 	int vals[4];
 
-	error = xos_read_vdu_variables((const os_vdu_var_list*)&vars, vals);
+	error = xos_read_vdu_variables(PTR_OS_VDU_VAR_LIST(&vars), vals);
 	if (error) {
 		LOG(("xos_read_vdu_variables: 0x%x: %s",
 			error->errnum, error->errmess));
@@ -2100,9 +2160,10 @@ void ro_gui_open_help_page(const char *page)
 	if ((length = snprintf(url, sizeof url,
 			"file:///<NetSurf$Dir>/Docs/%s_%s",
 			page, option_language)) >= 0 && length < (int)sizeof(url))
-		browser_window_create(url, NULL, 0, true);
+		browser_window_create(url, NULL, 0, true, false);
 }
 
+
 /**
  * Send the source of a content to a text editor.
  */
@@ -2156,8 +2217,9 @@ void ro_gui_view_source(struct content *content)
 		message.file_name[211] = '\0';
 		error = xosfile_save_stamped(message.file_name,
 				ro_content_filetype(content),
-				content->source_data,
-				content->source_data + content->source_size);
+				(byte *) content->source_data,
+				(byte *) content->source_data + 
+						content->source_size);
 		if (error) {
 			LOG(("xosfile_save_stamped failed: 0x%x: %s",
 					error->errnum, error->errmess));
@@ -2182,7 +2244,8 @@ void ro_gui_view_source(struct content *content)
 }
 
 
-void ro_gui_view_source_bounce(wimp_message *message) {
+void ro_gui_view_source_bounce(wimp_message *message)
+{
 	char *filename;
 	os_error *error;
 	char command[256];
@@ -2199,16 +2262,55 @@ void ro_gui_view_source_bounce(wimp_message *message) {
 }
 
 
+/**
+ * Send the debug dump of a content to a text editor.
+ */
+
+void ro_gui_dump_content(struct content *content)
+{
+	os_error *error;
+
+	/* open file for dump */
+	FILE *stream = fopen("<Wimp$ScrapDir>.WWW.NetSurf.dump", "w");
+	if (!stream) {
+		LOG(("fopen: errno %i", errno));
+		warn_user("SaveError", strerror(errno));
+		return;
+	}
+
+	/* output debug information to file */
+	switch (content->type) {
+	case CONTENT_HTML:
+		box_dump(stream, content->data.html.layout, 0);
+		break;
+	case CONTENT_CSS:
+		css_dump_stylesheet(content->data.css.css);
+		break;
+	default:
+		break;
+        }
+
+	fclose(stream);
+
+	/* launch file in editor */
+	error = xwimp_start_task("Filer_Run <Wimp$ScrapDir>.WWW.NetSurf.dump",
+			0);
+	if (error) {
+		LOG(("xwimp_start_task failed: 0x%x: %s",
+					error->errnum, error->errmess));
+		warn_user("WimpError", error->errmess);
+	}
+}
+
+
 /**
  * Broadcast an URL that we can't handle.
  */
 
 void gui_launch_url(const char *url)
 {
-#ifdef WITH_URL
 	/* Try ant broadcast first */
 	ro_url_broadcast(url);
-#endif
 }
 
 
@@ -2230,7 +2332,7 @@ void warn_user(const char *warning, const char *detail)
 				detail ? detail : "");
 		warn_buffer[sizeof warn_buffer - 1] = 0;
 		ro_gui_set_icon_string(dialog_warning, ICON_WARNING_MESSAGE,
-				warn_buffer);
+				warn_buffer, true);
 		xwimp_set_icon_state(dialog_warning, ICON_WARNING_HELP,
 				wimp_ICON_DELETED, wimp_ICON_DELETED);
 		ro_gui_dialog_open(dialog_warning);
@@ -2292,3 +2394,9 @@ bool ro_gui_prequit(void)
 {
 	return ro_gui_download_prequit();
 }
+
+void PDF_Password(char **owner_pass, char **user_pass, char *path)
+{
+	/*TODO:this waits to be written, until then no PDF encryption*/
+	*owner_pass = NULL;
+}
