@@ -37,13 +37,14 @@
  * Helpers are not supported (system variable detection is #if 0ed out)
  */
 
+#include "utils/config.h"
+#ifdef WITH_PLUGIN
+
 #include <assert.h>
-#include <ctype.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
 #include "oslib/mimemap.h"
 #include "oslib/os.h"
 #include "oslib/osfile.h"
@@ -51,7 +52,6 @@
 #include "oslib/osgbpb.h"
 #include "oslib/plugin.h"
 #include "oslib/wimp.h"
-
 #include "utils/config.h"
 #include "content/content.h"
 #include "content/fetch.h"
@@ -70,8 +70,6 @@
 #include "utils/utils.h"
 
 
-#ifdef WITH_PLUGIN
-
 typedef enum {
 	PLUGIN_PARAMETER_DATA    = 1,
 	PLUGIN_PARAMETER_URL     = 2,
@@ -252,7 +250,7 @@ void plugin_destroy(struct content *c)
 bool plugin_redraw(struct content *c, int x, int y,
 		int width, int height,
 		int clip_x0, int clip_y0, int clip_x1, int clip_y1,
-		float scale, unsigned long background_colour)
+		float scale, colour background_colour)
 {
 	/* do nothing */
 	LOG(("plugin_redraw"));
@@ -932,7 +930,8 @@ void plugin_url_access(wimp_message *message)
 			else if (!option_block_popups &&
 					strcasecmp(window, "_blank") == 0) {
 				/* don't do this if popups are blocked */
-				browser_window_create(url, NULL, 0, true);
+				browser_window_create(url, NULL, 0, true,
+						false);
 			}
 		}
 		else { /* POST request */
@@ -1416,23 +1415,23 @@ bool plugin_write_parameters_file(struct content *c,
 	fp = fopen(c->data.plugin.filename, "wb+");
 
 	while (pilist != 0) {
-		fwrite(&pilist->type, (unsigned int)sizeof(int), 1, fp);
-		fwrite(&pilist->rsize, (unsigned int)sizeof(int), 1, fp);
+		fwrite(&pilist->type, sizeof(int), 1, fp);
+		fwrite(&pilist->rsize, sizeof(int), 1, fp);
 
-		fwrite(&pilist->nsize, (unsigned int)sizeof(int), 1, fp);
-		fwrite(pilist->name, (unsigned int)strlen(pilist->name), 1, fp);
+		fwrite(&pilist->nsize, sizeof(int), 1, fp);
+		fwrite(pilist->name, strlen(pilist->name), 1, fp);
 		for (; pilist->npad != 0; pilist->npad--)
 			fputc('\0', fp);
 
-		fwrite(&pilist->vsize, (unsigned int)sizeof(int), 1, fp);
-		fwrite(pilist->value, (unsigned int)strlen(pilist->value), 1, fp);
+		fwrite(&pilist->vsize, sizeof(int), 1, fp);
+		fwrite(pilist->value, strlen(pilist->value), 1, fp);
 		for(; pilist->vpad != 0; pilist->vpad--)
 			fputc('\0', fp);
 
-		fwrite(&pilist->msize, (unsigned int)sizeof(int), 1, fp);
+		fwrite(&pilist->msize, sizeof(int), 1, fp);
 		if (pilist->msize > 0) {
 			fwrite(pilist->mime_type,
-				(unsigned int)strlen(pilist->mime_type), 1, fp);
+				strlen(pilist->mime_type), 1, fp);
 			for (; pilist->mpad != 0; pilist->mpad--)
 				fputc('\0', fp);
 		}
@@ -1448,7 +1447,7 @@ bool plugin_write_parameters_file(struct content *c,
 		ppi = 0;
 	}
 
-	fwrite("\0", sizeof(char), 4, fp);
+	fwrite("", sizeof(char), 4, fp);
 
 	fclose(fp);
 
@@ -1693,6 +1692,8 @@ void plugin_stream_callback(content_msg msg, struct content *c,
 			plugin_send_stream_new(p);
 			break;
 
+		case CONTENT_MSG_LAUNCH:
+			/* Fall through */
 		case CONTENT_MSG_ERROR:
 			/* The plugin we were fetching may have been
 			 * redirected, in that case, the object pointers
@@ -1704,14 +1705,6 @@ void plugin_stream_callback(content_msg msg, struct content *c,
 						plugin_STREAM_DESTROY_ERROR);
 			break;
 
-		case CONTENT_MSG_REDIRECT:
-			/* and re-start fetch with new URL */
-			p->c = 0;
-			if (!plugin_start_fetch(p, data.redirect))
-				plugin_destroy_stream(p,
-						plugin_STREAM_DESTROY_ERROR);
-			break;
-
 		case CONTENT_MSG_NEWPTR:
 			p->c = c;
 			break;
@@ -1725,11 +1718,9 @@ void plugin_stream_callback(content_msg msg, struct content *c,
 			/* ignore this */
 			break;
 
-#ifdef WITH_SSL
 		case CONTENT_MSG_SSL:
 			plugin_destroy_stream(p, plugin_STREAM_DESTROY_ERROR);
 			break;
-#endif
 
 		case CONTENT_MSG_READY:
 		case CONTENT_MSG_DONE:
@@ -1775,12 +1766,9 @@ void plugin_fetch_callback(fetch_msg msg, void *p, const void *data,
 			break;
 
 		case FETCH_TYPE:
-		case FETCH_REDIRECT:
 		case FETCH_NOTMODIFIED:
 		case FETCH_AUTH:
-#ifdef WITH_SSL
 		case FETCH_CERT_ERR:
-#endif
 		default:
 			/* not possible */
 			assert(0);
