@@ -17,6 +17,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "utils/config.h"
+
 #include <assert.h>
 #include <string.h>
 #include "oslib/font.h"
@@ -68,12 +70,10 @@
  *  \todo make use of print stylesheets
  */
 
-struct gui_window *print_current_window = 0;
+struct gui_window *ro_print_current_window = NULL;
 bool print_text_black = false;
 bool print_active = false;
 
-#ifdef WITH_PRINT
-
 /* 1 millipoint == 1/400 OS unit == 1/800 browser units */
 
 static int print_prev_message = 0;
@@ -102,23 +102,24 @@ static bool print_fonts_plot_rectangle(int x0, int y0, int width, int height,
 		int line_width, colour c, bool dotted, bool dashed);
 static bool print_fonts_plot_line(int x0, int y0, int x1, int y1, int width,
 		colour c, bool dotted, bool dashed);
-static bool print_fonts_plot_polygon(int *p, unsigned int n, colour fill);
+static bool print_fonts_plot_polygon(const int *p, unsigned int n, colour fill);
 static bool print_fonts_plot_fill(int x0, int y0, int x1, int y1, colour c);
 static bool print_fonts_plot_clip(int clip_x0, int clip_y0,
 		int clip_x1, int clip_y1);
-static bool print_fonts_plot_text(int x, int y, struct css_style *style,
+static bool print_fonts_plot_text(int x, int y, const struct css_style *style,
 		const char *text, size_t length, colour bg, colour c);
 static bool print_fonts_plot_disc(int x, int y, int radius, colour c,
                 bool filled);
 static bool print_fonts_plot_arc(int x, int y, int radius, int angle1, int angle2,
 		colour c);
 static bool print_fonts_plot_bitmap(int x, int y, int width, int height,
-		struct bitmap *bitmap, colour bg);
+		struct bitmap *bitmap, colour bg,
+		struct content *content);
 static bool print_fonts_plot_bitmap_tile(int x, int y, int width, int height,
 		struct bitmap *bitmap, colour bg,
-		bool repeat_x, bool repeat_y);
-static bool print_fonts_plot_group_start(const char *name);
-static bool print_fonts_plot_group_end(void);
+		bool repeat_x, bool repeat_y, struct content *content);
+static bool print_fonts_plot_path(const float *p, unsigned int n, colour fill, float width,
+		colour c, const float transform[6]);
 static void print_fonts_callback(void *context,
 		const char *font_name, unsigned int font_size,
 		const char *s8, unsigned short *s16, unsigned int n,
@@ -139,9 +140,11 @@ static const struct plotter_table print_fonts_plotters = {
 	print_fonts_plot_arc,
 	print_fonts_plot_bitmap,
 	print_fonts_plot_bitmap_tile,
-	print_fonts_plot_group_start,
-	print_fonts_plot_group_end,
-	NULL
+	NULL,
+	NULL,
+	NULL,
+	print_fonts_plot_path,
+	false
 };
 
 
@@ -196,7 +199,7 @@ void ro_gui_print_prepare(struct gui_window *g)
 
 	assert(g);
 
-	print_current_window = g;
+	ro_print_current_window = g;
 	print_prev_message = 0;
 
 	/* Read Printer Driver name */
@@ -285,11 +288,11 @@ bool ro_gui_print_apply(wimp_w w)
 		print_max_sheets = sheets;
 	else
 		print_max_sheets = -1;
-	print_current_window->option.background_images =
+	ro_print_current_window->option.background_images =
 			ro_gui_get_icon_selected_state(dialog_print,
 					ICON_PRINT_BG_IMAGES);
 
-	print_send_printsave(print_current_window->bw->current_content);
+	print_send_printsave(ro_print_current_window->bw->current_content);
 
 	return true;
 }
@@ -342,7 +345,7 @@ void print_send_printsave(struct content *c)
 		LOG(("xwimp_send_message: 0x%x: %s",
 				e->errnum, e->errmess));
 		warn_user("WimpError", e->errmess);
-		print_cleanup();
+		ro_print_cleanup();
 	}
 	print_prev_message = m.my_ref;
 }
@@ -380,16 +383,16 @@ bool print_send_printtypeknown(wimp_message *m)
  * \param m the bounced message
  */
 
-void print_save_bounce(wimp_message *m)
+void ro_print_save_bounce(wimp_message *m)
 {
 	if (m->my_ref == 0 || m->my_ref != print_prev_message)
 		return;
 
 	/* try to print anyway (we're graphics printing) */
-	if (print_current_window) {
-		print_document(print_current_window, "printer:");
+	if (ro_print_current_window) {
+		print_document(ro_print_current_window, "printer:");
 	}
-	print_cleanup();
+	ro_print_cleanup();
 }
 
 
@@ -399,7 +402,7 @@ void print_save_bounce(wimp_message *m)
  * \param m the message containing the error
  */
 
-void print_error(wimp_message *m)
+void ro_print_error(wimp_message *m)
 {
 	pdriver_message_print_error *p = (pdriver_message_print_error*)&m->data;
 	if (m->your_ref == 0 || m->your_ref != print_prev_message)
@@ -410,7 +413,7 @@ void print_error(wimp_message *m)
 	else
 		warn_user("PrintError", p->errmess);
 
-	print_cleanup();
+	ro_print_cleanup();
 }
 
 
@@ -420,15 +423,15 @@ void print_error(wimp_message *m)
  * \param m the message to handle
  */
 
-void print_type_odd(wimp_message *m)
+void ro_print_type_odd(wimp_message *m)
 {
 	if ((m->your_ref == 0 || m->your_ref == print_prev_message) &&
 						!print_in_background) {
 		/* reply to a previous message (ie printsave) */
-		if (print_current_window && print_send_printtypeknown(m)) {
-			print_document(print_current_window, "printer:");
+		if (ro_print_current_window && print_send_printtypeknown(m)) {
+			print_document(ro_print_current_window, "printer:");
 		}
-		print_cleanup();
+		ro_print_cleanup();
 	}
 	else {
 		/* broadcast message */
@@ -458,14 +461,14 @@ void print_type_odd(wimp_message *m)
  *   a fair bit)
  */
 
-bool print_ack(wimp_message *m)
+bool ro_print_ack(wimp_message *m)
 {
 	pdriver_info_type info_type;
 	pdriver_type type;
 	os_error *error;
 
 	if (m->your_ref == 0 || m->your_ref != print_prev_message ||
-			!print_current_window)
+			!ro_print_current_window)
 		return false;
 
 	/* read printer driver type */
@@ -474,15 +477,15 @@ bool print_ack(wimp_message *m)
 		LOG(("xpdriver_info: 0x%x: %s",
 				error->errnum, error->errmess));
 		warn_user("PrintError", error->errmess);
-		print_cleanup();
+		ro_print_cleanup();
 		return true;
 	}
 	type = info_type >> 16;
 
 	/* print to file */
-	if (!print_document(print_current_window,
+	if (!print_document(ro_print_current_window,
 			m->data.data_xfer.file_name)) {
-		print_cleanup();
+		ro_print_cleanup();
 		return true;
 	}
 
@@ -506,7 +509,7 @@ bool print_ack(wimp_message *m)
 	}
 	print_prev_message = m->my_ref;
 
-	print_cleanup();
+	ro_print_cleanup();
 	return true;
 }
 
@@ -517,13 +520,13 @@ bool print_ack(wimp_message *m)
  * \param m the message to handle
  */
 
-void print_dataload_bounce(wimp_message *m)
+void ro_print_dataload_bounce(wimp_message *m)
 {
 	if (m->your_ref == 0 || m->your_ref != print_prev_message)
 		return;
 
 	xosfile_delete(m->data.data_xfer.file_name, 0, 0, 0, 0, 0);
-	print_cleanup();
+	ro_print_cleanup();
 }
 
 
@@ -531,12 +534,12 @@ void print_dataload_bounce(wimp_message *m)
  * Cleanup after printing
  */
 
-void print_cleanup(void)
+void ro_print_cleanup(void)
 {
-	if (print_current_window)
-		print_current_window->option.background_images =
+	if (ro_print_current_window)
+		ro_print_current_window->option.background_images =
 							print_bg_images;
-	print_current_window = 0;
+	ro_print_current_window = NULL;
 	print_text_black = false;
 	print_prev_message = 0;
 	print_max_sheets = -1;
@@ -626,6 +629,7 @@ bool print_document(struct gui_window *g, const char *filename)
 	}
 
 	plot = ro_plotters;
+	plot.option_knockout = false;
 	ro_plot_set_scale(print_scale);
 	ro_gui_current_redraw_gui = g;
 	current_redraw_browser = NULL;  /* we don't want to print the
@@ -812,37 +816,66 @@ end:
 }
 
 
-bool print_fonts_plot_clg(colour c) { return true; }
+bool print_fonts_plot_clg(colour c)
+{
+	return true;
+}
 bool print_fonts_plot_rectangle(int x0, int y0, int width, int height,
 		int line_width, colour c, bool dotted, bool dashed)
-		{ return true; }
+{
+	return true;
+}
 bool print_fonts_plot_line(int x0, int y0, int x1, int y1, int width,
-		colour c, bool dotted, bool dashed) { return true; }
-bool print_fonts_plot_polygon(int *p, unsigned int n, colour fill)
-		{ return true; }
+		colour c, bool dotted, bool dashed)
+{
+	return true;
+}
+bool print_fonts_plot_polygon(const int *p, unsigned int n, colour fill)
+{
+	return true;
+}
 bool print_fonts_plot_fill(int x0, int y0, int x1, int y1, colour c)
-		{ return true; }
+{
+	return true;
+}
 bool print_fonts_plot_clip(int clip_x0, int clip_y0,
-		int clip_x1, int clip_y1) { return true; }
-bool print_fonts_plot_disc(int x, int y, int radius, colour colour, bool filled)
-		{ return true; }
+		int clip_x1, int clip_y1)
+{
+	return true;
+}
+bool print_fonts_plot_disc(int x, int y, int radius, colour colour,
+		bool filled)
+{
+	return true;
+}
 bool print_fonts_plot_arc(int x, int y, int radius, int angle1, int angle2,
 		colour c)
-		{ return true; }
+{
+	return true;
+}
 bool print_fonts_plot_bitmap(int x, int y, int width, int height,
-		struct bitmap *bitmap, colour bg) { return true; }
+		struct bitmap *bitmap, colour bg, struct content *content)
+{
+	return true;
+}
 bool print_fonts_plot_bitmap_tile(int x, int y, int width, int height,
 		struct bitmap *bitmap, colour bg,
-		bool repeat_x, bool repeat_y) { return true; }
-bool print_fonts_plot_group_start(const char *name) { return true; }
-bool print_fonts_plot_group_end(void) { return true; }
+		bool repeat_x, bool repeat_y, struct content *content)
+{
+	return true;
+}
+bool print_fonts_plot_path(const float *p, unsigned int n, colour fill, float width,
+		colour c, const float transform[6])
+{
+	return true;
+}
 
 
 /**
  * Plotter for text plotting during font listing.
  */
 
-bool print_fonts_plot_text(int x, int y, struct css_style *style,
+bool print_fonts_plot_text(int x, int y, const struct css_style *style,
 		const char *text, size_t length, colour bg, colour c)
 {
 	const char *font_family;
@@ -918,5 +951,3 @@ void print_fonts_callback(void *context,
 	print_fonts_count++;
 }
 
-
-#endif
