@@ -21,12 +21,12 @@
  * Free text search (implementation)
  */
 
+#include "utils/config.h"
+
 #include <ctype.h>
 #include <string.h>
-
 #include "oslib/hourglass.h"
 #include "oslib/wimp.h"
-
 #include "utils/config.h"
 #include "content/content.h"
 #include "desktop/browser.h"
@@ -42,8 +42,6 @@
 #include "utils/messages.h"
 #include "utils/utils.h"
 
-#ifdef WITH_SEARCH
-
 #ifndef NOF_ELEMENTS
 #define NOF_ELEMENTS(array) (sizeof(array)/sizeof(*(array)))
 #endif
@@ -61,13 +59,13 @@ struct list_entry {
 	struct list_entry *next;
 };
 
-struct gui_window *search_current_window = 0;
+struct gui_window *search_current_window = NULL;
 
-static char *search_string = 0;
+static char *search_string = NULL;
 static struct list_entry search_head = { 0, 0, NULL, NULL, NULL, NULL, NULL };
 static struct list_entry *search_found = &search_head;
-static struct list_entry *search_current = 0;
-static struct content *search_content = 0;
+static struct list_entry *search_current = NULL;
+static struct content *search_content = NULL;
 static bool search_prev_case_sens = false;
 
 #define RECENT_SEARCHES 8
@@ -81,10 +79,11 @@ wimp_menu *recent_search_menu = (wimp_menu *)&menu_recent;
 
 
 static void start_search(bool forwards);
-static void do_search(char *string, int string_len, bool case_sens,
+static void do_search(const char *string, int string_len, bool case_sens,
 		bool forwards);
 static const char *find_pattern(const char *string, int s_len,
-		const char *pattern, int p_len, bool case_sens, int *m_len);
+		const char *pattern, int p_len, bool case_sens, 
+		unsigned int *m_len);
 static bool find_occurrences_html(const char *pattern, int p_len,
 		struct box *cur, bool case_sens);
 static bool find_occurrences_text(const char *pattern, int p_len,
@@ -98,7 +97,7 @@ static void ro_gui_search_end(wimp_w w);
 static bool ro_gui_search_next(wimp_w w);
 static bool ro_gui_search_click(wimp_pointer *pointer);
 static bool ro_gui_search_keypress(wimp_key *key);
-static void ro_gui_search_add_recent(char *search);
+static void ro_gui_search_add_recent(const char *search);
 
 void ro_gui_search_init(void)
 {
@@ -156,7 +155,7 @@ bool ro_gui_search_click(wimp_pointer *pointer)
 	return false;
 }
 
-void ro_gui_search_add_recent(char *search)
+void ro_gui_search_add_recent(const char *search)
 {
 	char *tmp;
 	int i;
@@ -248,7 +247,7 @@ void ro_gui_search_prepare(struct gui_window *g)
 
 	search_current_window = g;
 
-	ro_gui_set_icon_string(dialog_search, ICON_SEARCH_TEXT, "");
+	ro_gui_set_icon_string(dialog_search, ICON_SEARCH_TEXT, "", true);
 	ro_gui_set_icon_selected_state(dialog_search,
 				ICON_SEARCH_CASE_SENSITIVE, false);
 	ro_gui_set_icon_selected_state(dialog_search,
@@ -327,7 +326,8 @@ bool ro_gui_search_keypress(wimp_key *key)
 void start_search(bool forwards)
 {
 	int string_len;
-	char *string;
+	const char *string;
+	int i = 0;
 
 	string = ro_gui_get_icon_string(dialog_search, ICON_SEARCH_TEXT);
 	assert(string);
@@ -335,7 +335,9 @@ void start_search(bool forwards)
 	ro_gui_search_add_recent(string);
 
 	string_len = strlen(string);
-	if (string_len <= 0) {
+	for(i = 0; i < string_len; i++)
+		if (string[i] != '#' && string[i] != '*') break;
+	if (i >= string_len) {
 		free_matches();
 		show_status(true);
 		ro_gui_set_icon_shaded_state(dialog_search,
@@ -414,7 +416,8 @@ void free_matches(void)
  * \param case_sens whether to perform a case sensitive search
  * \param forwards direction to search in
  */
-void do_search(char *string, int string_len, bool case_sens, bool forwards)
+void do_search(const char *string, int string_len, bool case_sens, 
+		bool forwards)
 {
 	struct rect bounds;
 	struct content *c;
@@ -550,7 +553,7 @@ void do_search(char *string, int string_len, bool case_sens, bool forwards)
  */
 
 const char *find_pattern(const char *string, int s_len, const char *pattern,
-		int p_len, bool case_sens, int *m_len)
+		int p_len, bool case_sens, unsigned int *m_len)
 {
 	struct { const char *ss, *s, *p; bool first; } context[16];
 	const char *ep = pattern + p_len;
@@ -643,7 +646,7 @@ const char *find_pattern(const char *string, int s_len, const char *pattern,
 	}
 
 	/* end of pattern reached */
-	*m_len = s - ss;
+	*m_len = max(s - ss, 1);
 	return ss;
 }
 
@@ -872,7 +875,6 @@ void show_all(bool all)
 void show_status(bool found)
 {
 	ro_gui_set_icon_string(dialog_search, ICON_SEARCH_STATUS,
-			found ? "" : messages_get("NotFound"));
+			found ? "" : messages_get("NotFound"), true);
 }
 
-#endif
