@@ -20,25 +20,24 @@
  * Single/Multi-line UTF-8 text area (implementation)
  */
 
+#include <inttypes.h>
 #include <stdbool.h>
 #include <stdint.h>
 #include <string.h>
-
 #include "swis.h"
-
 #include "oslib/colourtrans.h"
 #include "oslib/osbyte.h"
 #include "oslib/serviceinternational.h"
 #include "oslib/wimp.h"
 #include "oslib/wimpspriteop.h"
-
 #include "rufl.h"
-
 #include "riscos/gui.h"
+#include "riscos/oslib_pre7.h"
 #include "riscos/textarea.h"
 #include "riscos/ucstables.h"
 #include "riscos/wimp.h"
 #include "riscos/wimp_event.h"
+#include "riscos/wimputils.h"
 #include "utils/log.h"
 #include "utils/utf8.h"
 
@@ -276,7 +275,7 @@ bool textarea_update(uintptr_t self)
 	}
 
 	/* and open the window */
-	error = xwimp_open_window_nested((wimp_open *)&state, ta->parent,
+	error = xwimp_open_window_nested(PTR_WIMP_OPEN(&state), ta->parent,
 			wimp_CHILD_LINKS_PARENT_VISIBLE_BOTTOM_OR_LEFT
 					<< wimp_CHILD_XORIGIN_SHIFT |
 			wimp_CHILD_LINKS_PARENT_VISIBLE_TOP_OR_RIGHT
@@ -866,7 +865,7 @@ void textarea_reflow(struct text_area *ta, unsigned int line)
 		state.visible.x1 -= vscroll_width;
 
 		/* and reopen window */
-		error = xwimp_open_window_nested((wimp_open *)&state,
+		error = xwimp_open_window_nested(PTR_WIMP_OPEN(&state),
 				parent, linkage);
 		if (error) {
 			LOG(("xwimp_open_window_nested: 0x%x: %s",
@@ -906,7 +905,7 @@ bool textarea_mouse_click(wimp_pointer *pointer)
  */
 bool textarea_key_press(wimp_key *key)
 {
-	wchar_t c = (wchar_t)key->c;
+	uint32_t c = (uint32_t) key->c;
 	wimp_key keypress;
 	struct text_area *ta;
 	char utf8[7];
@@ -1091,7 +1090,7 @@ void textarea_redraw_internal(wimp_draw *redraw, bool update)
 		error = xcolourtrans_set_gcol(
 				(ta->flags & TEXTAREA_READONLY) ? 0xD9D9D900
 								: 0xFFFFFF00,
-				colourtrans_SET_BG | colourtrans_USE_ECFS,
+				colourtrans_SET_BG_GCOL | colourtrans_USE_ECFS_GCOL,
 				os_ACTION_OVERWRITE, 0, 0);
 		if (error) {
 			LOG(("xcolourtrans_set_gcol: 0x%x: %s",
