@@ -25,7 +25,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <wchar.h>
 #include "oslib/wimp.h"
 #include "content/urldb.h"
 #include "utils/log.h"
@@ -36,6 +35,7 @@
 #include "riscos/url_complete.h"
 #include "riscos/wimp.h"
 #include "riscos/wimp_event.h"
+#include "riscos/wimputils.h"
 #include "utils/utils.h"
 
 #define MAXIMUM_VISIBLE_LINES 7
@@ -52,7 +52,7 @@ static char *url_complete_original_url = NULL;
 static bool url_complete_memory_exhausted = false;
 
 static const char *url_complete_redraw[MAXIMUM_VISIBLE_LINES];
-static char url_complete_icon_null[] = "\0";
+static char url_complete_icon_null[] = "";
 static char url_complete_icon_sprite[12];
 static wimp_icon url_complete_icon;
 static wimp_icon url_complete_sprite;
@@ -69,7 +69,7 @@ static bool url_complete_callback(const char *url,
  */
 void ro_gui_url_complete_start(struct gui_window *g)
 {
-	char *url;
+	const char *url;
 
 	if ((!g->toolbar) || (!g->toolbar->display_url) ||
 			(g->window == url_complete_parent))
@@ -92,11 +92,11 @@ void ro_gui_url_complete_start(struct gui_window *g)
  * \param key  the key pressed (as UTF32 code or wimp key + bit31 set)
  * \return true to indicate keypress handled, false otherwise
  */
-bool ro_gui_url_complete_keypress(struct gui_window *g, wchar_t key)
+bool ro_gui_url_complete_keypress(struct gui_window *g, uint32_t key)
 {
 	wimp_window_state state;
 	char *match_url;
-	char *url;
+	const char *url;
 	int i, lines;
 	int old_selection;
 	int height;
@@ -203,7 +203,7 @@ bool ro_gui_url_complete_keypress(struct gui_window *g, wchar_t key)
 			return false;
 		}
 		url_complete_matches_reset = true;
-		ro_gui_url_complete_resize(g, (wimp_open *)&state);
+		ro_gui_url_complete_resize(g, PTR_WIMP_OPEN(&state));
 		url_complete_matches_reset = false;
 
 		/* redraw the relevant bits of the window */
@@ -296,12 +296,12 @@ bool ro_gui_url_complete_keypress(struct gui_window *g, wchar_t key)
 	if (url_complete_matches_selection == -1) {
 		ro_gui_set_icon_string(g->toolbar->toolbar_handle,
 				ICON_TOOLBAR_URL,
-				url_complete_original_url);
+				url_complete_original_url, true);
 	} else {
 		ro_gui_set_icon_string(g->toolbar->toolbar_handle,
 				ICON_TOOLBAR_URL,
 				url_complete_matches[
-					url_complete_matches_selection]);
+					url_complete_matches_selection], true);
 		free(url_complete_matched_string);
 		url_complete_matched_string = strdup(url_complete_matches[
 					url_complete_matches_selection]);
@@ -326,7 +326,7 @@ bool ro_gui_url_complete_keypress(struct gui_window *g, wchar_t key)
 		state.yscroll =
 			-((url_complete_matches_selection + 1) * 44) + height;
 
-	error = xwimp_open_window((wimp_open *)(&state));
+	error = xwimp_open_window(PTR_WIMP_OPEN(&state));
 	if (error) {
 		LOG(("xwimp_open_window: 0x%x: %s",
 				error->errnum, error->errmess));
@@ -561,7 +561,8 @@ void ro_gui_url_complete_redraw(wimp_draw *redraw)
 	/* no matches? no redraw */
 	if (!url_complete_matches) {
 		LOG(("Attempt to redraw with no matches made"));
-		ro_gui_user_redraw(redraw, false, NULL);
+		/* Fill is never used, so make it something obvious */
+		ro_gui_user_redraw(redraw, false, os_COLOUR_BLACK);
 		return;
 	}
 
@@ -585,7 +586,7 @@ void ro_gui_url_complete_redraw(wimp_draw *redraw)
 			url_complete_icon.extent.y1 = -line * 44;
 			url_complete_icon.extent.y0 = -(line + 1) * 44;
 			url_complete_icon.data.indirected_text.text =
-					url_complete_matches[line];
+					(char *)url_complete_matches[line];
 			url_complete_icon.data.indirected_text.size =
 					strlen(url_complete_matches[line]);
 
@@ -654,7 +655,7 @@ bool ro_gui_url_complete_click(wimp_pointer *pointer)
 	os_error *error;
 	int selection, old_selection;
 	struct gui_window *g;
-	char *url;
+	const char *url;
 
 	if ((mouse_x == pointer->pos.x) && (mouse_y == pointer->pos.y) &&
 			(!pointer->buttons))
@@ -719,13 +720,23 @@ bool ro_gui_url_complete_click(wimp_pointer *pointer)
 		ro_gui_set_icon_string(g->toolbar->toolbar_handle,
 				ICON_TOOLBAR_URL,
 				url_complete_matches[
+					url_complete_matches_selection], true);
+		global_history_add_recent(url_complete_matches[
 					url_complete_matches_selection]);
-		browser_window_go(g->bw,
+
+		/** \todo The interaction of components here is hideous */
+		/* Do NOT make any attempt to use any of the global url
+		 * completion variables after this call to browser_window_go.
+		 * They will be invalidated by (at least):
+		 *   + gui_window_set_url
+		 *   + destruction of (i)frames within the current page 
+		 * Any attempt to use them will probably result in a crash.
+		 */
+
+		 browser_window_go(g->bw,
 				url_complete_matches[
 					url_complete_matches_selection],
 				0, true);
-		global_history_add_recent(url_complete_matches[
-					url_complete_matches_selection]);
 		ro_gui_url_complete_close(NULL, 0);
 
 	/* Adjust just sets the text */
@@ -733,7 +744,7 @@ bool ro_gui_url_complete_click(wimp_pointer *pointer)
 		ro_gui_set_icon_string(g->toolbar->toolbar_handle,
 				ICON_TOOLBAR_URL,
 				url_complete_matches[
-					url_complete_matches_selection]);
+					url_complete_matches_selection], true);
 		ro_gui_url_complete_keypress(g, 0);
 	}
 	return true;
