@@ -1,5 +1,6 @@
 /*
  * Copyright 2004, 2005 Richard Wilson <info@tinct.net>
+ * Copyright 2008 John Tytgat <joty@netsurf-browser.org>
  *
  * This file is part of NetSurf, http://www.netsurf-browser.org/
  *
@@ -21,7 +22,7 @@
  */
 
 #include <assert.h>
-#include <ctype.h>
+#include <locale.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,6 +35,7 @@
 #include "oslib/wimpspriteop.h"
 #include "desktop/gui.h"
 #include "riscos/gui.h"
+#include "riscos/oslib_pre7.h"
 #include "riscos/theme.h"
 #include "riscos/wimp.h"
 #include "utils/log.h"
@@ -43,18 +45,18 @@
 
 static void ro_gui_wimp_cache_furniture_sizes(wimp_w w);
 static size_t ro_gui_strlen(const char *str);
+static int ro_gui_strncmp(const char *s1, const char *s2, size_t len);
 
 static wimpextend_furniture_sizes furniture_sizes;
 static wimp_w furniture_window = NULL;
 
-unsigned char last_sprite_found[16];
-
 /**
- * Gets the horzontal scrollbar height
+ * Gets the horizontal scrollbar height
  *
  * \param  w  the window to read (or NULL to read a cached value)
  */
-int ro_get_hscroll_height(wimp_w w) {
+int ro_get_hscroll_height(wimp_w w)
+{
 	ro_gui_wimp_cache_furniture_sizes(w);
 	return furniture_sizes.border_widths.y0;
 }
@@ -65,7 +67,8 @@ int ro_get_hscroll_height(wimp_w w) {
  *
  * \param  w  the window to read (or NULL to read a cached value)
  */
-int ro_get_vscroll_width(wimp_w w) {
+int ro_get_vscroll_width(wimp_w w)
+{
 	ro_gui_wimp_cache_furniture_sizes(w);
 	return furniture_sizes.border_widths.x1;
 }
@@ -76,7 +79,8 @@ int ro_get_vscroll_width(wimp_w w) {
  *
  * \param  w  the window to read (or NULL to read a cached value)
  */
-int ro_get_title_height(wimp_w w) {
+int ro_get_title_height(wimp_w w)
+{
 	ro_gui_wimp_cache_furniture_sizes(w);
 	return furniture_sizes.border_widths.y1;
 }
@@ -87,7 +91,8 @@ int ro_get_title_height(wimp_w w) {
  * \param  w  the window to cache information from
  * \return true on success, false on error (default values cached)
  */
-void ro_gui_wimp_cache_furniture_sizes(wimp_w w) {
+void ro_gui_wimp_cache_furniture_sizes(wimp_w w)
+{
 	os_error *error;
 
 	if (!w)
@@ -112,7 +117,8 @@ void ro_gui_wimp_cache_furniture_sizes(wimp_w w) {
  *
  * \param  mode  mode to read EIG factors for, or -1 for current
  */
-bool ro_gui_wimp_read_eig_factors(os_mode mode, int *xeig, int *yeig) {
+bool ro_gui_wimp_read_eig_factors(os_mode mode, int *xeig, int *yeig)
+{
 	os_error *error;
 
 	error = xos_read_mode_variable(mode, os_MODEVAR_XEIG_FACTOR, xeig, 0);
@@ -139,7 +145,8 @@ bool ro_gui_wimp_read_eig_factors(os_mode mode, int *xeig, int *yeig) {
  * \param  os_units  values to convert
  * \param  mode	     mode to use EIG factors for, or -1 for current
  */
-void ro_convert_os_units_to_pixels(os_coord *os_units, os_mode mode) {
+void ro_convert_os_units_to_pixels(os_coord *os_units, os_mode mode)
+{
 	int xeig = 1, yeig = 1;
 
 	ro_gui_wimp_read_eig_factors(mode, &xeig, &yeig);
@@ -154,7 +161,8 @@ void ro_convert_os_units_to_pixels(os_coord *os_units, os_mode mode) {
  * \param  pixels  values to convert
  * \param  mode	   mode to use EIG factors for, or -1 for current
  */
-void ro_convert_pixels_to_os_units(os_coord *pixels, os_mode mode) {
+void ro_convert_pixels_to_os_units(os_coord *pixels, os_mode mode)
+{
 	int xeig = 1, yeig = 1;
 
 	ro_gui_wimp_read_eig_factors(mode, &xeig, &yeig);
@@ -179,7 +187,8 @@ void ro_convert_pixels_to_os_units(os_coord *pixels, os_mode mode) {
  * \param  w  window handle
  * \param  i  icon handle
  */
-void ro_gui_force_redraw_icon(wimp_w w, wimp_i i) {
+void ro_gui_force_redraw_icon(wimp_w w, wimp_i i)
+{
 	wimp_icon_state ic;
 	os_error *error;
 
@@ -205,15 +214,20 @@ void ro_gui_force_redraw_icon(wimp_w w, wimp_i i) {
 
 
 /**
- * Read the contents of an icon.
+ * Read the contents of a text or sprite icon.
  *
  * \param  w  window handle
  * \param  i  icon handle
- * \return string in icon
+ * \return NUL terminated string in icon
+ *
+ * \todo this doesn't do local encoding -> UTF-8 to match what is done in
+ * ro_gui_set_icon_string.
  */
-char *ro_gui_get_icon_string(wimp_w w, wimp_i i) {
+const char *ro_gui_get_icon_string(wimp_w w, wimp_i i)
+{
 	wimp_icon_state ic;
 	os_error *error;
+	char *itext;
 
 	ic.w = w;
 	ic.i = i;
@@ -224,26 +238,55 @@ char *ro_gui_get_icon_string(wimp_w w, wimp_i i) {
 		warn_user("WimpError", error->errmess);
 		return NULL;
 	}
-	return ic.icon.data.indirected_text.text;
+	itext = (ic.icon.flags & wimp_ICON_INDIRECTED) ?
+		ic.icon.data.indirected_text.text
+		:
+		ic.icon.data.text;
+	/* Guarantee NUL termination.  */
+	itext[ro_gui_strlen(itext)] = '\0';
+	return itext;
 }
 
 
 /**
- * Set the contents of an icon to a string.
+ * Set the contents of a text or sprite icon to a string.
  *
  * \param  w	 window handle
  * \param  i	 icon handle
- * \param  text  string (UTF-8 encoded) (copied)
+ * \param  text  NUL terminated string (copied)
+ * \param  is_utf8 When true, the given string is UTF-8 encoded and will be
+ * converted to local encoding currently used by the Wimp. When false, the
+ * given string is assumed to be in local encoding in use by the Wimp.
  */
-void ro_gui_set_icon_string(wimp_w w, wimp_i i, const char *text) {
+void ro_gui_set_icon_string(wimp_w w, wimp_i i, const char *text, bool is_utf8)
+{
 	wimp_caret caret;
 	wimp_icon_state ic;
 	os_error *error;
-	int old_len, len;
+	size_t old_len, new_len;
 	char *local_text = NULL;
-	utf8_convert_ret err;
+	const char *text_for_icon;
+	char *dst_text;
+	size_t dst_max_len;
 	unsigned int button_type;
 
+	if (is_utf8) {
+		utf8_convert_ret err;
+		/* convert text to local encoding */
+		err = utf8_to_local_encoding(text, 0, &local_text);
+		if (err != UTF8_CONVERT_OK) {
+			/* A bad encoding should never happen, so assert this */
+			assert(err != UTF8_CONVERT_BADENC);
+			LOG(("utf8_to_enc failed"));
+			/* Paranoia */
+			local_text = NULL;
+		}
+		text_for_icon = local_text ? local_text : text;
+	}
+	else
+		text_for_icon = text;
+	new_len = strlen(text_for_icon);
+
 	/* get the icon data */
 	ic.w = w;
 	ic.i = i;
@@ -252,142 +295,63 @@ void ro_gui_set_icon_string(wimp_w w, wimp_i i, const char *text) {
 		LOG(("xwimp_get_icon_state: 0x%x: %s",
 				error->errnum, error->errmess));
 		warn_user("WimpError", error->errmess);
-		return;
+		goto exit;
 	}
 
-	/* convert text to local encoding */
-	err = utf8_to_local_encoding(text, 0, &local_text);
-	if (err != UTF8_CONVERT_OK) {
-		/* A bad encoding should never happen, so assert this */
-		assert(err != UTF8_CONVERT_BADENC);
-		LOG(("utf8_to_enc failed"));
-		/* Paranoia */
-		local_text = NULL;
+	if (ic.icon.flags & wimp_ICON_INDIRECTED) {
+		dst_text = ic.icon.data.indirected_text.text;
+		dst_max_len = ic.icon.data.indirected_text.size;
 	}
-	len = strlen(local_text ? local_text : text);
+	else {
+		dst_text = ic.icon.data.text;
+		dst_max_len = sizeof(ic.icon.data.text);
+	}
+	old_len = ro_gui_strlen(dst_text);
+	assert(old_len < dst_max_len);
 
 	/* check that the existing text is not the same as the updated text
 	 * to stop flicker */
-	if (ic.icon.data.indirected_text.size &&
-			!strncmp(ic.icon.data.indirected_text.text,
-			local_text ? local_text : text,
-			(unsigned int)ic.icon.data.indirected_text.size - 1)) {
-		free(local_text);
-		return;
-	}
+	if (dst_max_len) {
+		if (!ro_gui_strncmp(dst_text, text_for_icon, dst_max_len))
+			goto exit;
 
-	/* copy the text across */
-	old_len = ro_gui_strlen(ic.icon.data.indirected_text.text);
-	if (ic.icon.data.indirected_text.size) {
-		strncpy(ic.icon.data.indirected_text.text,
-			local_text ? local_text : text,
-			(unsigned int)ic.icon.data.indirected_text.size - 1);
-		ic.icon.data.indirected_text.text[
-				ic.icon.data.indirected_text.size - 1] = '\0';
-	}
+		/* copy the text across */
+		strncpy(dst_text, text_for_icon, dst_max_len - 1);
+		dst_text[dst_max_len - 1] = '\0';
 
-	/* handle the caret being in the icon */
-	button_type = (ic.icon.flags & wimp_ICON_BUTTON_TYPE)
-			>> wimp_ICON_BUTTON_TYPE_SHIFT;
-	if ((button_type == wimp_BUTTON_WRITABLE) ||
-			(button_type == wimp_BUTTON_WRITE_CLICK_DRAG)) {
-		error = xwimp_get_caret_position(&caret);
-		if (error) {
-			LOG(("xwimp_get_caret_position: 0x%x: %s",
-					error->errnum, error->errmess));
-			warn_user("WimpError", error->errmess);
-			free(local_text);
-			return;
-		}
-		if ((caret.w == w) && (caret.i == i)) {
-			if ((caret.index > len) || (caret.index == old_len))
-					caret.index = len;
-			error = xwimp_set_caret_position(w, i, caret.pos.x,
-				caret.pos.y, -1, caret.index);
+		/* handle the caret being in the icon */
+		button_type = (ic.icon.flags & wimp_ICON_BUTTON_TYPE)
+				>> wimp_ICON_BUTTON_TYPE_SHIFT;
+		if ((button_type == wimp_BUTTON_WRITABLE) ||
+				(button_type == wimp_BUTTON_WRITE_CLICK_DRAG)) {
+			error = xwimp_get_caret_position(&caret);
 			if (error) {
-				LOG(("xwimp_set_caret_position: 0x%x: %s",
+				LOG(("xwimp_get_caret_position: 0x%x: %s",
 						error->errnum, error->errmess));
 				warn_user("WimpError", error->errmess);
+				goto exit;
+			}
+			if ((caret.w == w) && (caret.i == i)) {
+				if ((size_t)caret.index > new_len
+						|| (size_t)caret.index == old_len)
+					caret.index = new_len;
+				error = xwimp_set_caret_position(w, i, caret.pos.x,
+						caret.pos.y, -1, caret.index);
+				if (error) {
+					LOG(("xwimp_set_caret_position: 0x%x: %s",
+							error->errnum, error->errmess));
+					warn_user("WimpError", error->errmess);
+				}
 			}
 		}
+		ro_gui_redraw_icon(w, i);
 	}
-	ro_gui_redraw_icon(w, i);
 
+exit:
 	free(local_text);
 }
 
 
-/**
- * Set the contents of an icon to a string.
- *
- * \param  w	 window handle
- * \param  i	 icon handle
- * \param  text  string (in local encoding) (copied)
- */
-void ro_gui_set_icon_string_le(wimp_w w, wimp_i i, const char *text) {
-	wimp_caret caret;
-	wimp_icon_state ic;
-	os_error *error;
-	int old_len, len;
-	unsigned int button_type;
-
-	/* get the icon data */
-	ic.w = w;
-	ic.i = i;
-	error = xwimp_get_icon_state(&ic);
-	if (error) {
-		LOG(("xwimp_get_icon_state: 0x%x: %s",
-				error->errnum, error->errmess));
-		warn_user("WimpError", error->errmess);
-		return;
-	}
-
-	/* check that the existing text is not the same as the updated text
-	 * to stop flicker */
-	if (ic.icon.data.indirected_text.size &&
-			!strncmp(ic.icon.data.indirected_text.text,
-			text,
-			(unsigned int)ic.icon.data.indirected_text.size - 1))
-		return;
-
-	/* copy the text across */
-	old_len = strlen(ic.icon.data.indirected_text.text);
-	if (ic.icon.data.indirected_text.size) {
-		strncpy(ic.icon.data.indirected_text.text, text,
-			(unsigned int)ic.icon.data.indirected_text.size - 1);
-		ic.icon.data.indirected_text.text[
-				ic.icon.data.indirected_text.size - 1] = '\0';
-	}
-
-	/* handle the caret being in the icon */
-	button_type = (ic.icon.flags & wimp_ICON_BUTTON_TYPE)
-			>> wimp_ICON_BUTTON_TYPE_SHIFT;
-	if ((button_type == wimp_BUTTON_WRITABLE) ||
-			(button_type == wimp_BUTTON_WRITE_CLICK_DRAG)) {
-		error = xwimp_get_caret_position(&caret);
-		if (error) {
-			LOG(("xwimp_get_caret_position: 0x%x: %s",
-					error->errnum, error->errmess));
-			warn_user("WimpError", error->errmess);
-			return;
-		}
-		if ((caret.w == w) && (caret.i == i)) {
-			len = strlen(text);
-			if ((caret.index > len) || (caret.index == old_len))
-					caret.index = len;
-			error = xwimp_set_caret_position(w, i, caret.pos.x,
-				caret.pos.y, -1, caret.index);
-			if (error) {
-				LOG(("xwimp_set_caret_position: 0x%x: %s",
-						error->errnum, error->errmess));
-				warn_user("WimpError", error->errmess);
-			}
-		}
-	}
-	ro_gui_redraw_icon(w, i);
-}
-
-
 /**
  * Set the contents of an icon to a number.
  *
@@ -395,10 +359,17 @@ void ro_gui_set_icon_string_le(wimp_w w, wimp_i i, const char *text) {
  * \param  i	  icon handle
  * \param  value  value
  */
-void ro_gui_set_icon_integer(wimp_w w, wimp_i i, int value) {
+void ro_gui_set_icon_integer(wimp_w w, wimp_i i, int value)
+{
 	char buffer[20]; // Big enough for 64-bit int
+
+	setlocale(LC_NUMERIC, "");
+
 	sprintf(buffer, "%d", value);
-	ro_gui_set_icon_string(w, i, buffer);
+
+	setlocale(LC_NUMERIC, "C");
+
+	ro_gui_set_icon_string(w, i, buffer, true);
 }
 
 
@@ -409,9 +380,12 @@ void ro_gui_set_icon_integer(wimp_w w, wimp_i i, int value) {
  * \param  i	  icon handle
  * \param  value  value
  */
-void ro_gui_set_icon_decimal(wimp_w w, wimp_i i, int value, int decimal_places) {
+void ro_gui_set_icon_decimal(wimp_w w, wimp_i i, int value, int decimal_places)
+{
 	char buffer[20]; // Big enough for 64-bit int
 
+	setlocale(LC_NUMERIC, "");
+
 	switch (decimal_places) {
 		case 0:
 			sprintf(buffer, "%d", value);
@@ -423,10 +397,13 @@ void ro_gui_set_icon_decimal(wimp_w w, wimp_i i, int value, int decimal_places)
 			sprintf(buffer, "%.2f", (float)value / 100);
 			break;
 		default:
-			assert("Unsupported decimal format");
+			assert(!"Unsupported decimal format");
 			break;
 	}
-	ro_gui_set_icon_string(w, i, buffer);
+
+	setlocale(LC_NUMERIC, "C");
+
+	ro_gui_set_icon_string(w, i, buffer, true);
 }
 
 
@@ -437,14 +414,20 @@ void ro_gui_set_icon_decimal(wimp_w w, wimp_i i, int value, int decimal_places)
  * \param  i	  icon handle
  * \param  value  value
  */
-int ro_gui_get_icon_decimal(wimp_w w, wimp_i i, int decimal_places) {
+int ro_gui_get_icon_decimal(wimp_w w, wimp_i i, int decimal_places)
+{
 	double value;
 	int multiple = 1;
 
 	for (; decimal_places > 0; decimal_places--)
 		multiple *= 10;
 
+	setlocale(LC_NUMERIC, "");
+
 	value = atof(ro_gui_get_icon_string(w, i)) * multiple;
+
+	setlocale(LC_NUMERIC, "C");
+
 	return (int)value;
 }
 
@@ -456,7 +439,8 @@ int ro_gui_get_icon_decimal(wimp_w w, wimp_i i, int decimal_places) {
  * \param  i	 icon handle
  * \param  state selected state
  */
-void ro_gui_set_icon_selected_state(wimp_w w, wimp_i i, bool state) {
+void ro_gui_set_icon_selected_state(wimp_w w, wimp_i i, bool state)
+{
 	os_error *error;
 	if (ro_gui_get_icon_selected_state(w, i) == state) return;
 	error = xwimp_set_icon_state(w, i,
@@ -474,7 +458,8 @@ void ro_gui_set_icon_selected_state(wimp_w w, wimp_i i, bool state) {
  * \param  w	 window handle
  * \param  i	 icon handle
  */
-bool ro_gui_get_icon_selected_state(wimp_w w, wimp_i i) {
+bool ro_gui_get_icon_selected_state(wimp_w w, wimp_i i)
+{
 	os_error *error;
 	wimp_icon_state ic;
 	ic.w = w;
@@ -497,7 +482,8 @@ bool ro_gui_get_icon_selected_state(wimp_w w, wimp_i i) {
  * \param  i	 icon handle
  * \param  state shaded state
  */
-void ro_gui_set_icon_shaded_state(wimp_w w, wimp_i i, bool state) {
+void ro_gui_set_icon_shaded_state(wimp_w w, wimp_i i, bool state)
+{
 	wimp_caret caret;
 	os_error *error;
 
@@ -544,7 +530,8 @@ void ro_gui_set_icon_shaded_state(wimp_w w, wimp_i i, bool state) {
  * \param  w	 window handle
  * \param  i	 icon handle
  */
-bool ro_gui_get_icon_shaded_state(wimp_w w, wimp_i i) {
+bool ro_gui_get_icon_shaded_state(wimp_w w, wimp_i i)
+{
 	wimp_icon_state ic;
 	ic.w = w;
 	ic.i = i;
@@ -560,7 +547,8 @@ bool ro_gui_get_icon_shaded_state(wimp_w w, wimp_i i) {
  * \param  i	icon handle
  * \param  type button type
  */
-void ro_gui_set_icon_button_type(wimp_w w, wimp_i i, int type) {
+void ro_gui_set_icon_button_type(wimp_w w, wimp_i i, int type)
+{
 	os_error *error;
 	error = xwimp_set_icon_state(w, i, wimp_ICON_BUTTON_TYPE,
 			(type << wimp_ICON_BUTTON_TYPE_SHIFT));
@@ -617,7 +605,8 @@ void ro_gui_set_icon_sprite(wimp_w w, wimp_i i, osspriteop_area *area,
  * \param  w	 window handle
  * \param  text  new title (copied)
  */
-void ro_gui_set_window_title(wimp_w w, const char *text) {
+void ro_gui_set_window_title(wimp_w w, const char *text)
+{
 	wimp_window_info_base window;
 	os_error *error;
 	char *title_local_enc;
@@ -672,7 +661,8 @@ void ro_gui_set_window_title(wimp_w w, const char *text) {
  * \w	the window to place the caret in
  * \return true if the caret was placed, false otherwise
  */
-bool ro_gui_set_caret_first(wimp_w w) {
+bool ro_gui_set_caret_first(wimp_w w)
+{
 	int icon, b;
 	wimp_window_state win_state;
 	wimp_window_info_base window;
@@ -796,10 +786,11 @@ osspriteop_area *ro_gui_load_sprite_file(const char *pathname)
 
 bool ro_gui_wimp_sprite_exists(const char *sprite)
 {
+	static char last_sprite_found[16];
 	os_error *error;
 
 	/* make repeated calls fast */
-	if (!strncmp(sprite, last_sprite_found, 16))
+	if (!strncmp(sprite, last_sprite_found, sizeof(last_sprite_found)))
 		return true;
 
 	/* fallback if not known to exist */
@@ -812,7 +803,7 @@ bool ro_gui_wimp_sprite_exists(const char *sprite)
 		}
 		return false;
 	}
-  	snprintf(last_sprite_found, 16, sprite);
+  	snprintf(last_sprite_found, sizeof(last_sprite_found), sprite);
 	return true;
 }
 
@@ -834,11 +825,11 @@ os_error *ro_gui_wimp_get_sprite(const char *name, osspriteop_header **sprite)
 	if (error) return error;
 
 	error = xosspriteop_select_sprite(osspriteop_USER_AREA,
-			rom_base, (osspriteop_id)name, sprite);
+			ram_base, (osspriteop_id)name, sprite);
 
 	if (error && error->errnum == error_SPRITE_OP_DOESNT_EXIST)
 		error = xosspriteop_select_sprite(osspriteop_USER_AREA,
-				ram_base, (osspriteop_id)name, sprite);
+				rom_base, (osspriteop_id)name, sprite);
 
 	return error;
 }
@@ -852,7 +843,8 @@ os_error *ro_gui_wimp_get_sprite(const char *name, osspriteop_header **sprite)
  */
 
 void ro_gui_user_redraw(wimp_draw *redraw, bool user_fill,
-		os_colour user_colour) {
+		os_colour user_colour)
+{
 	os_error *error;
 	osbool more;
 
@@ -866,7 +858,7 @@ void ro_gui_user_redraw(wimp_draw *redraw, bool user_fill,
 	while (more) {
 		if (user_fill) {
 			error = xcolourtrans_set_gcol(user_colour,
-					colourtrans_SET_BG,
+					colourtrans_SET_BG_GCOL,
 					os_ACTION_OVERWRITE, 0, 0);
 			if (error) {
 				LOG(("xcolourtrans_set_gcol: 0x%x: %s",
@@ -894,7 +886,8 @@ void ro_gui_user_redraw(wimp_draw *redraw, bool user_fill,
  * \param  xor_mask  the furniture flags to toggle
  */
 void ro_gui_wimp_update_window_furniture(wimp_w w, wimp_window_flags bic_mask,
-		wimp_window_flags xor_mask) {
+		wimp_window_flags xor_mask)
+{
 	wimp_window_state state;
 	wimp_w parent;
 	bits linkage;
@@ -942,7 +935,8 @@ void ro_gui_wimp_update_window_furniture(wimp_w w, wimp_window_flags bic_mask,
  * \param  w	     the window to modify
  * \param  mask	     the furniture flags to check
  */
-bool ro_gui_wimp_check_window_furniture(wimp_w w, wimp_window_flags mask) {
+bool ro_gui_wimp_check_window_furniture(wimp_w w, wimp_window_flags mask)
+{
 	wimp_window_state state;
 	os_error *error;
 
@@ -965,13 +959,36 @@ bool ro_gui_wimp_check_window_furniture(wimp_w w, wimp_window_flags mask) {
  */
 size_t ro_gui_strlen(const char *str)
 {
-	size_t len = 0;
+	const char *str_begin;
 
 	if (str == NULL)
 		return 0;
 
-	while (*(str++) >= ' ')
-		len++;
+	for (str_begin = str; *str++ >= ' '; /* */)
+		/* */;
 
-	return len;
+	return str - str_begin - 1;
+}
+
+/**
+ * RO GUI-specific strncmp, for control character terminated strings
+ *
+ * \param s1 The first string for comparison
+ * \param s2 The second string for comparison
+ * \param len Maximum number of bytes to be checked
+ * \return 0 for equal strings up to len bytes; pos for s1 being bigger than
+ * s2; neg for s1 being smaller than s2.
+ */
+int ro_gui_strncmp(const char *s1, const char *s2, size_t len)
+{
+	while (len--) {
+		char c1 = *s1++;
+		char c2 = *s2++;
+		if (c1 < ' ' || c2 < ' ')
+			return (c1 < ' ' ? 0 : c1) - (c2 < ' ' ? 0 : c2);
+		int diff = c1 - c2;
+		if (diff)
+			return diff;
+	}
+	return 0;
 }
