@@ -21,6 +21,7 @@
  */
 
 #include <assert.h>
+#include <inttypes.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,6 +36,7 @@
 #include "riscos/ucstables.h"
 #include "riscos/wimp.h"
 #include "riscos/wimp_event.h"
+#include "riscos/wimputils.h"
 #include "utils/log.h"
 #include "utils/utils.h"
 
@@ -119,7 +121,8 @@ static wimp_w ro_gui_wimp_event_submenu;
  * \param w	the window to memorise
  * \return true on success, false on memory exhaustion or for an unknown window
  */
-bool ro_gui_wimp_event_memorise(wimp_w w) {
+bool ro_gui_wimp_event_memorise(wimp_w w)
+{
 	struct event_window *window;
 	struct icon_event *event;
 	bool error = false;
@@ -163,7 +166,8 @@ bool ro_gui_wimp_event_memorise(wimp_w w) {
  * \param w	the window to restore
  * \return true on success, false for an unknown window
  */
-bool ro_gui_wimp_event_restore(wimp_w w) {
+bool ro_gui_wimp_event_restore(wimp_w w)
+{
 	struct event_window *window;
 	struct icon_event *event;
 
@@ -177,7 +181,7 @@ bool ro_gui_wimp_event_restore(wimp_w w) {
 			case EVENT_TEXT_FIELD:
 				if (event->previous_value.textual)
 					ro_gui_set_icon_string(window->w, event->i,
-						event->previous_value.textual);
+						event->previous_value.textual, true);
 				break;
 			case EVENT_CHECKBOX:
 			case EVENT_RADIO:
@@ -201,7 +205,8 @@ bool ro_gui_wimp_event_restore(wimp_w w) {
  * \param w	the window to memorise
  * \return true on success, false for an unknown window
  */
-bool ro_gui_wimp_event_validate(wimp_w w) {
+bool ro_gui_wimp_event_validate(wimp_w w)
+{
 	struct event_window *window;
 	struct icon_event *event;
 	int value;
@@ -235,7 +240,8 @@ bool ro_gui_wimp_event_validate(wimp_w w) {
  *
  * \param w	the window to free resources for
  */
-void ro_gui_wimp_event_finalise(wimp_w w) {
+void ro_gui_wimp_event_finalise(wimp_w w)
+{
 	struct event_window *window;
 	struct icon_event *event;
 
@@ -271,7 +277,8 @@ void ro_gui_wimp_event_finalise(wimp_w w) {
  * \param help_prefix	the prefix to associate with the window (used directly)
  * \return true on success, or NULL for memory exhaustion
  */
-bool ro_gui_wimp_event_set_help_prefix(wimp_w w, const char *help_prefix) {
+bool ro_gui_wimp_event_set_help_prefix(wimp_w w, const char *help_prefix)
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -288,7 +295,8 @@ bool ro_gui_wimp_event_set_help_prefix(wimp_w w, const char *help_prefix) {
  * \param w	the window to get the prefix for
  * \return the associated prefix, or NULL
  */
-const char *ro_gui_wimp_event_get_help_prefix(wimp_w w) {
+const char *ro_gui_wimp_event_get_help_prefix(wimp_w w)
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_find_window(w);
@@ -304,7 +312,8 @@ const char *ro_gui_wimp_event_get_help_prefix(wimp_w w) {
  * \param w	the window to associate the data with
  * \param user	the data to associate
  */
-bool ro_gui_wimp_event_set_user_data(wimp_w w, void *user) {
+bool ro_gui_wimp_event_set_user_data(wimp_w w, void *user)
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -322,7 +331,8 @@ bool ro_gui_wimp_event_set_user_data(wimp_w w, void *user) {
  * \param w	the window to retrieve the data for
  * \return the associated data, or NULL
  */
-void *ro_gui_wimp_event_get_user_data(wimp_w w) {
+void *ro_gui_wimp_event_get_user_data(wimp_w w)
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_find_window(w);
@@ -342,7 +352,8 @@ void *ro_gui_wimp_event_get_user_data(wimp_w w) {
  * \return true if the event was handled, false otherwise
  */
 bool ro_gui_wimp_event_menu_selection(wimp_w w, wimp_i i, wimp_menu *menu,
-		wimp_selection *selection) {
+		wimp_selection *selection)
+{
 	struct event_window *window;
 	struct icon_event *event;
 	wimp_menu_entry *menu_entry;
@@ -371,8 +382,8 @@ bool ro_gui_wimp_event_menu_selection(wimp_w w, wimp_i i, wimp_menu *menu,
 	if (menu_entry->menu_flags & wimp_MENU_TICKED)
 		return true;
 
-	ro_gui_set_icon_string_le(window->w, event->data.menu_gright.field,
-			menu_entry->data.indirected_text.text);
+	ro_gui_set_icon_string(window->w, event->data.menu_gright.field,
+			menu_entry->data.indirected_text.text, false);
 	ro_gui_wimp_event_prepare_menu(window->w, event);
 	if (window->menu_selection)
 		window->menu_selection(window->w, event->i);
@@ -431,7 +442,8 @@ bool ro_gui_wimp_event_menu_selection(wimp_w w, wimp_i i, wimp_menu *menu,
  * \param pointer	the current pointer state
  * \return true if the event was handled, false otherwise
  */
-bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer) {
+bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer)
+{
 	struct event_window *window;
 	struct icon_event *event;
 	wimp_w w;
@@ -517,7 +529,7 @@ bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer) {
 						pointer->w);
 				ro_gui_menu_closed(false);
 				gui_poll(true);
-				error = xwimp_open_window((wimp_open *) &open);
+				error = xwimp_open_window(PTR_WIMP_OPEN(&open));
 				if (error) {
 					LOG(("xwimp_open_window: 0x%x: %s",
 							error->errnum, error->errmess));
@@ -577,9 +589,10 @@ bool ro_gui_wimp_event_mouse_click(wimp_pointer *pointer) {
  * /param w	the window owning the menu
  * /param event	the icon event owning the menu
  */
-void ro_gui_wimp_event_prepare_menu(wimp_w w, struct icon_event *event) {
+void ro_gui_wimp_event_prepare_menu(wimp_w w, struct icon_event *event)
+{
 	int i;
-	char *text;
+	const char *text;
 	unsigned int button_type;
 	wimp_icon_state ic;
 	wimp_menu *menu;
@@ -619,7 +632,9 @@ void ro_gui_wimp_event_prepare_menu(wimp_w w, struct icon_event *event) {
  * /param window	the window to perform the action on
  * /param state		the mouse button state
  */
-void ro_gui_wimp_event_ok_click(struct event_window *window, wimp_mouse_state state) {
+void ro_gui_wimp_event_ok_click(struct event_window *window,
+		wimp_mouse_state state)
+{
 	struct icon_event *search;
 
 	for (search = window->first; search; search = search->next)
@@ -650,16 +665,17 @@ void ro_gui_wimp_event_ok_click(struct event_window *window, wimp_mouse_state st
  * \param key	the key state
  * \return true if keypress handled, false otherwise
  */
-bool ro_gui_wimp_event_keypress(wimp_key *key) {
-	static int *ucstable = NULL;
+bool ro_gui_wimp_event_keypress(wimp_key *key)
+{
+	static const int *ucstable = NULL;
 	static int alphabet = 0;
-	static wchar_t wc = 0;	/* buffer for UTF8 alphabet */
+	static uint32_t wc = 0;	/* buffer for UTF8 alphabet */
 	static int shift = 0;
 	struct event_window *window;
 	struct icon_event *event;
 	wimp_pointer pointer;
 	wimp_key k;
-	wchar_t c = (wchar_t)key->c;
+	uint32_t c = (uint32_t) key->c;
 	int t_alphabet;
 	os_error *error;
 
@@ -703,22 +719,25 @@ bool ro_gui_wimp_event_keypress(wimp_key *key) {
 	}
 
 	if (t_alphabet != alphabet) {
+		void *ostable;
 		osbool unclaimed;
 		/* Alphabet has changed, so read UCS table location */
 		alphabet = t_alphabet;
 
 		error = xserviceinternational_get_ucs_conversion_table(
-						alphabet, &unclaimed,
-						(void**)&ucstable);
-		if (error) {
+						alphabet, &unclaimed, &ostable);
+		if (error != NULL) {
 			LOG(("failed reading UCS conversion table: 0x%x: %s",
 					error->errnum, error->errmess));
-			/* try using our own table instead */
+			/* Try using our own table instead */
 			ucstable = ucstable_from_alphabet(alphabet);
-		}
-		if (unclaimed)
+		} else if (unclaimed) {
 			/* Service wasn't claimed so use our own ucstable */
 			ucstable = ucstable_from_alphabet(alphabet);
+		} else {
+			/* Use the table provided by the OS */
+			ucstable = ostable;
+		}
 	}
 
 	if (c < 256) {
@@ -847,7 +866,8 @@ bool ro_gui_wimp_event_keypress(wimp_key *key) {
  *
  * \param open	the window open request
  */
-bool ro_gui_wimp_event_open_window(wimp_open *open) {
+bool ro_gui_wimp_event_open_window(wimp_open *open)
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_find_window(open->w);
@@ -864,7 +884,8 @@ bool ro_gui_wimp_event_open_window(wimp_open *open) {
  *
  * \param w	the window being closed
  */
-bool ro_gui_wimp_event_close_window(wimp_w w) {
+bool ro_gui_wimp_event_close_window(wimp_w w)
+{
 	struct event_window *window;
 
 	LOG(("Close event received for window 0x%x", (unsigned int)w));
@@ -884,7 +905,8 @@ bool ro_gui_wimp_event_close_window(wimp_w w) {
  *
  * \param redraw	the window redraw request
  */
-bool ro_gui_wimp_event_redraw_window(wimp_draw *redraw) {
+bool ro_gui_wimp_event_redraw_window(wimp_draw *redraw)
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_find_window(redraw->w);
@@ -899,8 +921,10 @@ bool ro_gui_wimp_event_redraw_window(wimp_draw *redraw) {
 /**
  * Register a numeric field to be automatically handled
  */
-bool ro_gui_wimp_event_register_numeric_field(wimp_w w, wimp_i i, wimp_i up, wimp_i down,
-		int min, int max, int stepping, int decimal_places) {
+bool ro_gui_wimp_event_register_numeric_field(wimp_w w, wimp_i i,
+		wimp_i up, wimp_i down,
+		int min, int max, int stepping, int decimal_places)
+{
 	struct icon_event *event;
 
 	event = ro_gui_wimp_event_get_event(w, i, EVENT_NUMERIC_FIELD);
@@ -941,7 +965,9 @@ bool ro_gui_wimp_event_register_text_field(wimp_w w, wimp_i i) {
 /**
  * Register an icon menu to be automatically handled
  */
-bool ro_gui_wimp_event_register_menu_gright(wimp_w w, wimp_i i, wimp_i gright, wimp_menu *menu) {
+bool ro_gui_wimp_event_register_menu_gright(wimp_w w, wimp_i i,
+		wimp_i gright, wimp_menu *menu)
+{
 	struct icon_event *event;
 
 	event = ro_gui_wimp_event_get_event(w, gright, EVENT_MENU_GRIGHT);
@@ -957,7 +983,8 @@ bool ro_gui_wimp_event_register_menu_gright(wimp_w w, wimp_i i, wimp_i gright, w
 /**
  * Register a checkbox to be automatically handled
  */
-bool ro_gui_wimp_event_register_checkbox(wimp_w w, wimp_i i) {
+bool ro_gui_wimp_event_register_checkbox(wimp_w w, wimp_i i)
+{
 	struct icon_event *event;
 
 	event = ro_gui_wimp_event_get_event(w, i, EVENT_CHECKBOX);
@@ -970,7 +997,8 @@ bool ro_gui_wimp_event_register_checkbox(wimp_w w, wimp_i i) {
 /**
  * Register a group of radio icons to be automatically handled
  */
-bool ro_gui_wimp_event_register_radio(wimp_w w, wimp_i *i) {
+bool ro_gui_wimp_event_register_radio(wimp_w w, wimp_i *i)
+{
 	struct event_window *window;
 	struct icon_event *event;
 
@@ -984,7 +1012,7 @@ bool ro_gui_wimp_event_register_radio(wimp_w w, wimp_i *i) {
 		if (!event)
 			return false;
 		event->data.radio_group = window->max_radio_group;
-		*i++;
+		i++;
 	}
 	return true;
 }
@@ -994,7 +1022,8 @@ bool ro_gui_wimp_event_register_radio(wimp_w w, wimp_i *i) {
  * Register a function to be called when a particular button is pressed.
  */
 bool ro_gui_wimp_event_register_button(wimp_w w, wimp_i i,
-		void (*callback)(wimp_pointer *pointer)) {
+		void (*callback)(wimp_pointer *pointer))
+{
 	struct icon_event *event;
 
 	event = ro_gui_wimp_event_get_event(w, i, EVENT_BUTTON);
@@ -1008,7 +1037,8 @@ bool ro_gui_wimp_event_register_button(wimp_w w, wimp_i i,
 /**
  * Register a function to be called for the Cancel action on a window.
  */
-bool ro_gui_wimp_event_register_cancel(wimp_w w, wimp_i i) {
+bool ro_gui_wimp_event_register_cancel(wimp_w w, wimp_i i)
+{
 	struct icon_event *event;
 
 	event = ro_gui_wimp_event_get_event(w, i, EVENT_CANCEL);
@@ -1022,7 +1052,8 @@ bool ro_gui_wimp_event_register_cancel(wimp_w w, wimp_i i) {
  * Register a function to be called for the OK action on a window.
  */
 bool ro_gui_wimp_event_register_ok(wimp_w w, wimp_i i,
-		bool (*callback)(wimp_w w)) {
+		bool (*callback)(wimp_w w))
+{
 	struct event_window *window;
 	struct icon_event *event;
 
@@ -1043,7 +1074,8 @@ bool ro_gui_wimp_event_register_ok(wimp_w w, wimp_i i,
  * in a window that don't have registered actions.
  */
 bool ro_gui_wimp_event_register_mouse_click(wimp_w w,
-		bool (*callback)(wimp_pointer *pointer)) {
+		bool (*callback)(wimp_pointer *pointer))
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -1064,7 +1096,8 @@ bool ro_gui_wimp_event_register_mouse_click(wimp_w w,
  *
  */
 bool ro_gui_wimp_event_register_keypress(wimp_w w,
-		bool (*callback)(wimp_key *key)) {
+		bool (*callback)(wimp_key *key))
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -1079,7 +1112,8 @@ bool ro_gui_wimp_event_register_keypress(wimp_w w,
  * Register a function to be called for all window opening requests.
  */
 bool ro_gui_wimp_event_register_open_window(wimp_w w,
-		void (*callback)(wimp_open *open)) {
+		void (*callback)(wimp_open *open))
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -1093,7 +1127,8 @@ bool ro_gui_wimp_event_register_open_window(wimp_w w,
  * Register a function to be called after the window has been closed.
  */
 bool ro_gui_wimp_event_register_close_window(wimp_w w,
-		void (*callback)(wimp_w w)) {
+		void (*callback)(wimp_w w))
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -1107,7 +1142,8 @@ bool ro_gui_wimp_event_register_close_window(wimp_w w,
  * Register a function to be called for all window redraw operations.
  */
 bool ro_gui_wimp_event_register_redraw_window(wimp_w w,
-		void (*callback)(wimp_draw *redraw)) {
+		void (*callback)(wimp_draw *redraw))
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -1121,7 +1157,8 @@ bool ro_gui_wimp_event_register_redraw_window(wimp_w w,
  * Register a function to be called following a menu selection.
  */
 bool ro_gui_wimp_event_register_menu_selection(wimp_w w,
-		void (*callback)(wimp_w w, wimp_i i)) {
+		void (*callback)(wimp_w w, wimp_i i))
+{
 	struct event_window *window;
 
 	window = ro_gui_wimp_event_get_window(w);
@@ -1137,7 +1174,8 @@ bool ro_gui_wimp_event_register_menu_selection(wimp_w w,
  *
  * \param w	the window to find data for
  */
-struct event_window *ro_gui_wimp_event_get_window(wimp_w w) {
+struct event_window *ro_gui_wimp_event_get_window(wimp_w w)
+{
 	struct event_window *window;
 	int h;
 
@@ -1167,7 +1205,8 @@ struct event_window *ro_gui_wimp_event_get_window(wimp_w w) {
  * \return  pointer to the event data or NULL if not found
  */
 
-struct event_window *ro_gui_wimp_event_remove_window(wimp_w w) {
+struct event_window *ro_gui_wimp_event_remove_window(wimp_w w)
+{
 	struct event_window **prev;
 	int h = WIN_HASH(w);
 
@@ -1193,7 +1232,8 @@ struct event_window *ro_gui_wimp_event_remove_window(wimp_w w) {
  *
  * \param w	the window to find data for
  */
-struct event_window *ro_gui_wimp_event_find_window(wimp_w w) {
+struct event_window *ro_gui_wimp_event_find_window(wimp_w w)
+{
 	struct event_window *window;
 	int h = WIN_HASH(w);
 
@@ -1205,7 +1245,9 @@ struct event_window *ro_gui_wimp_event_find_window(wimp_w w) {
 	return NULL;
 }
 
-struct icon_event *ro_gui_wimp_event_get_event(wimp_w w, wimp_i i, event_type type) {
+struct icon_event *ro_gui_wimp_event_get_event(wimp_w w, wimp_i i,
+		event_type type)
+{
 	struct event_window *window;
 	struct icon_event *event;
 
@@ -1234,14 +1276,16 @@ struct icon_event *ro_gui_wimp_event_get_event(wimp_w w, wimp_i i, event_type ty
 /**
  * Handle menus being closed
  */
-void ro_gui_wimp_event_menus_closed(void) {
+void ro_gui_wimp_event_menus_closed(void)
+{
   	ro_gui_wimp_event_register_submenu(0);
 }
 
 /**
  * Register a submenu as being opened
  */
-void ro_gui_wimp_event_register_submenu(wimp_w w) {
+void ro_gui_wimp_event_register_submenu(wimp_w w)
+{
 	if (ro_gui_wimp_event_submenu)
 		ro_gui_wimp_event_close_window(ro_gui_wimp_event_submenu);
 	ro_gui_wimp_event_submenu = w;
