@@ -27,6 +27,7 @@
 
 #include <assert.h>
 #include <ctype.h>
+#include <inttypes.h>
 #include <math.h>
 #include <stdio.h>
 #include <stdbool.h>
@@ -35,7 +36,7 @@
 #include "oslib/colourtrans.h"
 #include "oslib/osbyte.h"
 #include "oslib/osfile.h"
-#include <oslib/osspriteop.h>
+#include "oslib/osspriteop.h"
 #include "oslib/wimp.h"
 #include "oslib/wimpspriteop.h"
 #include "utils/config.h"
@@ -44,7 +45,6 @@
 #include "css/css.h"
 #include "desktop/browser.h"
 #include "desktop/frames.h"
-#include "desktop/knockout.h"
 #include "desktop/plotters.h"
 #include "desktop/textinput.h"
 #include "desktop/tree.h"
@@ -59,12 +59,14 @@
 #include "riscos/gui/status_bar.h"
 #include "riscos/menus.h"
 #include "riscos/options.h"
+#include "riscos/oslib_pre7.h"
 #include "riscos/save.h"
 #include "riscos/theme.h"
 #include "riscos/thumbnail.h"
 #include "riscos/url_complete.h"
 #include "riscos/wimp.h"
 #include "riscos/wimp_event.h"
+#include "riscos/wimputils.h"
 #include "utils/log.h"
 #include "utils/talloc.h"
 #include "utils/url.h"
@@ -76,12 +78,20 @@
 #define wimp_KEY_END wimp_KEY_COPY
 #endif
 
+#ifndef wimp_WINDOW_GIVE_SHADED_ICON_INFO
+	/* RISC OS 5+. Requires OSLib trunk. */
+#define wimp_WINDOW_GIVE_SHADED_ICON_INFO ((wimp_extra_window_flags) 0x10u)
+#endif
+
 #define SCROLL_VISIBLE_PADDING 32
 
 /** Remembers which iconised sprite numbers are in use */
 static bool iconise_used[64];
 static int iconise_next = 0;
 
+/** Whether a pressed mouse button has become a drag */
+static bool mouse_drag;
+
 /** List of all browser windows. */
 static struct gui_window *window_list = 0;
 /** GUI window which is being redrawn. Valid only during redraw. */
@@ -97,7 +107,7 @@ static float scale_snap_to[] = {0.10, 0.125, 0.25, 0.333, 0.5, 0.75,
 /** An entry in ro_gui_pointer_table. */
 struct ro_gui_pointer_entry {
 	bool wimp_area;  /** The pointer is in the Wimp's sprite area. */
-	char sprite_name[12];
+	char sprite_name[16];
 	int xactive;
 	int yactive;
 };
@@ -138,8 +148,8 @@ static void ro_gui_window_launch_url(struct gui_window *g, const char *url);
 static void ro_gui_window_clone_options(struct browser_window *new_bw,
 		struct browser_window *old_bw);
 static browser_mouse_state ro_gui_mouse_drag_state(wimp_mouse_state buttons);
-static bool ro_gui_window_import_text(struct gui_window *g, const char *filename,
-		bool toolbar);
+static bool ro_gui_window_import_text(struct gui_window *g,
+		const char *filename, bool toolbar);
 
 struct update_box {
 	int x0;
@@ -165,7 +175,7 @@ struct update_box *pending_updates;
  */
 
 struct gui_window *gui_create_browser_window(struct browser_window *bw,
-		struct browser_window *clone)
+		struct browser_window *clone, bool new_tab)
 {
 	int screen_width, screen_height, win_width, win_height, scroll_width;
 	static int window_count = 2;
@@ -271,7 +281,8 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	window.scroll_outer = wimp_COLOUR_DARK_GREY;
 	window.scroll_inner = wimp_COLOUR_MID_LIGHT_GREY;
 	window.highlight_bg = wimp_COLOUR_CREAM;
-	window.extra_flags = 0;
+	window.extra_flags = wimp_WINDOW_USE_EXTENDED_SCROLL_REQUEST |
+			wimp_WINDOW_GIVE_SHADED_ICON_INFO;
 	window.extent.x0 = 0;
 	window.extent.y0 = -(window.visible.y1 - window.visible.y0);
 	window.extent.x1 = window.visible.x1 - window.visible.x0;
@@ -384,14 +395,15 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	state.next = wimp_TOP;
 	if (bw->parent) {
 		top = browser_window_owner(bw);
-		error = xwimp_open_window_nested((wimp_open *)&state, top->window->window,
+		error = xwimp_open_window_nested(PTR_WIMP_OPEN(&state), 
+				top->window->window,
 				wimp_CHILD_LINKS_PARENT_WORK_AREA
 						<< wimp_CHILD_XORIGIN_SHIFT |
 				wimp_CHILD_LINKS_PARENT_WORK_AREA
 						<< wimp_CHILD_YORIGIN_SHIFT);
 	}
 
-	ro_gui_window_open((wimp_open*)&state);
+	ro_gui_window_open(PTR_WIMP_OPEN(&state));
 
 	/* Claim the caret for top-level windows */
 	if (bw->browser_window_type == BROWSER_WINDOW_NORMAL) {
@@ -672,7 +684,7 @@ void gui_window_set_scroll(struct gui_window *g, int sx, int sy)
 	state.yscroll = -sy * 2 * g->bw->scale;
 	if (g->toolbar)
 		state.yscroll += ro_gui_theme_toolbar_full_height(g->toolbar);
-	ro_gui_window_open((wimp_open *)&state);
+	ro_gui_window_open(PTR_WIMP_OPEN(&state));
 }
 
 
@@ -756,7 +768,7 @@ void gui_window_scroll_visible(struct gui_window *g, int x0, int y0, int x1, int
 
 	state.xscroll = cx0;
 	state.yscroll = -cy0 + toolbar_height;
-	ro_gui_window_open((wimp_open *)&state);
+	ro_gui_window_open(PTR_WIMP_OPEN(&state));
 }
 
 
@@ -850,7 +862,7 @@ void gui_window_position_frame(struct gui_window *g, int x0, int y0, int x1, int
 	state.visible.x1 = px0 + x1;
 	state.visible.y1 = py1 - y0 * 2 * scale;
 	g->update_extent = true;
-	ro_gui_window_open((wimp_open *)&state);
+	ro_gui_window_open(PTR_WIMP_OPEN(&state));
 }
 
 
@@ -910,7 +922,7 @@ void gui_window_update_extent(struct gui_window *g)
 	flags = state.flags & (wimp_WINDOW_HSCROLL | wimp_WINDOW_VSCROLL);
 	update = g->bw->reformat_pending;
 	g->update_extent = true;
-	ro_gui_window_open((wimp_open *)&state);
+	ro_gui_window_open(PTR_WIMP_OPEN(&state));
 
 	state.w = g->window;
 	error = xwimp_get_window_state(&state);
@@ -1013,13 +1025,13 @@ void gui_window_set_url(struct gui_window *g, const char *url)
 {
 	wimp_caret caret;
 	os_error *error;
-	char *toolbar_url;
+	const char *toolbar_url;
 
 	if (!g->toolbar)
 		return;
 
 	ro_gui_set_icon_string(g->toolbar->toolbar_handle,
-			ICON_TOOLBAR_URL, url);
+			ICON_TOOLBAR_URL, url, true);
 	ro_gui_force_redraw_icon(g->toolbar->toolbar_handle,
 			ICON_TOOLBAR_FAVICON);
 
@@ -1079,7 +1091,7 @@ void gui_window_stop_throbber(struct gui_window *g)
 	if (g->toolbar) {
 		strcpy(throb_buf, "throbber0");
 		ro_gui_set_icon_string(g->toolbar->toolbar_handle,
-				ICON_TOOLBAR_THROBBER, throb_buf);
+				ICON_TOOLBAR_THROBBER, throb_buf, true);
 		if ((g->toolbar->descriptor) && (g->toolbar->descriptor->throbber_redraw))
 			ro_gui_force_redraw_icon(g->toolbar->toolbar_handle,
 					ICON_TOOLBAR_THROBBER);
@@ -1368,7 +1380,7 @@ void gui_window_save_as_link(struct gui_window *g, struct content *c)
 {
 	if (!c)
 		return;
-	ro_gui_save_prepare(GUI_SAVE_LINK_URL, c);
+	ro_gui_save_prepare(GUI_SAVE_LINK_URL, NULL, NULL, c->url, c->title);
 	ro_gui_dialog_open_persistent(g->window, dialog_saveas, true);
 }
 
@@ -1404,11 +1416,9 @@ void ro_gui_window_redraw_all(void)
 void ro_gui_window_redraw(wimp_draw *redraw)
 {
 	osbool more;
-	bool knockout = true;
 	struct gui_window *g = (struct gui_window *)ro_gui_wimp_event_get_user_data(redraw->w);
 	float scale = g->bw->scale;
 	struct content *c = g->bw->current_content;
-	int clip_x0, clip_y0, clip_x1, clip_y1, clear_x1, clear_y1;
 	os_error *error;
 
 	/*	Handle no content quickly
@@ -1418,16 +1428,18 @@ void ro_gui_window_redraw(wimp_draw *redraw)
 		return;
 	}
 
+	/* We can't render locked content as it is being in the process of
+	   being transformed.  We won't update anything (i.e. leaving
+	   window area as is) instead of showing random data in case of
+	   buffered redraw.  */
+	if (c->locked)
+		return;
+
 	plot = ro_plotters;
 	ro_plot_set_scale(scale);
 	ro_gui_current_redraw_gui = g;
 	current_redraw_browser = g->bw;
 
-	/* rendering textplain has no advantages using knockout rendering other
-	 * than to slow things down. */
-	if (c->type == CONTENT_TEXTPLAIN)
-		knockout = false;
-
 	/* HTML rendering handles scale itself */
 	if (c->type == CONTENT_HTML)
 		scale = 1;
@@ -1440,34 +1452,45 @@ void ro_gui_window_redraw(wimp_draw *redraw)
 		return;
 	}
 	while (more) {
+		int clip_x0, clip_y0, clip_x1, clip_y1;
+
+		/* OS's redraw request coordinates are in screen coordinates,
+		 * with an origin at the bottom left of the screen.
+		 * Find the coordinate of the top left of the document in terms
+		 * of OS screen coordinates.
+		 * NOTE: OS units are 2 per px. */
 		ro_plot_origin_x = redraw->box.x0 - redraw->xscroll;
 		ro_plot_origin_y = redraw->box.y1 - redraw->yscroll;
-		clip_x0 = (redraw->clip.x0 - ro_plot_origin_x) / 2;
-		clip_y0 = (ro_plot_origin_y - redraw->clip.y1) / 2;
-		clip_x1 = (redraw->clip.x1 - ro_plot_origin_x) / 2;
-		clip_y1 = (ro_plot_origin_y - redraw->clip.y0) / 2;
-		clear_x1 = redraw->clip.x1 - ro_plot_origin_x;
-		clear_y1 = redraw->clip.y0 - ro_plot_origin_y;
+
+		/* Convert OS redraw rectangle request coordinates into NetSurf
+		 * coordinates. NetSurf coordinates have origin at top left of
+		 * document and units are in px. */
+		clip_x0 = (redraw->clip.x0 - ro_plot_origin_x) / 2; /* left   */
+		clip_y0 = (ro_plot_origin_y - redraw->clip.y1) / 2; /* top    */
+		clip_x1 = (redraw->clip.x1 - ro_plot_origin_x) / 2; /* right  */
+		clip_y1 = (ro_plot_origin_y - redraw->clip.y0) / 2; /* bottom */
 
 		if (ro_gui_current_redraw_gui->option.buffer_everything)
 			ro_gui_buffer_open(redraw);
 
-		if (knockout) {
-			knockout_plot_start(&plot);
-			plot.clip(clip_x0, clip_y0, clip_x1, clip_y1);
-			plot.clg(0x00ffffff);
-		}
+		/* Set up NetSurf's plotters with current clip rectangle */
+		plot.clip(clip_x0, clip_y0, clip_x1, clip_y1);
 
+		if (c->type != CONTENT_HTML)
+			plot.clg(0x00ffffff);
+
+		/* Redraw the clip rectangle area of the content */
 		content_redraw(c, 0, 0,
 				c->width * scale, c->height * scale,
 				clip_x0, clip_y0, clip_x1, clip_y1,
 				g->bw->scale,
 				0xFFFFFF);
-		if (knockout)
-			knockout_plot_end();
+
 		if (ro_gui_current_redraw_gui->option.buffer_everything)
 			ro_gui_buffer_close();
 
+		/* Check to see if there are more rectangles to draw and
+		 * get next one */
 		error = xwimp_get_rectangle(redraw, &more);
 		/* RISC OS 3.7 returns an error here if enough buffer was
 		   claimed to cause a new dynamic area to be created. It
@@ -1571,7 +1594,7 @@ void ro_gui_window_update_boxes(void) {
 			if (data->redraw.full_redraw) {
 				if (clear_background) {
 					error = xcolourtrans_set_gcol(os_COLOUR_WHITE,
-							colourtrans_SET_BG,
+							colourtrans_SET_BG_GCOL,
 							os_ACTION_OVERWRITE, 0, 0);
 					if (error) {
 						LOG(("xcolourtrans_set_gcol: 0x%x: %s",
@@ -1881,7 +1904,7 @@ void ro_gui_window_open(wimp_open *open)
 	}
 
 	/* first resize stops any flickering by making the URL window on top */
-	ro_gui_url_complete_resize(g, (wimp_open *)&state);
+	ro_gui_url_complete_resize(g, PTR_WIMP_OPEN(&state));
 
 	error = xwimp_open_window_nested_with_flags(&state, parent, linkage);
 	if (error) {
@@ -1917,6 +1940,7 @@ void ro_gui_window_close(wimp_w w) {
 	char *temp_name, *r;
 	char *filename;
 	struct content *content = NULL;
+	bool destroy;
 
 	error = xwimp_get_pointer_info(&pointer);
 	if (error) {
@@ -1928,6 +1952,7 @@ void ro_gui_window_close(wimp_w w) {
 	if (g->bw)
 		content = g->bw->current_content;
 	if (pointer.buttons & wimp_CLICK_ADJUST) {
+		destroy = !ro_gui_shift_pressed();
 		filename = (content && content->url) ? url_to_path(content->url) : NULL;
 		if (filename) {
 			temp_name = malloc(strlen(filename) + 32);
@@ -1939,7 +1964,7 @@ void ro_gui_window_close(wimp_w w) {
 						*r = '\0';
 						break;
 					}
-					*r--;
+					r--;
 				}
 				error = xos_cli(temp_name);
 				if (error) {
@@ -1953,11 +1978,14 @@ void ro_gui_window_close(wimp_w w) {
 			free(filename);
 		} else {
 			/* this is pointless if we are about to close the window */
-			if (ro_gui_shift_pressed())
+			if (!destroy)
 				ro_gui_menu_handle_action(w, BROWSER_NAVIGATE_UP, true);
 		}
 	}
-	if (!ro_gui_shift_pressed())
+	else
+		destroy = true;
+
+	if (destroy)
 		browser_window_destroy(g->bw);
 }
 
@@ -2010,7 +2038,7 @@ void ro_gui_throb(void)
 			top_g->throbber = 1;
 		sprintf(throb_buf, "throbber%i", top_g->throbber);
 		ro_gui_set_icon_string(top_g->toolbar->toolbar_handle,
-				ICON_TOOLBAR_THROBBER, throb_buf);
+				ICON_TOOLBAR_THROBBER, throb_buf, true);
 		if (top_g->toolbar->descriptor->throbber_redraw)
 			ro_gui_force_redraw_icon(top_g->toolbar->toolbar_handle,
 					ICON_TOOLBAR_THROBBER);
@@ -2110,7 +2138,7 @@ bool ro_gui_toolbar_click(wimp_pointer *pointer)
 		case ICON_TOOLBAR_BACK:
 			if (pointer->buttons == wimp_CLICK_ADJUST) {
 				new_bw = browser_window_create(NULL,
-						g->bw, NULL, false);
+						g->bw, NULL, false, false);
 				ro_gui_menu_handle_action(new_bw->window->window,
 						BROWSER_NAVIGATE_BACK, true);
 			} else {
@@ -2122,7 +2150,7 @@ bool ro_gui_toolbar_click(wimp_pointer *pointer)
 		case ICON_TOOLBAR_FORWARD:
 			if (pointer->buttons == wimp_CLICK_ADJUST) {
 				new_bw = browser_window_create(NULL,
-						g->bw, NULL, false);
+						g->bw, NULL, false, false);
 				ro_gui_menu_handle_action(new_bw->window->window,
 						BROWSER_NAVIGATE_FORWARD, true);
 			} else {
@@ -2157,12 +2185,10 @@ bool ro_gui_toolbar_click(wimp_pointer *pointer)
 			ro_gui_menu_handle_action(g->window,
 					BROWSER_NAVIGATE_HOME, true);
 			break;
-#ifdef WITH_SEARCH
 		case ICON_TOOLBAR_SEARCH:
 			ro_gui_menu_handle_action(g->window,
 					BROWSER_FIND_TEXT, true);
 			break;
-#endif
 		case ICON_TOOLBAR_SCALE:
 			ro_gui_menu_handle_action(g->window,
 					BROWSER_SCALE_VIEW, true);
@@ -2193,7 +2219,8 @@ bool ro_gui_toolbar_click(wimp_pointer *pointer)
 			if (pointer->buttons == wimp_CLICK_ADJUST) {
 				if (g->bw && g->bw->current_content) {
 					new_bw = browser_window_create(NULL,
-							g->bw, NULL, false);
+							g->bw, NULL, false,
+							false);
 					/* do it without loading the content into the new window */
 					ro_gui_window_navigate_up(new_bw->window,
 							g->bw->current_content->url);
@@ -2206,6 +2233,7 @@ bool ro_gui_toolbar_click(wimp_pointer *pointer)
 		case ICON_TOOLBAR_URL:
 			if (pointer->buttons & (wimp_DRAG_SELECT | wimp_DRAG_ADJUST)) {
 				if (g->bw->current_content) {
+					struct content *c = g->bw->current_content;
 					gui_save_type save_type;
 
 					if (ro_gui_shift_pressed())
@@ -2213,7 +2241,7 @@ bool ro_gui_toolbar_click(wimp_pointer *pointer)
 					else
 						save_type = GUI_SAVE_LINK_TEXT;
 
-					gui_drag_save_object(save_type, g->bw->current_content, g);
+					ro_gui_drag_save_link(save_type, c->url, c->title, g);
 				}
 			}
 			else
@@ -2274,11 +2302,11 @@ bool ro_gui_window_keypress(wimp_key *key)
 	struct content *content;
 	wimp_window_state state;
 	int y;
-	char *toolbar_url;
+	const char *toolbar_url;
 	os_error *error;
 	wimp_pointer pointer;
 	float scale;
-	wchar_t c = (wchar_t)key->c;
+	uint32_t c = (uint32_t) key->c;
 
 	/* Find gui window */
 	if ((g = ro_gui_window_lookup(key->w)) != NULL) {
@@ -2357,7 +2385,7 @@ bool ro_gui_window_keypress(wimp_key *key)
 
 		/* Reset c to incoming character / key code
 		 * as we may have corrupted it above */
-		c = (wchar_t)key->c;
+		c = (uint32_t) key->c;
 	}
 
 	switch (c) {
@@ -2374,7 +2402,7 @@ bool ro_gui_window_keypress(wimp_key *key)
 				return false;
 			ro_gui_url_complete_close(NULL, 0);
 			ro_gui_set_icon_string(g->toolbar->toolbar_handle,
-					ICON_TOOLBAR_URL, "www.");
+					ICON_TOOLBAR_URL, "www.", true);
 			xwimp_set_caret_position(g->toolbar->toolbar_handle,
 					ICON_TOOLBAR_URL, 0, 0, -1, 4);
 			ro_gui_url_complete_start(g);
@@ -2404,12 +2432,10 @@ bool ro_gui_window_keypress(wimp_key *key)
 			return ro_gui_menu_handle_action(g->window,
 					BROWSER_EXPORT_DRAW, false);
 
-#ifdef WITH_SEARCH
 		case 6:			/* Ctrl + F */
 		case IS_WIMP_KEY + wimp_KEY_F4:	/* Search */
 			return ro_gui_menu_handle_action(g->window,
 					BROWSER_FIND_TEXT, false);
-#endif
 
 		case IS_WIMP_KEY + wimp_KEY_F5:	/* Reload */
 			return ro_gui_menu_handle_action(g->window,
@@ -2439,16 +2465,7 @@ bool ro_gui_window_keypress(wimp_key *key)
 
 		case IS_WIMP_KEY + wimp_KEY_F9:
 			/* Dump content for debugging. */
-			switch (content->type) {
-				case CONTENT_HTML:
-					box_dump(content->data.html.layout, 0);
-					break;
-				case CONTENT_CSS:
-					css_dump_stylesheet(content->data.css.css);
-					break;
-				default:
-					break;
-			}
+			ro_gui_dump_content(content);
 			return true;
 
 		case IS_WIMP_KEY + wimp_KEY_CONTROL + wimp_KEY_F9:
@@ -2533,11 +2550,9 @@ bool ro_gui_window_keypress(wimp_key *key)
 			}
 			return true;
 
-#ifdef WITH_PRINT
 		case IS_WIMP_KEY + wimp_KEY_PRINT:
 			return ro_gui_menu_handle_action(g->window,
 					BROWSER_PRINT, false);
-#endif
 
 		case IS_WIMP_KEY | wimp_KEY_LEFT:
 		case IS_WIMP_KEY | wimp_KEY_RIGHT:
@@ -2610,7 +2625,7 @@ bool ro_gui_window_keypress(wimp_key *key)
 			break;
 	}
 
-	error = xwimp_open_window((wimp_open *) &state);
+	error = xwimp_open_window(PTR_WIMP_OPEN(&state));
 	if (error) {
 		LOG(("xwimp_open_window: 0x%x: %s",
 				error->errnum, error->errmess));
@@ -2625,49 +2640,78 @@ bool ro_gui_window_keypress(wimp_key *key)
  */
 void ro_gui_scroll_request(wimp_scroll *scroll)
 {
-	int x, y;
 	struct gui_window *g = ro_gui_window_lookup(scroll->w);
 
-	x = scroll->visible.x1 - scroll->visible.x0 - 32;
-	y = scroll->visible.y1 - scroll->visible.y0 - 32;
-	if (g && g->toolbar)
-		y -= ro_gui_theme_toolbar_full_height(g->toolbar);
+	if (g && g->bw->current_content && ro_gui_shift_pressed()) {
+		/* extended scroll request with shift held down; change zoom */
+		float scale, inc;
 
-	switch (scroll->xmin) {
-		case wimp_SCROLL_PAGE_LEFT:
-			scroll->xscroll -= x;
-			break;
-		case wimp_SCROLL_COLUMN_LEFT:
-			scroll->xscroll -= 32;
-			break;
-		case wimp_SCROLL_COLUMN_RIGHT:
-			scroll->xscroll += 32;
-			break;
-		case wimp_SCROLL_PAGE_RIGHT:
-			scroll->xscroll += x;
-			break;
-		default:
-			break;
+		if (scroll->ymin & 3)
+			inc = 0.02;  /* RO5 sends the msg 5 times; don't ask my why */
+		else
+			inc = (1 << (ABS(scroll->ymin)>>2)) / 20.0F;
+
+		if (scroll->ymin > 0) {
+			scale = g->bw->scale + inc;
+			if (scale > scale_snap_to[SCALE_SNAP_TO_SIZE - 1])
+				scale = scale_snap_to[SCALE_SNAP_TO_SIZE - 1];
+		} else {
+			scale = g->bw->scale - inc;
+			if (scale < scale_snap_to[0])
+				scale = scale_snap_to[0];
+		}
+		if (g->bw->scale != scale)
+			browser_window_set_scale(g->bw, scale, true);
+	} else {
+		int x = scroll->visible.x1 - scroll->visible.x0 - 32;
+		int y = scroll->visible.y1 - scroll->visible.y0 - 32;
+		os_error *error;
+
+		if (g && g->toolbar)
+			y -= ro_gui_theme_toolbar_full_height(g->toolbar);
+
+		switch (scroll->xmin) {
+			case wimp_SCROLL_PAGE_LEFT:
+				scroll->xscroll -= x;
+				break;
+			case wimp_SCROLL_COLUMN_LEFT:
+				scroll->xscroll -= 32;
+				break;
+			case wimp_SCROLL_COLUMN_RIGHT:
+				scroll->xscroll += 32;
+				break;
+			case wimp_SCROLL_PAGE_RIGHT:
+				scroll->xscroll += x;
+				break;
+			default:
+				scroll->xscroll += (x * (scroll->xmin>>2)) >> 2;
+				break;
+		}
+
+		switch (scroll->ymin) {
+			case wimp_SCROLL_PAGE_UP:
+				scroll->yscroll += y;
+				break;
+			case wimp_SCROLL_LINE_UP:
+				scroll->yscroll += 32;
+				break;
+			case wimp_SCROLL_LINE_DOWN:
+				scroll->yscroll -= 32;
+				break;
+			case wimp_SCROLL_PAGE_DOWN:
+				scroll->yscroll -= y;
+				break;
+			default:
+				scroll->yscroll += (y * (scroll->ymin>>2)) >> 2;
+				break;
+		}
+
+		error = xwimp_open_window((wimp_open *) scroll);
+		if (error) {
+			LOG(("xwimp_open_window: 0x%x: %s",
+					error->errnum, error->errmess));
+		}
 	}
-
-	switch (scroll->ymin) {
-		case wimp_SCROLL_PAGE_UP:
-			scroll->yscroll += y;
-			break;
-		case wimp_SCROLL_LINE_UP:
-			scroll->yscroll += 32;
-			break;
-		case wimp_SCROLL_LINE_DOWN:
-			scroll->yscroll -= 32;
-			break;
-		case wimp_SCROLL_PAGE_DOWN:
-			scroll->yscroll -= y;
-			break;
-		default:
-			break;
-	}
-
-	wimp_open_window((wimp_open *) scroll);
 }
 
 
@@ -2745,12 +2789,12 @@ bool ro_gui_window_to_screen_pos(struct gui_window *g, int x, int y, os_coord *p
 
 bool ro_gui_window_dataload(struct gui_window *g, wimp_message *message)
 {
-	int box_x = 0, box_y = 0;
 	struct box *box;
 	struct box *file_box = 0;
 	struct box *text_box = 0;
 	struct browser_window *bw = g->bw;
 	struct content *content;
+	int box_x, box_y;
 	os_error *error;
 	os_coord pos;
 
@@ -2768,6 +2812,11 @@ bool ro_gui_window_dataload(struct gui_window *g, wimp_message *message)
 
 	content = bw->current_content;
 	box = content->data.html.layout;
+
+	/* Consider the margins of the html page now */
+	box_x = box->margin[LEFT];
+	box_y = box->margin[TOP];
+
 	while ((box = box_at_point(box, pos.x, pos.y, &box_x, &box_y, &content))) {
 		if (box->style &&
 				box->style->visibility == CSS_VISIBILITY_HIDDEN)
@@ -2822,7 +2871,7 @@ bool ro_gui_window_dataload(struct gui_window *g, wimp_message *message)
 
 		const char *filename = message->data.data_xfer.file_name;
 
-		browser_window_mouse_click(g->bw, BROWSER_MOUSE_CLICK_1, pos.x, pos.y);
+		browser_window_mouse_click(g->bw, BROWSER_MOUSE_PRESS_1, pos.x, pos.y);
 
 		if (!ro_gui_window_import_text(g, filename, false))
 			return true;  /* it was for us, it just didn't work! */
@@ -3001,11 +3050,19 @@ browser_mouse_state ro_gui_mouse_click_state(wimp_mouse_state buttons)
 {
 	browser_mouse_state state = 0;
 
-	if (buttons & (wimp_CLICK_SELECT)) state |= BROWSER_MOUSE_CLICK_1;
-	if (buttons & (wimp_CLICK_ADJUST)) state |= BROWSER_MOUSE_CLICK_2;
+	if (buttons & (wimp_CLICK_SELECT))
+		state |= BROWSER_MOUSE_PRESS_1 | BROWSER_MOUSE_CLICK_1;
+	if (buttons & (wimp_CLICK_ADJUST))
+		state |= BROWSER_MOUSE_PRESS_2 | BROWSER_MOUSE_CLICK_2;
 
-	if (buttons & (wimp_DRAG_SELECT)) state |= BROWSER_MOUSE_DRAG_1;
-	if (buttons & (wimp_DRAG_ADJUST)) state |= BROWSER_MOUSE_DRAG_2;
+	if (buttons & (wimp_DRAG_SELECT)) {
+		state |= BROWSER_MOUSE_DRAG_1;
+		mouse_drag = true;
+	}
+	if (buttons & (wimp_DRAG_ADJUST)) {
+		state |= BROWSER_MOUSE_DRAG_2;
+		mouse_drag = true;
+	}
 
 	if (ro_gui_shift_pressed()) state |= BROWSER_MOUSE_MOD_1;
 	if (ro_gui_ctrl_pressed())  state |= BROWSER_MOUSE_MOD_2;
@@ -3023,9 +3080,14 @@ browser_mouse_state ro_gui_mouse_drag_state(wimp_mouse_state buttons)
 {
 	browser_mouse_state state = 0;
 
+
 	if (buttons & (wimp_CLICK_SELECT)) state |= BROWSER_MOUSE_HOLDING_1;
 	if (buttons & (wimp_CLICK_ADJUST)) state |= BROWSER_MOUSE_HOLDING_2;
 
+	if (!(buttons & (wimp_CLICK_SELECT) || buttons & (wimp_CLICK_ADJUST)))
+		mouse_drag = false;
+	if (mouse_drag) state |= BROWSER_MOUSE_DRAG_ON;
+
 	if (ro_gui_shift_pressed()) state |= BROWSER_MOUSE_MOD_1;
 	if (ro_gui_ctrl_pressed())  state |= BROWSER_MOUSE_MOD_2;
 
@@ -3057,6 +3119,18 @@ bool ro_gui_ctrl_pressed(void)
 }
 
 
+/**
+ * Returns true iff one or more Alt keys is held down
+ */
+
+bool ro_gui_alt_pressed(void)
+{
+	int alt = 0;
+	xosbyte1(osbyte_SCAN_KEYBOARD, 2 ^ 0x80, 0, &alt);
+	return (alt == 0xff);
+}
+
+
 /**
  * Completes scrolling of a browser window
  *
@@ -3336,8 +3410,8 @@ bool ro_gui_window_navigate_up(struct gui_window *g, const char *url) {
 
 	res = url_parent(url, &parent);
 	if (res == URL_FUNC_OK) {
-		res = url_compare(url, parent, &compare);
-		if (!compare && (res == URL_FUNC_OK))
+		res = url_compare(url, parent, false, &compare);
+		if ((res == URL_FUNC_OK) && !compare)
 			browser_window_go(g->bw, parent, 0, true);
 		free(parent);
 	}
