@@ -22,14 +22,17 @@
  * URL parsing and joining (implementation).
  */
 
+#define _GNU_SOURCE /* for strdup() */
+#include <sys/types.h>
 #include <assert.h>
 #include <ctype.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #include <strings.h>
-#include <sys/types.h>
 #include <regex.h>
+#include <unistd.h>
+#include "curl/curl.h"
 #include "utils/log.h"
 #include "utils/url.h"
 #include "utils/utils.h"
@@ -106,7 +109,7 @@ bool url_host_is_ip_address(const char *host) {
                 } else {
                 	n = true;
                 }
-                *host++;
+                host++;
         } while (1);
 }
 
@@ -114,7 +117,9 @@ bool url_host_is_ip_address(const char *host) {
  * Normalize a URL.
  *
  * \param  url	   an absolute URL
- * \param  result  pointer to pointer to buffer to hold cleaned up url
+ * \param  result  pointer to pointer to buffer to hold cleaned up url. Caller
+ *                 gets ownership of pointer to buffer value. On failure the
+ *                 pointer to buffer value will be NULL.
  * \return  URL_FUNC_OK on success
  *
  * If there is no scheme, http:// is added. The scheme and host are
@@ -126,41 +131,73 @@ url_func_result url_normalize(const char *url, char **result)
 {
 	char c;
 	int m;
-	int i;
+	size_t i;
 	size_t len;
+	size_t bufsize;
+	char* norm;
 	bool http = false;
 	regmatch_t match[10];
 
 	*result = NULL;
 
-	if ((m = regexec(&url_re, url, 10, match, 0))) {
-		LOG(("url '%s' failed to match regex", url));
-		return URL_FUNC_FAILED;
+	/* skip past any leading whitespace (likely if URL was copy-pasted) */
+	while (isspace(*url))
+		url++;
+
+	/* allocate sufficiently large buffer for new URL */
+	len = strlen(url);
+	/* "+ 1" for the terminating NUL character.  */
+	bufsize = len + 1 + SLEN("http://") + SLEN("/");
+	/* work out how much extra to leave for internal whitespace */
+	for(i = 0; i < len; i++) {
+		if(isspace(url[i])) bufsize += 2; /* ' ' -> '%20' */
+	}
+	if ((norm = malloc(bufsize)) == NULL) {
+		LOG(("malloc failed"));
+		return URL_FUNC_NOMEM;
+	}
+	*result = norm;
+	strcpy(norm, url);
+
+	/* truncate trailing whitespace (significant should be uriencoded) */
+	for (i = len - 1; (i > 0) && isspace(norm[i]); i--) {
+		norm[i] = '\0';
+		len--;
 	}
 
-	len = strlen(url);
+	/* encode any remaining (internal) whitespace */
+	for (i = 0; i < len; i++) {
+		if(isspace(norm[i])) {
+			char space = norm[i];
+			memmove(norm + i + 2, norm + i, 1 + len - i);
+			len += 2;
+			norm[  i] = '%';
+			norm[++i] = digit2lowcase_hex(space >> 4);
+			norm[++i] = digit2lowcase_hex(space & 0xf);
+		}
+	}
+
+	/* finally verify that it's actually an URL we're working on
+	 * (RFC regex too fussy to tolerate above WSP problems) */
+	if ((m = regexec(&url_re, norm, 10, match, 0))) {
+		LOG(("url '%s' failed to match regex", url));
+		free(norm);
+		*result = NULL;
+		return URL_FUNC_FAILED;
+	}
 
 	if (match[URL_RE_SCHEME].rm_so == -1) {
 		/* scheme missing: add http:// and reparse */
 /*		LOG(("scheme missing: using http"));*/
-		if ((*result = malloc(len + 13)) == NULL) {
-			LOG(("malloc failed"));
-			return URL_FUNC_NOMEM;
-		}
-		strcpy(*result, "http://");
-		strcpy(*result + sizeof("http://")-1, url);
-		if ((m = regexec(&url_re, *result, 10, match, 0))) {
-			LOG(("url '%s' failed to match regex", (*result)));
-			free(*result);
+		memmove(norm + SLEN("http://"), norm, len + 1);
+		memcpy(norm, "http://", SLEN("http://")); /* do NOT copy NUL */
+		len += SLEN("http://");
+		if ((m = regexec(&url_re, norm, 10, match, 0))) {
+			LOG(("url '%s' failed to match regex", norm));
+			free(norm);
+			*result = NULL;
 			return URL_FUNC_FAILED;
 		}
-		len += sizeof("http://")-1;
-	} else {
-		if ((*result = malloc(len + 6)) == NULL) {
-			LOG(("malloc failed"));
-			return URL_FUNC_NOMEM;
-		}
-		strcpy(*result, url);
 	}
 
 	/*for (unsigned int i = 0; i != 10; i++) {
@@ -175,69 +212,70 @@ url_func_result url_normalize(const char *url, char **result)
 	/* make scheme lower-case */
 	if (match[URL_RE_SCHEME].rm_so != -1) {
 		for (i = match[URL_RE_SCHEME].rm_so;
-				i != match[URL_RE_SCHEME].rm_eo; i++)
-			(*result)[i] = tolower((*result)[i]);
+				(regoff_t) i != match[URL_RE_SCHEME].rm_eo; i++)
+			norm[i] = tolower(norm[i]);
 		if (match[URL_RE_SCHEME].rm_eo == 4
-				&& (*result)[0] == 'h'
-				&& (*result)[1] == 't'
-				&& (*result)[2] == 't'
-				&& (*result)[3] == 'p')
+				&& norm[0] == 'h'
+				&& norm[1] == 't'
+				&& norm[2] == 't'
+				&& norm[3] == 'p')
 			http = true;
 	}
 
 	/* make empty path into "/" */
 	if (match[URL_RE_PATH].rm_so != -1 &&
 			match[URL_RE_PATH].rm_so == match[URL_RE_PATH].rm_eo) {
-		memmove((*result) + match[URL_RE_PATH].rm_so + 1,
-				(*result) + match[URL_RE_PATH].rm_so,
+		memmove(norm + match[URL_RE_PATH].rm_so + 1,
+				norm + match[URL_RE_PATH].rm_so,
 				len - match[URL_RE_PATH].rm_so + 1);
-		(*result)[match[URL_RE_PATH].rm_so] = '/';
+		norm[match[URL_RE_PATH].rm_so] = '/';
 		len++;
 	}
 
 	/* make host lower-case */
 	if (match[URL_RE_AUTHORITY].rm_so != -1) {
 		for (i = match[URL_RE_AUTHORITY].rm_so;
-				i != match[URL_RE_AUTHORITY].rm_eo; i++) {
-			if ((*result)[i] == ':') {
-				if (http && (*result)[i + 1] == '8' &&
-						(*result)[i + 2] == '0' &&
-						i + 3 ==
+				(regoff_t) i != match[URL_RE_AUTHORITY].rm_eo; 
+				i++) {
+			if (norm[i] == ':' && (i + 3) < len) {
+				if (http && norm[i + 1] == '8' &&
+						norm[i + 2] == '0' &&
+						(regoff_t) i + 3 ==
 						match[URL_RE_AUTHORITY].rm_eo) {
-					memmove((*result) + i,
-							(*result) + i + 3,
+					memmove(norm + i,
+							norm + i + 3,
 							len -
 							match[URL_RE_AUTHORITY].
 							rm_eo);
 					len -= 3;
-					(*result)[len] = '\0';
-				} else if (i + 1 == match[4].rm_eo) {
-					memmove((*result) + i,
-							(*result) + i + 1,
+					norm[len] = '\0';
+				} else if ((regoff_t) i + 1 == match[4].rm_eo) {
+					memmove(norm + i,
+							norm + i + 1,
 							len -
 							match[URL_RE_AUTHORITY].
 							rm_eo);
 					len--;
-					(*result)[len] = '\0';
+					norm[len] = '\0';
 				}
 				break;
 			}
-			(*result)[i] = tolower((*result)[i]);
+			norm[i] = tolower(norm[i]);
 		}
 	}
 
 	/* unescape non-"reserved" escaped characters */
-	for (i = 0; (unsigned)i != len; i++) {
-		if ((*result)[i] != '%')
+	for (i = 0; i + 2 < len; i++) {
+		if (norm[i] != '%')
 			continue;
-		c = tolower((*result)[i + 1]);
+		c = tolower(norm[i + 1]);
 		if ('0' <= c && c <= '9')
 			m = 16 * (c - '0');
 		else if ('a' <= c && c <= 'f')
 			m = 16 * (c - 'a' + 10);
 		else
 			continue;
-		c = tolower((*result)[i + 2]);
+		c = tolower(norm[i + 2]);
 		if ('0' <= c && c <= '9')
 			m += c - '0';
 		else if ('a' <= c && c <= 'f')
@@ -251,11 +289,12 @@ url_func_result url_normalize(const char *url, char **result)
 			continue;
 		}
 
-		(*result)[i] = m;
-		memmove((*result) + i + 1, (*result) + i + 3, len - i - 2);
+		norm[i] = m;
+		memmove(norm + i + 1, norm + i + 3, len - i - 2);
 		len -= 2;
 	}
 
+	/* norm and *result point to same memory, so just return ok */
 	return URL_FUNC_OK;
 }
 
@@ -273,8 +312,11 @@ url_func_result url_join(const char *rel, const char *base, char **result)
 {
 	url_func_result status = URL_FUNC_NOMEM;
 	struct url_components_internal base_components = {0,0,0,0,0,0};
+	struct url_components_internal *base_ptr = &base_components;
 	struct url_components_internal rel_components = {0,0,0,0,0,0};
+	struct url_components_internal *rel_ptr = &rel_components;
 	struct url_components_internal merged_components = {0,0,0,0,0,0};
+	struct url_components_internal *merged_ptr = &merged_components;
 	char *merge_path = NULL, *split_point;
 	char *input, *output, *start = NULL;
 	int len, buf_len;
@@ -286,8 +328,7 @@ url_func_result url_join(const char *rel, const char *base, char **result)
 
 
 	/* break down the relative URL (not cached, corruptable) */
-	status = url_get_components(rel,
-			(struct url_components *)&rel_components);
+	status = url_get_components(rel, (struct url_components *) rel_ptr);
 	if (status != URL_FUNC_OK) {
 		LOG(("relative url '%s' failed to get components", rel));
 		return URL_FUNC_FAILED;
@@ -299,11 +340,9 @@ url_func_result url_join(const char *rel, const char *base, char **result)
 		goto url_join_reform_url;
 
 	/* break down the base URL (possibly cached, not corruptable) */
-	status = url_get_components(base,
-			(struct url_components *)&base_components);
+	status = url_get_components(base, (struct url_components *) base_ptr);
 	if (status != URL_FUNC_OK) {
-		url_destroy_components(
-				(struct url_components *)&rel_components);
+		url_destroy_components((struct url_components *) rel_ptr);
 		LOG(("base url '%s' failed to get components", base));
 		return URL_FUNC_FAILED;
 	}
@@ -443,8 +482,7 @@ url_join_reform_url:
 	}
 
 	/* 5.3 */
-	*result = url_reform_components(
-			(struct url_components *)&merged_components);
+	*result = url_reform_components((struct url_components *) merged_ptr);
   	if (!(*result))
 		goto url_join_no_mem;
 
@@ -454,8 +492,8 @@ url_join_reform_url:
 url_join_no_mem:
 	free(start);
 	free(merge_path);
-	url_destroy_components((struct url_components *)&base_components);
-	url_destroy_components((struct url_components *)&rel_components);
+	url_destroy_components((struct url_components *) base_ptr);
+	url_destroy_components((struct url_components *) rel_ptr);
 	return status;
 }
 
@@ -668,7 +706,6 @@ url_func_result url_path(const char *url, char **result)
 {
 	url_func_result status;
 	struct url_components components;
-	int len;
 
 	assert(url);
 
@@ -677,21 +714,85 @@ url_func_result url_path(const char *url, char **result)
 		if (!components.path) {
 			status = URL_FUNC_FAILED;
 		} else {
-			len = strlen(components.path);
-			while (components.path[len - 1] != '/')
-				len--;
-			*result = malloc(len + 2);
+			*result = strdup(components.path);
 			if (!(*result))
 				status = URL_FUNC_NOMEM;
-			else
-				snprintf((*result), len + 1, "%s",
-						components.path);
 		}
 	}
 	url_destroy_components(&components);
 	return status;
 }
 
+/**
+ * Extract leafname from an URL
+ *
+ * \param url	  an absolute URL
+ * \param result  pointer to pointer to buffer to hold result
+ * \return URL_FUNC_OK on success
+ */
+
+url_func_result url_leafname(const char *url, char **result)
+{
+	url_func_result status;
+	struct url_components components;
+
+	assert(url);
+
+	status = url_get_components(url, &components);
+	if (status == URL_FUNC_OK) {
+		if (!components.path) {
+			status = URL_FUNC_FAILED;
+		} else {
+			char *slash = strrchr(components.path, '/');
+
+			assert (slash != NULL);
+
+			*result = strdup(slash + 1);
+			if (!(*result))
+				status = URL_FUNC_NOMEM;
+		}
+	}
+	url_destroy_components(&components);
+	return status;
+}
+
+/**
+ * Extract fragment from an URL
+ * This will unescape any %xx entities in the fragment
+ *
+ * \param url     an absolute URL
+ * \param result  pointer to pointer to buffer to hold result
+ * \return URL_FUNC_OK on success
+ */
+
+url_func_result url_fragment(const char *url, char **result)
+{
+	url_func_result status;
+	struct url_components components;
+
+	assert(url);
+
+	status = url_get_components(url, &components);
+	if (status == URL_FUNC_OK) {
+		if (!components.fragment) {
+			status = URL_FUNC_FAILED;
+		} else {
+			char *frag = curl_unescape(components.fragment,
+					strlen(components.fragment));
+			if (!frag) {
+				status = URL_FUNC_NOMEM;
+			} else {
+				*result = strdup(frag);
+				if (!(*result))
+					status = URL_FUNC_NOMEM;
+				curl_free(frag);
+			}
+		}
+	}
+
+	url_destroy_components(&components);
+	return status;
+}
 
 /**
  * Attempt to find a nice filename for a URL.
@@ -807,17 +908,19 @@ no_path:
 /**
  * Escape a string suitable for inclusion in an URL.
  *
- * \param  unescaped  the unescaped string
- * \param  sptoplus   true iff spaces should be converted to +
- * \param  result     pointer to pointer to buffer to hold escaped string
+ * \param  unescaped      the unescaped string
+ * \param  toskip         number of bytes to skip in unescaped string
+ * \param  sptoplus       true iff spaces should be converted to +
+ * \param  escexceptions  NULL or a string of characters excluded to be escaped
+ * \param  result         pointer to pointer to buffer to hold escaped string
  * \return  URL_FUNC_OK on success
  */
 
-url_func_result url_escape(const char *unescaped, bool sptoplus,
-		char **result)
+url_func_result url_escape(const char *unescaped, size_t toskip,
+		bool sptoplus, const char *escexceptions, char **result)
 {
-	int len;
-	char *escaped, *d;
+	size_t len;
+	char *escaped, *d, *tmpres;
 	const char *c;
 
 	if (!unescaped || !result)
@@ -826,21 +929,25 @@ url_func_result url_escape(const char *unescaped, bool sptoplus,
 	*result = NULL;
 
 	len = strlen(unescaped);
+	if (len < toskip)
+		return URL_FUNC_FAILED;
+	len -= toskip;
 
 	escaped = malloc(len * 3 + 1);
 	if (!escaped)
 		return URL_FUNC_NOMEM;
 
-	for (c = unescaped, d = escaped; *c; c++) {
+	for (c = unescaped + toskip, d = escaped; *c; c++) {
 		/* Check if we should escape this byte.
 		 * '~' is unreserved and should not be percent encoded, if
 		 * you believe the spec; however, leaving it unescaped
 		 * breaks a bunch of websites, so we escape it anyway. */
-		if (!isascii(*c) || strchr(":/?#[]@" /* gen-delims */
-					"!$&'()*+,;=" /* sub-delims */
-					"<>%\"{}|\\^`~", /* others */
-					*c) ||
-				*c <= 0x20 || *c == 0x7f) {
+		if (!isascii(*c)
+			|| (strchr(":/?#[]@" /* gen-delims */
+				  "!$&'()*+,;=" /* sub-delims */
+				  "<>%\"{}|\\^`~" /* others */,	*c)
+				&& (!escexceptions || !strchr(escexceptions, *c)))
+			|| *c <= 0x20 || *c == 0x7f) {
 			if (*c == 0x20 && sptoplus) {
 				*d++ = '+';
 			} else {
@@ -853,16 +960,17 @@ url_func_result url_escape(const char *unescaped, bool sptoplus,
 			*d++ = *c;
 		}
 	}
-
 	*d++ = '\0';
 
-	(*result) = malloc(d - escaped);
-	if (!(*result)) {
+	tmpres = malloc(d - escaped + toskip);
+	if (!tmpres) {
 		free(escaped);
 		return URL_FUNC_NOMEM;
 	}
 
-	memcpy((*result), escaped, d - escaped);
+	memcpy(tmpres, unescaped, toskip); 
+	memcpy(tmpres + toskip, escaped, d - escaped);
+	*result = tmpres;
 
 	free(escaped);
 
@@ -872,12 +980,14 @@ url_func_result url_escape(const char *unescaped, bool sptoplus,
 /**
  * Compare two absolute, normalized URLs
  *
- * \param url1 URL 1
- * \param url2 URL 2
- * \param result Pointer to location to store result (true if URLs match)
+ * \param url1     URL 1
+ * \param url2     URL 2
+ * \param nofrag   Ignore fragment part in comparison
+ * \param result   Pointer to location to store result (true if URLs match)
  * \return URL_FUNC_OK on success
  */
-url_func_result url_compare(const char *url1, const char *url2, bool *result)
+url_func_result url_compare(const char *url1, const char *url2,
+		bool nofrag, bool *result)
 {
 	url_func_result status;
 	struct url_components c1, c2;
@@ -905,7 +1015,7 @@ url_func_result url_compare(const char *url1, const char *url2, bool *result)
 			((c1.path && c2.path) || (!c1.path && !c2.path)) &&
 			((c1.query && c2.query) ||
 					(!c1.query && !c2.query)) &&
-			((c1.fragment && c2.fragment) ||
+			(nofrag || (c1.fragment && c2.fragment) ||
 					(!c1.fragment && !c2.fragment))) {
 
 		if (c1.scheme)
@@ -921,7 +1031,7 @@ url_func_result url_compare(const char *url1, const char *url2, bool *result)
 		if (c1.query)
 			res &= strcmp(c1.query, c2.query) == 0;
 
-		if (c1.fragment)
+		if (!nofrag && c1.fragment)
 			res &= strcmp(c1.fragment, c2.fragment) == 0;
 	} else {
 		/* Can't match */
@@ -979,10 +1089,12 @@ url_func_result url_get_components(const char *url,
 	if (isalpha(*scheme)) {
 		for (scheme = url + 1;
 				((*scheme != ':') && (*scheme != '\0'));
-				*scheme++)
+				scheme++) {
 			if (!isalnum(*scheme) && (*scheme != '+') &&
 					(*scheme != '-') && (*scheme != '.'))
 				break;
+		}
+
 		if (*scheme == ':') {
 			memcpy(storage_end, url, scheme - url);
 			storage_end[scheme - url] = '\0';
